/******************************************************************************
*                                                                             *
*    Accelerator.cpp                        Copyright(c) 2007-2009 itow,y.    *
*                                                                             *
******************************************************************************/

/*
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <windows.h>
#include <commctrl.h>
#include <tchar.h>
#include "LafPlayer.h"
#include "Settings.h"
#include "Util.h"
#include "Help.h"
#include "resource.h"


#define MAKE_ACCEL_PARAM(i,mod,key) (((i)<<16)|((mod)<<8)|(key))
#define GET_ACCEL_INDEX(param) ((param)>>16)
#define GET_ACCEL_MOD(param) ((BYTE)(((param)>>8)&0xFF))
#define GET_ACCEL_KEY(param) ((BYTE)((param)&0xFF))


static const struct {
	unsigned int KeyCode;
	LPCTSTR pText;
} AccelKeyList[] = {
	{VK_BACK,		TEXT("BS")},
	{VK_TAB,		TEXT("Tab")},
	{VK_RETURN,		TEXT("Enter")},
	{VK_ESCAPE,		TEXT("Esc")},
	{VK_SPACE,		TEXT("Space")},
	{VK_PRIOR,		TEXT("PgUp")},
	{VK_NEXT,		TEXT("PgDown")},
	{VK_END,		TEXT("End")},
	{VK_HOME,		TEXT("Home")},
	{VK_LEFT,		TEXT("")},
	{VK_UP,			TEXT("")},
	{VK_RIGHT,		TEXT("")},
	{VK_DOWN,		TEXT("")},
	{VK_INSERT,		TEXT("Ins")},
	{VK_DELETE,		TEXT("Del")},
	{'0',			TEXT("0")},
	{'1',			TEXT("1")},
	{'2',			TEXT("2")},
	{'3',			TEXT("3")},
	{'4',			TEXT("4")},
	{'5',			TEXT("5")},
	{'6',			TEXT("6")},
	{'7',			TEXT("7")},
	{'8',			TEXT("8")},
	{'9',			TEXT("9")},
	{'A',			TEXT("A")},
	{'B',			TEXT("B")},
	{'C',			TEXT("C")},
	{'D',			TEXT("D")},
	{'E',			TEXT("E")},
	{'F',			TEXT("F")},
	{'G',			TEXT("G")},
	{'H',			TEXT("H")},
	{'I',			TEXT("I")},
	{'J',			TEXT("J")},
	{'K',			TEXT("K")},
	{'L',			TEXT("L")},
	{'M',			TEXT("M")},
	{'N',			TEXT("N")},
	{'O',			TEXT("O")},
	{'P',			TEXT("P")},
	{'Q',			TEXT("Q")},
	{'R',			TEXT("R")},
	{'S',			TEXT("S")},
	{'T',			TEXT("T")},
	{'U',			TEXT("U")},
	{'V',			TEXT("V")},
	{'W',			TEXT("W")},
	{'X',			TEXT("X")},
	{'Y',			TEXT("Y")},
	{'Z',			TEXT("Z")},
	{VK_NUMPAD0,	TEXT("Num0")},
	{VK_NUMPAD1,	TEXT("Num1")},
	{VK_NUMPAD2,	TEXT("Num2")},
	{VK_NUMPAD3,	TEXT("Num3")},
	{VK_NUMPAD4,	TEXT("Num4")},
	{VK_NUMPAD5,	TEXT("Num5")},
	{VK_NUMPAD6,	TEXT("Num6")},
	{VK_NUMPAD7,	TEXT("Num7")},
	{VK_NUMPAD8,	TEXT("Num8")},
	{VK_NUMPAD9,	TEXT("Num9")},
	{VK_MULTIPLY,	TEXT("Num*")},
	{VK_ADD,		TEXT("Num+")},
	{VK_SUBTRACT,	TEXT("Num-")},
	{VK_DECIMAL,	TEXT("Num.")},
	{VK_DIVIDE,		TEXT("Num/")},
	{VK_F1,			TEXT("F1")},
	{VK_F2,			TEXT("F2")},
	{VK_F3,			TEXT("F3")},
	{VK_F4,			TEXT("F4")},
	{VK_F5,			TEXT("F5")},
	{VK_F6,			TEXT("F6")},
	{VK_F7,			TEXT("F7")},
	{VK_F8,			TEXT("F8")},
	{VK_F9,			TEXT("F9")},
	{VK_F10,		TEXT("F10")},
	{VK_F11,		TEXT("F11")},
	{VK_F12,		TEXT("F12")},
};

static const struct {
	WORD Command;
	BYTE Modifiers;
	BYTE Key;
} DefaultAccelList[] = {
	{CM_OPEN,			MOD_CONTROL,	'O'},
	{CM_CLOSE,			MOD_CONTROL,	'W'},
	{CM_PREVFILE,		MOD_ALT,		VK_LEFT},
	{CM_NEXTFILE,		MOD_ALT,		VK_RIGHT},
	{CM_REFRESH,		0,				VK_F5},
	{CM_PLAY_PAUSE,		0,				VK_SPACE},
	{CM_STOP,			0,				VK_DECIMAL},
	{CM_BACKWARD,		MOD_CONTROL,	VK_LEFT},
	{CM_FORWARD,		MOD_CONTROL,	VK_RIGHT},
	{CM_HOME,			MOD_CONTROL,	VK_HOME},
	{CM_END,			MOD_CONTROL,	VK_END},
	{CM_REPEAT,			MOD_CONTROL,	'R'},
	{CM_VOLUMEUP,		MOD_CONTROL,	VK_UP},
	{CM_VOLUMEDOWN,		MOD_CONTROL,	VK_DOWN},
	{CM_MUTE,			MOD_CONTROL,	'M'},
	{CM_ZOOMIN,			MOD_CONTROL,	VK_PRIOR},
	{CM_ZOOMOUT,		MOD_CONTROL,	VK_NEXT},
	{CM_ZOOM50,			MOD_CONTROL,	'H'},
	{CM_ZOOM100,		MOD_CONTROL,	'R'},
	{CM_ZOOM200,		MOD_CONTROL,	'D'},
	{CM_FULLSCREEN,		MOD_ALT,		VK_RETURN},
	{CM_ALWAYSONTOP,	MOD_CONTROL,	'T'},
	{CM_PLAYLIST,		MOD_CONTROL,	'P'},
	{CM_HELP,			0,				VK_F1},
};

#define MAX_LEVELS 3
#define COMMAND_FIRST 100


extern int OptionsStartPage;


enum {
	PARENT_FILE,
	PARENT_PLAY,
	PARENT_VIEW,
	PARENT_HELP,
	PARENT_FILE_SORT,
	PARENT_PLAY_VOLUME,
	PARENT_VIEW_ZOOM,
	PARENT_VIEW_ASPECT,
	PARENT_VIEW_PRESET
};

static struct {
	LPCTSTR pText;
	WORD Command;
	WORD Level;
	BYTE Modifiers;
	BYTE Key;
} AcceleratorList[] = {
	{NULL,						PARENT_FILE,				0},
	{TEXT("Open"),				CM_OPEN,					1},
	{TEXT("Close"),				CM_CLOSE,					1},
	{TEXT("PrevFile"),			CM_PREVFILE,				1},
	{TEXT("NextFile"),			CM_NEXTFILE,				1},
	{TEXT("FirstFile"),			CM_FIRSTFILE,				1},
	{TEXT("LastFile"),			CM_LASTFILE,				1},
	{NULL,						PARENT_FILE_SORT,			1},
	{TEXT("SortName"),			CM_SORT_NAME,				2},
	{TEXT("SortNameNumber"),	CM_SORT_NAME_NUMBER,		2},
	{TEXT("SortExtension"),		CM_SORT_EXTENSION,			2},
	{TEXT("SortType"),			CM_SORT_TYPE,				2},
	{TEXT("SortSize"),			CM_SORT_SIZE,				2},
	{TEXT("SortLastWriteTime"),	CM_SORT_LASTWRITETIME,		2},
	{TEXT("SortDescending"),	CM_SORTDESCENDING,			2},
	{TEXT("Refresh"),			CM_REFRESH,					1},
	{TEXT("AddToPlaylist"),		CM_ADDTOPLAYLIST,			1},
	{TEXT("MakePlaylist"),		CM_FOLDERPLAYLIST,			1},
	{TEXT("Exit"),				CM_EXIT,					1},
	{NULL,						PARENT_PLAY,				0},
	{TEXT("PlayPause"),			CM_PLAY_PAUSE,				1},
	{TEXT("Stop"),				CM_STOP,					1},
	{TEXT("PrevFrame"),			CM_PREVFRAME,				1},
	{TEXT("NextFrame"),			CM_NEXTFRAME,				1},
	{TEXT("Backward"),			CM_BACKWARD,				1},
	{TEXT("Forward"),			CM_FORWARD,					1},
	{TEXT("Home"),				CM_HOME,					1},
	{TEXT("End"),				CM_END,						1},
	{TEXT("Repeat"),			CM_REPEAT,					1},
	{NULL,						PARENT_PLAY_VOLUME,			1},
	{TEXT("VolumeUp"),			CM_VOLUMEUP,				2},
	{TEXT("VolumeDown"),		CM_VOLUMEDOWN,				2},
	{TEXT("Mute"),				CM_MUTE,					2},
	{TEXT("RepeatPlaylist"),	CM_REPEATPLAYLIST,			1},
	{TEXT("SequentialPlay"),	CM_SEQUENTIALPLAY,			1},
	{NULL,						PARENT_VIEW,				0},
	{TEXT("ZoomIn"),			CM_ZOOMIN,					1},
	{TEXT("ZoomOut"),			CM_ZOOMOUT,					1},
	{NULL,						PARENT_VIEW_ZOOM,			1},
	{TEXT("Zoom50"),			CM_ZOOM50,					2},
	{TEXT("Zoom75"),			CM_ZOOM75,					2},
	{TEXT("Zoom100"),			CM_ZOOM100,					2},
	{TEXT("Zoom150"),			CM_ZOOM150,					2},
	{TEXT("Zoom200"),			CM_ZOOM200,					2},
	{TEXT("Zoom300"),			CM_ZOOM300,					2},
	{NULL,						PARENT_VIEW_ASPECT,			1},
	{TEXT("AspectDefault"),		CM_ASPECT_DEFAULT,			2},
	{TEXT("Aspect4x3"),			CM_ASPECT_4X3,				2},
	{TEXT("Aspect16x9"),		CM_ASPECT_16X9,				2},
	{TEXT("KeepZoom"),			CM_KEEPZOOMRATE,			1},
	{TEXT("FitToWindow"),		CM_FITTOWINDOW,				1},
	{TEXT("FullScreen"),		CM_FULLSCREEN,				1},
	{TEXT("AlwaysOnTop"),		CM_ALWAYSONTOP,				1},
	{TEXT("Playlist"),			CM_PLAYLIST,				1},
	{TEXT("MenuBar"),			CM_MENUBAR,					1},
	{TEXT("Toolbar"),			CM_TOOLBAR,					1},
	{TEXT("StatusBar"),			CM_STATUSBAR,				1},
	{TEXT("PlayBar"),			CM_PLAYBAR,					1},
	{TEXT("SeekBar"),			CM_SEEKBAR,					1},
	{NULL,						PARENT_VIEW_PRESET,			1},
	{TEXT("Preset1"),			CM_PRESET_FIRST,			2},
	{TEXT("Preset2"),			CM_PRESET_FIRST+1,			2},
	{TEXT("Preset3"),			CM_PRESET_FIRST+2,			2},
	{TEXT("Options"),			CM_OPTIONS,					1},
	{NULL,						PARENT_HELP,				0},
	{TEXT("Help"),				CM_HELP,					1},
	{TEXT("About"),				CM_ABOUT,					1},
};

static bool fDefaultAccelerator=true;
static HACCEL hAccel=NULL;




static void SetMenuAccelText(HMENU hmenu,int Index)
{
	TCHAR szText[128],*p;

	GetMenuString(hmenu,AcceleratorList[Index].Command,szText,numberof(szText),
																MF_BYCOMMAND);
	p=szText;
	while (*p!='\t') {
		if (*p=='\0') {
			if (AcceleratorList[Index].Key==0)
				return;
			break;
		}
		p++;
	}
	if (AcceleratorList[Index].Key!=0) {
		*p='\t';
		*(p+1)='\0';
		if ((AcceleratorList[Index].Modifiers&MOD_SHIFT)!=0)
			lstrcat(szText,TEXT("Shift+"));
		if ((AcceleratorList[Index].Modifiers&MOD_CONTROL)!=0)
			lstrcat(szText,TEXT("Ctrl+"));
		if ((AcceleratorList[Index].Modifiers&MOD_ALT)!=0)
			lstrcat(szText,TEXT("Alt+"));
		for (int i=0;i<numberof(AccelKeyList);i++) {
			if ((unsigned int)AcceleratorList[Index].Key==
													AccelKeyList[i].KeyCode) {
				lstrcat(szText,AccelKeyList[i].pText);
				break;
			}
		}
	} else
		*p='\0';
	ModifyMenu(hmenu,AcceleratorList[Index].Command,
		MF_BYCOMMAND | MFT_STRING |
			GetMenuState(hmenu,AcceleratorList[Index].Command,MF_BYCOMMAND),
									AcceleratorList[Index].Command,szText);
}


static int CountAccel()
{
	int NumAccel,i;

	NumAccel=0;
	for (i=0;i<numberof(AcceleratorList);i++)
		if (AcceleratorList[i].Key!=0)
			NumAccel++;
	return NumAccel;
}


static BOOL CreateAccel(HACCEL *phAccel)
{
	int NumAccel,i,j;
	LPACCEL paccl;

	NumAccel=CountAccel();
	if (NumAccel==0) {
		*phAccel=NULL;
		return TRUE;
	}
	paccl=new ACCEL[NumAccel];
	j=0;
	for (i=0;j<NumAccel;i++) {
		if (AcceleratorList[i].Key!=0) {
			paccl[j].fVirt=FVIRTKEY;
			if ((AcceleratorList[i].Modifiers&MOD_SHIFT)!=0)
				paccl[j].fVirt|=FSHIFT;
			if ((AcceleratorList[i].Modifiers&MOD_CONTROL)!=0)
				paccl[j].fVirt|=FCONTROL;
			if ((AcceleratorList[i].Modifiers&MOD_ALT)!=0)
				paccl[j].fVirt|=FALT;
			paccl[j].key=AcceleratorList[i].Key;
			paccl[j].cmd=AcceleratorList[i].Command;
			j++;
		}
	}
	*phAccel=CreateAcceleratorTable(paccl,NumAccel);
	delete [] paccl;
	return *phAccel!=NULL;
}


BOOL InitAccel(HMENU hmenu)
{
	Settings *pSettings;
	int i,j;

	for (i=0;i<numberof(DefaultAccelList);i++) {
		for (j=0;AcceleratorList[j].Command!=DefaultAccelList[i].Command;j++);
		AcceleratorList[j].Modifiers=DefaultAccelList[i].Modifiers;
		AcceleratorList[j].Key=DefaultAccelList[i].Key;
	}
	pSettings=OpenProgramSettings(TEXT("Accelerators"),Settings::OPEN_READ);
	if (pSettings!=NULL) {
		int NumAccel;

		if (pSettings->ReadInt(TEXT("AccelCount"),&NumAccel) && NumAccel>=0) {
			fDefaultAccelerator=false;
			for (i=0;i<numberof(AcceleratorList);i++) {
				int Length;
				int Key[2];

				Length=2;
				if (pSettings->ReadIntArray(AcceleratorList[i].pText,Key,
														&Length) && Length>0) {
					AcceleratorList[i].Modifiers=
						Length==2?Key[1]&(MOD_ALT | MOD_CONTROL | MOD_SHIFT):0;
					AcceleratorList[i].Key=Key[0];
				} else {
					AcceleratorList[i].Modifiers=0;
					AcceleratorList[i].Key=0;
				}
			}
		}
		delete pSettings;
	}
	if (!CreateAccel(&hAccel))
		return FALSE;
	for (i=0;i<numberof(AcceleratorList);i++)
		if (AcceleratorList[i].Key!=0)
			SetMenuAccelText(hmenu,i);
	return TRUE;
}


void FreeAccel()
{
	Settings *pSettings;

	if (hAccel!=NULL)
		DestroyAcceleratorTable(hAccel);
	pSettings=OpenProgramSettings(TEXT("Accelerators"),Settings::OPEN_WRITE);
	if (pSettings!=NULL) {
		pSettings->Clear();
		if (!fDefaultAccelerator) {
			int NumAccel,i,j;
			int Key[2];

			NumAccel=CountAccel();
			pSettings->WriteInt(TEXT("AccelCount"),NumAccel);
			j=0;
			for (i=0;j<NumAccel;i++) {
				if (AcceleratorList[i].Key!=0) {
					Key[0]=AcceleratorList[i].Key;
					Key[1]=AcceleratorList[i].Modifiers;
					pSettings->WriteIntArray(AcceleratorList[i].pText,Key,2);
					j++;
				}
			}
		} else
			pSettings->WriteInt(TEXT("AccelCount"),-1);
		delete pSettings;
	}
}


BOOL TranslateAccel(HWND hwnd,LPMSG pmsg)
{
	if (hAccel!=NULL)
		return TranslateAccelerator(hwnd,hAccel,pmsg);
	return FALSE;
}


void SetMenuAccel(HMENU hmenu)
{
	int Count,i,j;
	unsigned int State;
	int Command;

	Count=GetMenuItemCount(hmenu);
	for (i=0;i<Count;i++) {
		State=GetMenuState(hmenu,i,MF_BYPOSITION);
		if ((State&MF_POPUP)!=0) {
			SetMenuAccel(GetSubMenu(hmenu,i));
		} else if ((State&MF_SEPARATOR)==0) {
			Command=(INT)GetMenuItemID(hmenu,i);
			for (j=0;j<numberof(AcceleratorList);j++)
				if ((int)AcceleratorList[j].Command==Command)
					SetMenuAccelText(hmenu,j);
		}
	}
}


static void LoadAccelSub(Settings *pSettings,HWND hwndTree,HTREEITEM htiParent)
{
	TV_ITEM tvi;
	int i;

	tvi.mask=TVIF_PARAM;
	tvi.hItem=TreeView_GetChild(hwndTree,htiParent);
	while (tvi.hItem!=NULL) {
		TreeView_GetItem(hwndTree,&tvi);
		i=GET_ACCEL_INDEX(tvi.lParam);
		if (AcceleratorList[i].Command>=COMMAND_FIRST) {
			int Key[2];
			int Length;
			LPARAM lNewParam;

			Length=2;
			if (pSettings->ReadIntArray(AcceleratorList[i].pText,Key,&Length)
																&& Length>0)
				lNewParam=MAKE_ACCEL_PARAM(i,
						Length==2?Key[1]&(MOD_ALT | MOD_CONTROL | MOD_SHIFT):0,
																	Key[0]);
			else
				lNewParam=MAKE_ACCEL_PARAM(i,0,0);
			if (lNewParam!=tvi.lParam) {
				tvi.lParam=lNewParam;
				TreeView_SetItem(hwndTree,&tvi);
			}
		} else
			LoadAccelSub(pSettings,hwndTree,tvi.hItem);
		tvi.hItem=TreeView_GetNextSibling(hwndTree,tvi.hItem);
	}
}


static BOOL LoadAccel(HWND hwndTree,LPCTSTR pFileName)
{
	Settings AccelSettings;

	if (!AccelSettings.Open(pFileName,TEXT("Accelerators"),
														Settings::OPEN_READ))
		return FALSE;
	LoadAccelSub(&AccelSettings,hwndTree,TVI_ROOT);
	return TRUE;
}


static BOOL SaveAccelSub(HANDLE hFile,HWND hwndTree,HTREEITEM htiParent)
{
	TV_ITEM tvi;
	int i;
	BYTE Key;

	tvi.mask=TVIF_PARAM;
	tvi.hItem=TreeView_GetChild(hwndTree,htiParent);
	while (tvi.hItem!=NULL) {
		TreeView_GetItem(hwndTree,&tvi);
		i=GET_ACCEL_INDEX(tvi.lParam);
		if (AcceleratorList[i].Command>=COMMAND_FIRST) {
			Key=GET_ACCEL_KEY(tvi.lParam);
			if (Key!=0) {
				char Buff[64];
				DWORD Size,Write;

				Size=wsprintfA(Buff,"%s=%d,%d\r\n",AcceleratorList[i].pText,
												Key,GET_ACCEL_MOD(tvi.lParam));
				if (!WriteFile(hFile,Buff,Size,&Write,NULL) || Write!=Size)
					return FALSE;
			}
		} else {
			if (!SaveAccelSub(hFile,hwndTree,tvi.hItem))
				return FALSE;
		}
		tvi.hItem=TreeView_GetNextSibling(hwndTree,tvi.hItem);
	}
	return TRUE;
}


static BOOL SaveAccel(HWND hwndTree,LPCTSTR pFileName)
{
	HANDLE hFile;
	DWORD Size,Write;
	BOOL fResult;

	hFile=CreateFile(pFileName,GENERIC_WRITE,FILE_SHARE_READ,NULL,
									CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
	if (hFile==INVALID_HANDLE_VALUE)
		return FALSE;
	Size=sizeof("[Accelerators]\r\n")-1;
	if (!WriteFile(hFile,"[Accelerators]\r\n",Size,&Write,NULL)
															|| Write!=Size) {
		CloseHandle(hFile);
		return FALSE;
	}
	fResult=SaveAccelSub(hFile,hwndTree,TVI_ROOT);
	CloseHandle(hFile);
	return fResult;
}


static int CheckAccelKey(HWND hwndTree,HTREEITEM htiParent,BYTE bMod,BYTE bKey)
{
	HTREEITEM htiSel,hti;
	TV_ITEM tvi;
	int i;

	htiSel=TreeView_GetSelection(hwndTree);
	hti=TreeView_GetChild(hwndTree,htiParent);
	tvi.mask=TVIF_PARAM;
	while (hti!=NULL) {
		if (hti!=htiSel) {
			tvi.hItem=hti;
			TreeView_GetItem(hwndTree,&tvi);
			i=GET_ACCEL_INDEX(tvi.lParam);
			if (AcceleratorList[i].Command>=COMMAND_FIRST) {
				if (GET_ACCEL_MOD(tvi.lParam)==bMod
											&& GET_ACCEL_KEY(tvi.lParam)==bKey)
					return i;
			} else {
				i=CheckAccelKey(hwndTree,hti,bMod,bKey);
				if (i>=0)
					return i;
			}
		}
		hti=TreeView_GetNextSibling(hwndTree,hti);
	}
	return -1;
}


static void ClearAllAccelKeys(HWND hwndTree,HTREEITEM htiParent)
{
	TV_ITEM tvi;
	int i;

	tvi.mask=TVIF_PARAM;
	tvi.hItem=TreeView_GetChild(hwndTree,htiParent);
	while (tvi.hItem!=NULL) {
		TreeView_GetItem(hwndTree,&tvi);
		i=GET_ACCEL_INDEX(tvi.lParam);
		if (AcceleratorList[i].Command>=COMMAND_FIRST) {
			if (GET_ACCEL_KEY(tvi.lParam)!=0) {
				tvi.lParam=MAKE_ACCEL_PARAM(i,0,0);
				TreeView_SetItem(hwndTree,&tvi);
			}
		} else
			ClearAllAccelKeys(hwndTree,tvi.hItem);
		tvi.hItem=TreeView_GetNextSibling(hwndTree,tvi.hItem);
	}
}


static void SetDefaultAccelKeys(HWND hwndTree,HTREEITEM htiParent)
{
	TV_ITEM tvi;
	int i,j;

	tvi.mask=TVIF_PARAM;
	tvi.hItem=TreeView_GetChild(hwndTree,htiParent);
	while (tvi.hItem!=NULL) {
		TreeView_GetItem(hwndTree,&tvi);
		i=GET_ACCEL_INDEX(tvi.lParam);
		if (AcceleratorList[i].Command>=COMMAND_FIRST) {
			for (j=0;j<numberof(DefaultAccelList);j++) {
				if (DefaultAccelList[j].Command==AcceleratorList[i].Command) {
					tvi.lParam=MAKE_ACCEL_PARAM(i,
												DefaultAccelList[j].Modifiers,
												DefaultAccelList[j].Key);
					TreeView_SetItem(hwndTree,&tvi);
					break;
				}
			}
		} else
			SetDefaultAccelKeys(hwndTree,tvi.hItem);
		tvi.hItem=TreeView_GetNextSibling(hwndTree,tvi.hItem);
	}
}


static BOOL ApplyAccelKeys(HWND hwndTree,HTREEITEM htiParent,HMENU hmenu)
{
	BOOL fModified;
	TV_ITEM tvi;
	int i;
	BYTE bMod,bKey;

	fModified=FALSE;
	tvi.mask=TVIF_PARAM;
	tvi.hItem=TreeView_GetChild(hwndTree,htiParent);
	while (tvi.hItem!=NULL) {
		TreeView_GetItem(hwndTree,&tvi);
		i=GET_ACCEL_INDEX(tvi.lParam);
		if (AcceleratorList[i].Command>=COMMAND_FIRST) {
			bMod=GET_ACCEL_MOD(tvi.lParam);
			bKey=GET_ACCEL_KEY(tvi.lParam);
			if (bKey!=AcceleratorList[i].Key
									|| bMod!=AcceleratorList[i].Modifiers) {
				AcceleratorList[i].Modifiers=bMod;
				AcceleratorList[i].Key=bKey;
				SetMenuAccelText(hmenu,i);
				fModified=TRUE;
			}
		} else {
			if (ApplyAccelKeys(hwndTree,tvi.hItem,hmenu))
				fModified=TRUE;
		}
		tvi.hItem=TreeView_GetNextSibling(hwndTree,tvi.hItem);
	}
	return fModified;
}


static void SetAccelDlgItems(HWND hDlg)
{
	HWND hwndTree=GetDlgItem(hDlg,IDC_ACCEL_TREE);
	TV_ITEM tvi;
	int nIndex;
	BYTE bKey,bMod;

	tvi.mask=TVIF_PARAM;
	tvi.hItem=TreeView_GetSelection(hwndTree);
	TreeView_GetItem(hwndTree,&tvi);
	nIndex=GET_ACCEL_INDEX(tvi.lParam);
	EnableDlgItems(hDlg,IDC_ACCEL_KEY,IDC_ACCEL_SET,
							AcceleratorList[nIndex].Command>=COMMAND_FIRST);
	bKey=GET_ACCEL_KEY(tvi.lParam);
	if (bKey!=0) {
		int i;

		for (i=0;i<numberof(AccelKeyList);i++) {
			if (AccelKeyList[i].KeyCode==(unsigned int)bKey) {
				SendDlgItemMessage(hDlg,IDC_ACCEL_KEY,CB_SETCURSEL,i+1,0);
				break;
			}
		}
		bMod=GET_ACCEL_MOD(tvi.lParam);
		CheckDlgButton(hDlg,IDC_ACCEL_SHIFT,
								(bMod&MOD_SHIFT)!=0?BST_CHECKED:BST_UNCHECKED);
		CheckDlgButton(hDlg,IDC_ACCEL_CONTROL,
							(bMod&MOD_CONTROL)!=0?BST_CHECKED:BST_UNCHECKED);
		CheckDlgButton(hDlg,IDC_ACCEL_ALT,
								(bMod&MOD_ALT)!=0?BST_CHECKED:BST_UNCHECKED);
	} else {
		SendDlgItemMessage(hDlg,IDC_ACCEL_KEY,CB_SETCURSEL,0,0);
		CheckDlgButton(hDlg,IDC_ACCEL_SHIFT,BST_UNCHECKED);
		CheckDlgButton(hDlg,IDC_ACCEL_CONTROL,BST_UNCHECKED);
		CheckDlgButton(hDlg,IDC_ACCEL_ALT,BST_UNCHECKED);
	}
	EnableDlgItem(hDlg,IDC_ACCEL_CLEAR,bKey!=0);
}


BOOL CALLBACK AccelOptionsDlgProc(HWND hDlg,UINT uMsg,WPARAM wParam,
																LPARAM lParam)
{
	static bool fDefault;
	static HWND hwndTree;

	switch (uMsg) {
	case WM_INITDIALOG:
		{
			HINSTANCE hinst=GetResourceInstance();
			TV_INSERTSTRUCT tvis;
			TCHAR szText[64];
			HTREEITEM htiParent[MAX_LEVELS],hti;
			int i;
			BOOL fParent;

			fDefault=fDefaultAccelerator;
			hwndTree=GetDlgItem(hDlg,IDC_ACCEL_TREE);
			htiParent[0]=TVI_ROOT;
			tvis.hInsertAfter=TVI_LAST;
			tvis.UNION_MEMBER(u) item.pszText=szText;
			for (i=0;i<numberof(AcceleratorList);i++) {
				tvis.hParent=htiParent[AcceleratorList[i].Level];
				tvis.UNION_MEMBER(u) item.mask=TVIF_TEXT | TVIF_PARAM;
				tvis.UNION_MEMBER(u) item.lParam=MAKE_ACCEL_PARAM(i,
												AcceleratorList[i].Modifiers,
												AcceleratorList[i].Key);
				fParent=i+1<numberof(AcceleratorList)
						&& AcceleratorList[i+1].Level>AcceleratorList[i].Level;
				LoadString(hinst,
							fParent?IDS_MENUBASE+AcceleratorList[i].Command:
									IDS_SHORTCMBASE+AcceleratorList[i].Command,
													szText,numberof(szText));
				hti=TreeView_InsertItem(hwndTree,&tvis);
				if (fParent)
					htiParent[AcceleratorList[i].Level+1]=hti;
			}
			//LoadString(hinst,IDS_NONE,szText,numberof(szText));
			SendDlgItemMessage(hDlg,IDC_ACCEL_KEY,CB_ADDSTRING,0,
											(LPARAM)TEXT("Ȃ")/*szText*/);
			for (i=0;i<numberof(AccelKeyList);i++)
				SendDlgItemMessage(hDlg,IDC_ACCEL_KEY,CB_ADDSTRING,0,
												(LPARAM)AccelKeyList[i].pText);
		}
		return TRUE;

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDC_ACCEL_SET:
			{
				TV_ITEM tvi;
				int nKey;

				tvi.mask=TVIF_PARAM;
				tvi.hItem=TreeView_GetSelection(hwndTree);
				TreeView_GetItem(hwndTree,&tvi);
				nKey=SendDlgItemMessage(hDlg,IDC_ACCEL_KEY,CB_GETCURSEL,0,0);
				if (nKey>0) {
					BYTE bMod;
					int i;

					bMod=0;
					if (IsDlgButtonChecked(hDlg,IDC_ACCEL_SHIFT)==BST_CHECKED)
						bMod|=MOD_SHIFT;
					if (IsDlgButtonChecked(hDlg,IDC_ACCEL_CONTROL)==
																BST_CHECKED)
						bMod|=MOD_CONTROL;
					if (IsDlgButtonChecked(hDlg,IDC_ACCEL_ALT)==BST_CHECKED)
						bMod|=MOD_ALT;
					i=CheckAccelKey(hwndTree,TVI_ROOT,bMod,
												AccelKeyList[nKey-1].KeyCode);
					if (i>=0) {
						TCHAR szText[64],szMessage[128];

						LoadString(GetResourceInstance(),
								IDS_SHORTCMBASE+AcceleratorList[i].Command,
													szText,numberof(szText));
						wsprintf(szMessage,
							TEXT(" [%s] Ɋ蓖ĂĂ܂B"),szText);
						MessageBox(hDlg,szMessage,NULL,
												MB_OK | MB_ICONINFORMATION);
						return TRUE;
					}
					tvi.lParam=MAKE_ACCEL_PARAM(GET_ACCEL_INDEX(tvi.lParam),
											bMod,AccelKeyList[nKey-1].KeyCode);
				} else
					tvi.lParam=
							MAKE_ACCEL_PARAM(GET_ACCEL_INDEX(tvi.lParam),0,0);
				TreeView_SetItem(hwndTree,&tvi);
				EnableDlgItem(hDlg,IDC_ACCEL_CLEAR,nKey>0);
				fDefault=false;
			}
			return TRUE;
		case IDC_ACCEL_CLEAR:
			{
				TV_ITEM tvi;

				tvi.mask=TVIF_PARAM;
				tvi.hItem=TreeView_GetSelection(hwndTree);
				TreeView_GetItem(hwndTree,&tvi);
				tvi.lParam=MAKE_ACCEL_PARAM(GET_ACCEL_INDEX(tvi.lParam),0,0);
				TreeView_SetItem(hwndTree,&tvi);
				SetAccelDlgItems(hDlg);
				fDefault=false;
			}
			return TRUE;
		case IDC_ACCEL_LOAD:
			{
				TCHAR szFileName[MAX_PATH],szFilter[128],szInitDir[MAX_PATH];
				int Length;
				OPENFILENAME ofn;

				GetModuleFileName(NULL,szFileName,MAX_PATH);
				GetFileDirectory(szInitDir,szFileName);
				szFileName[0]='\0';
				Length=LoadString(GetResourceInstance(),IDS_KEYFILEFILTER,
												szFilter,numberof(szFilter)-1);
				szFilter[Length+1]='\0';	// for MS\[XRpC
				InitOpenFileName(&ofn);
				ofn.hwndOwner=hDlg;
				ofn.lpstrFilter=szFilter;
				ofn.nFilterIndex=1;
				ofn.lpstrFile=szFileName;
				ofn.nMaxFile=MAX_PATH;
				ofn.lpstrInitialDir=szInitDir;
				ofn.Flags=OFN_HIDEREADONLY | OFN_FILEMUSTEXIST | OFN_EXPLORER;
				if (GetOpenFileName(&ofn)) {
					if (!LoadAccel(hwndTree,szFileName)) {
						ResMessageBox(hDlg,IDS_FAIL_LOADFILE,0,
												MB_OK | MB_ICONEXCLAMATION);
						return TRUE;
					}
					SetAccelDlgItems(hDlg);
					fDefault=false;
				}
			}
			return TRUE;
		case IDC_ACCEL_SAVE:
			{
				TCHAR szFileName[MAX_PATH],szFilter[128],szInitDir[MAX_PATH];
				int Length;
				OPENFILENAME ofn;

				GetModuleFileName(NULL,szFileName,MAX_PATH);
				GetFileDirectory(szInitDir,szFileName);
				szFileName[0]='\0';
				Length=LoadString(GetResourceInstance(),IDS_KEYFILEFILTER,
												szFilter,numberof(szFilter)-1);
				szFilter[Length+1]='\0';	// for MS\[XRpC
				InitOpenFileName(&ofn);
				ofn.hwndOwner=hDlg;
				ofn.lpstrFilter=szFilter;
				ofn.nFilterIndex=1;
				ofn.lpstrFile=szFileName;
				ofn.nMaxFile=MAX_PATH;
				ofn.lpstrInitialDir=szInitDir;
				ofn.Flags=OFN_HIDEREADONLY | OFN_EXPLORER |
														OFN_OVERWRITEPROMPT;
				ofn.lpstrDefExt=TEXT("lpkey");
				if (GetSaveFileName(&ofn)) {
					if (!SaveAccel(hwndTree,szFileName)) {
						ResMessageBox(hDlg,IDS_FAIL_SAVEFILE,0,
												MB_OK | MB_ICONEXCLAMATION);
						return TRUE;
					}
				}
			}
			return TRUE;
		case IDC_ACCEL_DEFAULT:
			ClearAllAccelKeys(hwndTree,TVI_ROOT);
			SetDefaultAccelKeys(hwndTree,TVI_ROOT);
			SetAccelDlgItems(hDlg);
			fDefault=true;
			return TRUE;
		}
		return TRUE;

	case WM_NOTIFY:
		switch (((LPNMHDR)lParam)->code) {
		case TVN_SELCHANGED:
			SetAccelDlgItems(hDlg);
			break;
		case PSN_APPLY:
			if (ApplyAccelKeys(hwndTree,TVI_ROOT,
										GetMenu(GetParent(GetParent(hDlg))))) {
				HACCEL hNewAccel;

				if (!CreateAccel(&hNewAccel)) {
					ResMessageBox(hDlg,IDS_FAIL_CREATEACCEL,0,
												MB_OK | MB_ICONEXCLAMATION);
					break;
				}
				if (hAccel!=NULL)
					DestroyAcceleratorTable(hAccel);
				hAccel=hNewAccel;
				fDefaultAccelerator=fDefault;
			}
			break;
		case PSN_SETACTIVE:
			OptionsStartPage=5;
			break;
		case PSN_HELP:
			ShowHelp(IDH_OPTIONS_ACCEL);
			break;
		}
		break;
	}
	return FALSE;
}
