/******************************************************************************
*                                                                             *
*    DirectoryFileList.h                    Copyright(c) 2005-2009 itow,y.    *
*                                                                             *
******************************************************************************/

/*
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef DIRECTORYFILELIST_H
#define DIRECTORYFILELIST_H


class FileInfo {
	LPTSTR m_pszFileName;
	ULONGLONG m_FileSize;
	FILETIME m_LastWriteTime;
	LPTSTR m_pszTypeName;
	HICON m_hIcon;
public:
	FileInfo(LPCTSTR pszFileName,ULONGLONG FileSize,const FILETIME &LastWriteTime);
	~FileInfo();
	LPCTSTR GetFileName() const { return m_pszFileName; }
	ULONGLONG GetFileSize() const { return m_FileSize; }
	const FILETIME &GetLastWriteTime() const { return m_LastWriteTime; }
	LPCTSTR GetTypeName();
	HICON GetIcon();
};

class DirectoryFileList {
	FileInfo **m_ppList;
	int m_ListLength;
	int m_NumFiles;
	TCHAR m_szDirectory[MAX_PATH];
	bool AddFile(const WIN32_FIND_DATA *pwfd);
	typedef int (*CompareFileInfoFunc)(FileInfo *pFile1,FileInfo *pFile2);
	void SortSub(FileInfo **ppFirst,FileInfo **ppLast,
								CompareFileInfoFunc pCompare,bool fDescending);
public:
	DirectoryFileList();
	~DirectoryFileList();
	void Clear();
	bool FindFiles(LPCTSTR pszDirectory,LPCTSTR pszMasks);
	void GetDirectory(LPTSTR pszDirectory);
	LPCTSTR GetDirectory() { return m_szDirectory; }
	bool AddFile(LPCTSTR pszFileName);
	bool GetFile(int Index,LPTSTR pszFileName);
	int GetCount() { return m_NumFiles; }
	int GetFileIndex(LPCTSTR pszFileName);
	enum SortType {
		SORT_NAME,
		SORT_NAME_NUMBER,
		SORT_TYPE,
		SORT_EXTENSION,
		SORT_SIZE,
		SORT_LASTWRITETIME
	};
	void Sort(SortType Type,bool fDescending=false);
	HICON GetFileIcon(int Index);
};


#endif	// ndef DIRECTORYFILELIST_H
