/******************************************************************************
*                                                                             *
*    LafPlayer.cpp                          Copyright(c) 2005-2009 itow,y.    *
*                                                                             *
******************************************************************************/

/*
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <windows.h>
#include <windowsx.h>
#include <shlobj.h>
#include <commctrl.h>
#include <tchar.h>
#include "LafPlayer.h"
#ifdef USE_HTML_HELP
#include <htmlhelp.h>
#endif
#include "Media.h"
#include "Playlist.h"
#include "FileList.h"
#include "DirectoryFileList.h"
#include "StatusBar.h"
#include "Settings.h"
#include "Colors.h"
#include "Theme.h"
#include "Accelerator.h"
#include "History.h"
#include "Util.h"
#include "MP3.h"
#include "Help.h"
#include "resource.h"


#define MAIN_WINDOW_CLASS TEXT("LafPlayer Main Window")
#define TITLE_TEXT TEXT("LafPlayer")

#define THEMES_DIRECTORY TEXT("Themes")

#define LAFPLAYER_MSG_FILEOPEN	WM_USER
// BOOL OnFileOpen(HWND hwnd,ATOM atom,int nNo,int nNumFiles)
#define HANDLE_LAFPLAYER_MSG_FILEOPEN(hwnd,wParam,lParam,fn) \
	(LRESULT)(DWORD)(fn)((hwnd),(ATOM)(wParam),LOWORD(lParam),HIWORD(lParam))
#define FORWARD_LAFPLAYER_MSG_FILEOPEN(hwnd,atom,nNo,nNumFiles,fn) \
	(BOOL)(DWORD)(fn)((hwnd),LAFPLAYER_MSG_FILEOPEN,(WPARAM)(ATOM)(atom), \
													MAKELPARAM(nNo,nNumFiles))

#define IDC_MEDIA		400
#define IDC_TOOLBAR		401
#define IDC_STATUSBAR	402

#define FILE_MENU	0
#define PLAY_MENU	1
#define VIEW_MENU	2
#define HELP_MENU	3

#define FILE_HISTORY_MENU	3
#define DIR_HISTORY_MENU	4
#define FILE_SORT_MENU		11
#define PLAY_VOLUME_MENU	11
#define VIEW_ZOOM_MENU		2
#define VIEW_ASPECT_MENU	3
#define VIEW_PRESET_MENU	19


static HINSTANCE hInst;
static HWND hwndMain;
static HMENU hmenuMain;
static HWND hwndToolbar;
//static HWND hwndStatusBar;
static LPStatusBar *pStatusBar;
static HWND hwndSizeGrip;
static MediaPlayer *pMediaPlayer;
static Playlist *pPlaylist;

#define INI_FILE_NAME TEXT("LafPlayer.ini")
static bool fUseUserProfile=false;
static TCHAR szIniFileName[MAX_PATH];
static TCHAR szHelpFileName[MAX_PATH];
#ifdef USE_HTML_HELP
static HINSTANCE hHtmlHelpLib=NULL;
typedef HWND (WINAPI *HtmlHelpFunc)(HWND hwndCaller,LPCTSTR pszFile,
												UINT uCommand,DWORD dwData);
static HtmlHelpFunc pfnHtmlHelp=NULL;
static DWORD dwHtmlHelpCookie=0;
#endif
static int nWindowLeft=CW_USEDEFAULT;
static int nWindowTop=CW_USEDEFAULT;
static int nWindowWidth=CW_USEDEFAULT;
static int nWindowHeight=CW_USEDEFAULT;
static bool fWindowMaximized=false;
static int nPlaylistLeft=CW_USEDEFAULT;
static int nPlaylistTop=CW_USEDEFAULT;
static int nPlaylistWidth=CW_USEDEFAULT;
static int nPlaylistHeight=CW_USEDEFAULT;
static TCHAR szFileOpenDirectory[MAX_PATH];
static int nOpenFilterIndex=1;
static bool fOpenFromPlaylist=false;
static History FileHistory;
static History DirHistory;
static int FileListMenuCurFile;
static HFONT hfontMenu;
static bool fPlayRepeat=false;
static bool fSequentialPlay=false;
static bool fRepeatPlaylist=false;
static bool fShowPlaylist=false;
static bool fShowMenuBar=true;
static bool fShowToolbar=true;
static bool fShowStatusBar=true;
static bool fShowPlayBar=true;
static bool fShowSeekBar=true;
static bool fKeepZoomRate=false;
#define MIN_ZOOM	1
#define MAX_ZOOM	1000
static int nPrevZoomRate=100;
static const int ZoomRateList[] = {50,75,100,150,200,300};
static const struct {
	int x,y;
} AspectRatioList [] = {{0,0},{4,3},{16,9}};
static MediaType PrevMediaType=MEDIA_UNDEFINED;
static bool fFitToWindow=false;
static bool fAlwaysOnTop=false;
static int DefaultVolume=500;

static DirectoryFileList *pDirFileList=NULL;
static DirectoryFileList::SortType DirSortType=DirectoryFileList::SORT_NAME;
static bool fDirSortDescending=false;

int OptionsStartPage=0;

static bool fKeepSingleTask=true;
static bool fSaveWindowPos=true;
static bool fSaveWindowSize=true;
enum TitleBarFormatType {
	TITLE_FORMAT_PROGRAM_FILENAME,
	TITLE_FORMAT_FILENAME_PROGRAM,
	TITLE_FORMAT_PROGRAM
};
static TitleBarFormatType TitleBarFormat=TITLE_FORMAT_PROGRAM_FILENAME;
enum TitleFilePathType {
	TITLE_FILEPATH_FILENAME,
	TITLE_FILEPATH_FULLPATH,
	TITLE_FILEPATH_DIRECTORY_FILENAME
};
static TitleFilePathType TitleFilePath=TITLE_FILEPATH_FILENAME;
static bool fShowTipHelp=true;
static bool fFindSubDirectory=false;
#define DEFAULT_FILE_MASKS \
	TEXT("*.aif;*.aifc;*.aiff;*.au;*.snd;*.avi;*.mid;*.midi;*.rmi;*.mp2;*.mpa;*.mp3;*.mpg;*.mpe;*.mpeg;*.wav;*.wma;*.wmv")
static TCHAR szFileMasks[MAX_FILE_MASK_LENGTH];//=DEFAULT_FILE_MASKS;
static MediaDeviceType MediaDevice=MEDIA_DEVICE_DIRECTSHOW;

#define MAX_PRESET_NAME 32
#define PRESET_FLAG_MENUBAR		0x00000001
#define PRESET_FLAG_TOOLBAR		0x00000002
#define PRESET_FLAG_STATUSBAR	0x00000004
#define PRESET_FLAG_PLAYBAR		0x00000008
#define PRESET_FLAG_SEEKBAR		0x00000010
struct PresetInfo {
	TCHAR szName[MAX_PRESET_NAME];
	unsigned int Flags;
};
#define NUM_PRESETS 3
static PresetInfo PresetList[NUM_PRESETS] = {
	{TEXT("ŏ"),		0},
	{TEXT("ő"),		PRESET_FLAG_MENUBAR | PRESET_FLAG_TOOLBAR |
			PRESET_FLAG_STATUSBAR | PRESET_FLAG_PLAYBAR | PRESET_FLAG_SEEKBAR},
	{TEXT("Vv"),
			PRESET_FLAG_MENUBAR | PRESET_FLAG_PLAYBAR | PRESET_FLAG_SEEKBAR},
};

#define MAX_FILE_HISTORY 36
static int nMaxFileHistory=10;
static bool fSaveFileHistory=true;
#define MAX_DIRECTORY_HISTORY 36
static int nMaxDirectoryHistory=10;
static bool fSaveDirectoryHistory=true;
static bool fUseDirHistoryMenu=false;

#define MAX_FILE_LIST_MENU_FILES 100
static int MaxFileListMenuFiles=20;

static bool fPlayOnOpen=true;
#define MIN_ZOOM_STEP	1
#define MAX_ZOOM_STEP	100
static int nZoomStep=20;
#define MIN_BACKWARD_SPEED	1000
#define MAX_BACKWARD_SPEED	4000
static int nBackwardSpeed=2000;
#define MIN_FORWARD_SPEED	1500
#define MAX_FORWARD_SPEED	4000
static int nForwardSpeed=2000;
static bool fShowFullScreenSeekBar=false;
static bool fShowFullScreenPlayBar=false;
static bool fFullScreenFitToDisplay=false;
static bool fFullScreenHideCursor=false;

static bool fPlaylistKeepForeground=true;
static bool fPlaylistSingleClickOpen=false;
static Playlist::FileNameType PlaylistFileNameType=Playlist::FILENAME_FILENAME;
static bool fPlaylistRestoreList=false;
#define MAX_PLAYLIST_HISTORY 36
static int MaxPlaylistHistory=10;

static TCHAR szCurThemeFileName[MAX_PATH];
enum ColorSchemeType {
	COLOR_SCHEME_BACKGROUND,
	COLOR_SCHEME_FULLSCREENBG,
	COLOR_SCHEME_STATUSTEXT,
	COLOR_SCHEME_STATUSBG,
	COLOR_SCHEME_PLAYLISTTEXT,
	COLOR_SCHEME_PLAYLISTBG,
	COLOR_SCHEME_PLAYLISTSELTEXT,
	COLOR_SCHEME_PLAYLISTSELBG,
	COLOR_SCHEME_PLAYLIST_PLAYINGTEXT,
	COLOR_SCHEME_PLAYLIST_PLAYINGSELTEXT,
	COLOR_SCHEME_PLAYLIST_PLAYEDTEXT,
	COLOR_SCHEME_TIPTEXT,
	COLOR_SCHEME_TIPBG,
	COLOR_SCHEME_SEEKBAR,
	COLOR_SCHEME_VOLUME
};
static struct {
	ColorInfo Color;
	LPCTSTR pName;
} ColorSchemeList[] = {
	{PackColorInfo(COLOR_TYPE_BLACK,0,0,0),			TEXT("Background")},
	{PackColorInfo(COLOR_TYPE_BLACK,0,0,0),			TEXT("FullScreenBg")},
	{PackColorInfo(COLOR_TYPE_CUSTOM,128,192,255),	TEXT("StatusText")},
	{PackColorInfo(COLOR_TYPE_BLACK,0,0,0),			TEXT("StatusBg")},
	{PackColorInfo(COLOR_TYPE_CUSTOM,224,224,224),	TEXT("PlaylistText")},
	{PackColorInfo(COLOR_TYPE_BLACK,0,0,0),			TEXT("PlaylistBg")},
	{PackColorInfo(COLOR_TYPE_BLACK,0,0,0),			TEXT("PlaylistSelText")},
	{PackColorInfo(COLOR_TYPE_CUSTOM,128,192,255),	TEXT("PlaylistSelBg")},
	{PackColorInfo(COLOR_TYPE_WHITE,255,255,255),	TEXT("PlaylistPlayingText")},
	{PackColorInfo(COLOR_TYPE_CUSTOM,32,48,64),	TEXT("PlaylistPlayingSelText")},
	{PackColorInfo(COLOR_TYPE_CUSTOM,192,192,192),	TEXT("PlaylistPlayedText")},
	{PackColorInfo(COLOR_TYPE_BLACK,0,0,0),			TEXT("TipText")},
	{PackColorInfo(COLOR_TYPE_CUSTOM,128,192,255),	TEXT("TipBg")},
	{PackColorInfo(COLOR_TYPE_3DFACE,192,192,192),	TEXT("SeekBar")},
	{PackColorInfo(COLOR_TYPE_CUSTOM,128,192,255),	TEXT("Volume")},
};
static TCHAR szThemePluginFileName[MAX_PATH]=TEXT("");
struct ToolbarImageInfo {
	HIMAGELIST himl;
	HIMAGELIST himlHot;
	HIMAGELIST himlDisabled;
};
static ToolbarImageInfo MainToolbarInfo={NULL,NULL,NULL};
static ToolbarImageInfo PlayBarToolbarInfo={NULL,NULL,NULL};
static ToolbarImageInfo PlaylistToolbarInfo={NULL,NULL,NULL};




HINSTANCE GetInstance()
{
	return hInst;
}


HINSTANCE GetResourceInstance()
{
	return hInst;
}


HWND GetMainWindow()
{
	return hwndMain;
}


/*
	\[X̕ŃbZ[W{bNX\
*/
int ResMessageBox(HWND hwndOwner,UINT uTextID,UINT uCaptionID,UINT uType)
{
	TCHAR szText[MAX_MESSAGE_LENGTH],szCaption[MAX_MESSAGE_LENGTH];

	if (!LoadString(hInst,uTextID,szText,numberof(szText)))
		return 0;
	if (uCaptionID!=0) {
		if (!LoadString(hInst,uCaptionID,szCaption,numberof(szCaption)))
			return 0;
	}
	return MessageBox(hwndOwner,szText,uCaptionID!=0?szCaption:NULL,uType);
}


Settings *OpenProgramSettings(LPCTSTR pszSection,unsigned int Flags)
{
	Settings *pSettings=new Settings;

	if (!pSettings->Open(szIniFileName,pszSection,Flags)) {
		delete pSettings;
		return NULL;
	}
	return pSettings;
}


#ifdef USE_HTML_HELP
static BOOL LoadHtmlHelpLibrary()
{
	if (pfnHtmlHelp!=NULL)
		return TRUE;
	hHtmlHelpLib=(HINSTANCE)LoadLibrary(TEXT("hhctrl.ocx"));
	if (hHtmlHelpLib==NULL)
		return FALSE;
	pfnHtmlHelp=(HtmlHelpFunc)GetProcAddress(hHtmlHelpLib,
#ifndef UNICODE
											"HtmlHelpA"
#else
											"HtmlHelpW"
#endif
											);
	if (pfnHtmlHelp==NULL) {
		FreeLibrary(hHtmlHelpLib);
		hHtmlHelpLib=NULL;
		return FALSE;
	}
	(*pfnHtmlHelp)(NULL,NULL,HH_INITIALIZE,(DWORD)&dwHtmlHelpCookie);
	return TRUE;
}
#endif


/*
	wv\
*/
void ShowHelp(int ID)
{
#ifndef USE_HTML_HELP
	WinHelp(hwndMain,szHelpFileName,HELP_CONTEXT,ID);
#else
	if (LoadHtmlHelpLibrary())
		(*pfnHtmlHelp)(NULL,szHelpFileName,HH_HELP_CONTEXT,ID);
#endif
}


/*
	lw肳ꂽ͈͂Ɏ߂
*/
inline int RangeLimit(int nValue,int nMin,int nMax)
{
	return nValue<nMin?nMin:nValue>nMax?nMax:nValue;
}


/*
	ԂtH[}bg
*/
static LPTSTR FormatTime(LPTSTR pszText,UINT uMilliSeconds,BOOL fDecimal)
{
	TCHAR szTmp[16];

	wsprintf(pszText,TEXT("%02d:"),uMilliSeconds/3600000);
	wsprintf(szTmp,TEXT("%02d:"),uMilliSeconds/60000%60);
	lstrcat(pszText,szTmp);
	wsprintf(szTmp,TEXT("%02d"),uMilliSeconds/1000%60);
	lstrcat(pszText,szTmp);
	if (fDecimal) {
		TCHAR szDecimal[4];

		GetLocaleInfo(LOCALE_USER_DEFAULT,LOCALE_SDECIMAL,szDecimal,
														numberof(szDecimal));
		wsprintf(szTmp,TEXT("%s%02d"),szDecimal,uMilliSeconds/10%100);
		lstrcat(pszText,szTmp);
	}
	return pszText;
}


/*
	FIp_CAO\

	:
		(i)hwndOwner	_CAO̐eEBhẼnhB
		(i/o)pcrResult	擾FԂϐւ̃|C^B
	߂l:
		ɐFIꂽꍇTRUEALZyуG[ꍇ
		FALSEԂB
*/
BOOL ChooseColorDialog(HWND hwndOwner,COLORREF *pcrResult)
{
	CHOOSECOLOR cc;
	static COLORREF crCustomColors[16]={
		RGB(0xFF,0xFF,0xFF),RGB(0xFF,0xFF,0xFF),RGB(0xFF,0xFF,0xFF),
		RGB(0xFF,0xFF,0xFF),RGB(0xFF,0xFF,0xFF),RGB(0xFF,0xFF,0xFF),
		RGB(0xFF,0xFF,0xFF),RGB(0xFF,0xFF,0xFF),RGB(0xFF,0xFF,0xFF),
		RGB(0xFF,0xFF,0xFF),RGB(0xFF,0xFF,0xFF),RGB(0xFF,0xFF,0xFF),
		RGB(0xFF,0xFF,0xFF),RGB(0xFF,0xFF,0xFF),RGB(0xFF,0xFF,0xFF),
		RGB(0xFF,0xFF,0xFF)
	};

	cc.lStructSize=sizeof(CHOOSECOLOR);
	cc.hwndOwner=hwndOwner;
	cc.hInstance=NULL;
	cc.rgbResult=*pcrResult;
	cc.lpCustColors=crCustomColors;
	cc.Flags=CC_RGBINIT;
	if (!ChooseColor(&cc))
		return FALSE;
	*pcrResult=cc.rgbResult;
	return TRUE;
}


/*
	MCIőΉĂgq擾
*/
static void GetMCIExtensions(LPTSTR pszExtensions)
{
	TCHAR szBuff[1024];

	GetProfileString(TEXT("mci extensions"),NULL,TEXT(""),szBuff,
															numberof(szBuff));
	if (szBuff[0]!='\0') {
		static const LPCTSTR pszPlaylistFormats[] = {
			TEXT("asx"),
			TEXT("m3u"),
			TEXT("wax"),
			TEXT("wpl"),
			TEXT("wvx"),
			TEXT("wmx"),
		};
		LPTSTR p,q;
		int i;

		p=szBuff;
		q=pszExtensions;
		do {
			CharLowerBuff(p,lstrlen(p));
			for (i=0;i<numberof(pszPlaylistFormats);i++)
				if (lstrcmp(p,pszPlaylistFormats[i])==0)
					break;
			if (i<numberof(pszPlaylistFormats)) {
				p+=lstrlen(p);
			} else {
				if ((q-pszExtensions)+lstrlen(p)+3>MAX_FILE_MASK_LENGTH)
					break;
				*q++='*';
				*q++='.';
				while (*p!='\0')
					*q++=*p++;
				*q++=';';
			}
			p++;
		} while (*p!='\0');
		*(q-1)='\0';
	} else
		lstrcpy(pszExtensions,DEFAULT_FILE_MASKS);
}


#if 0


static inline TCHAR GetAccelChar(int nIndex)
{
	return nIndex<10?nIndex+'0':nIndex-10+'A';
}


/*
	t@CɃt@Cǉ
*/
static BOOL FileHistory_Add(LPCTSTR pszFileName)
{
	HMENU hmenu,hmenuPopup;
	int nMenuItems,i;
	TCHAR szText[MAX_PATH+4];

	if (nMaxFileHistory<=0)
		return FALSE;
	hmenu=hmenuMain;
	hmenuPopup=GetSubMenu(GetSubMenu(hmenu,0),FILE_HISTORY_MENU);
	nMenuItems=GetMenuItemCount(hmenuPopup);
	if (nMenuItems==0)
		EnableMenuItem(GetSubMenu(hmenu,0),FILE_HISTORY_MENU,
												MF_BYPOSITION | MFS_ENABLED);
	for (i=0;i<nMenuItems;) {
		GetMenuString(hmenuPopup,i,szText,numberof(szText),MF_BYPOSITION);
		if (lstrcmpi(pszFileName,szText+4)==0) {
			if (i==0) {
				// ɍŏɂ
				return TRUE;
			} else {
				DeleteMenu(hmenuPopup,i,MF_BYPOSITION);
				nMenuItems--;
				continue;
			}
		}
		szText[1]=GetAccelChar(i+1);
		ModifyMenu(hmenuPopup,i,MF_BYPOSITION | MFT_STRING,
											CM_FILEHISTORY_FIRST+i+1,szText);
		i++;
	}
	if (i>=nMaxFileHistory)
		DeleteMenu(hmenuPopup,nMaxFileHistory-1,MF_BYPOSITION);
	lstrcpy(szText,TEXT("&0: "));
	lstrcat(szText,pszFileName);
	InsertMenu(hmenuPopup,0,MF_BYPOSITION | MFT_STRING,CM_FILEHISTORY_FIRST,
																	szText);
	return TRUE;
}


/*
	t@Ct@C폜
*/
static BOOL FileHistory_Delete(int nNo)
{
	HMENU hmenu,hmenuPopup;
	int nItems,i;
	TCHAR szText[MAX_PATH+4];

	hmenu=GetSubMenu(hmenuMain,0);
	hmenuPopup=GetSubMenu(hmenu,FILE_HISTORY_MENU);
	nItems=GetMenuItemCount(hmenuPopup);
	if (nNo<0 || nNo>=nItems)
		return FALSE;
	DeleteMenu(hmenuPopup,nNo,MF_BYPOSITION);
	nItems--;
	if (nItems>0) {
		for (i=nNo;i<nItems;i++) {
			GetMenuString(hmenuPopup,i,szText,numberof(szText),MF_BYPOSITION);
			szText[1]=GetAccelChar(i);
			ModifyMenu(hmenuPopup,i,MF_BYPOSITION | MFT_STRING,
												CM_FILEHISTORY_FIRST+i,szText);
		}
	} else {
		EnableMenuItem(hmenu,FILE_HISTORY_MENU,MF_BYPOSITION | MFS_GRAYED);
	}
	return TRUE;
}


/*
	t@CNA
*/
static void FileHistory_Clear()
{
	HMENU hmenu,hmenuPopup;
	int nMenuItems,i;

	hmenu=GetSubMenu(hmenuMain,FILE_MENU);
	hmenuPopup=GetSubMenu(hmenu,FILE_HISTORY_MENU);
	nMenuItems=GetMenuItemCount(hmenuPopup);
	if (nMenuItems==0)
		return;
	for (i=0;i<nMenuItems;i++)
		DeleteMenu(hmenuPopup,0,MF_BYPOSITION);
	EnableMenuItem(hmenu,FILE_HISTORY_MENU,MF_BYPOSITION | MFS_GRAYED);
}


/*
	t@C̍ő吔ύX
*/
static void FileHistory_ChangeMax()
{
	if (nMaxFileHistory==0) {
		FileHistory_Clear();
	} else {
		HMENU hmenu;
		int nItems,i;

		hmenu=GetSubMenu(GetSubMenu(hmenuMain,FILE_MENU),FILE_HISTORY_MENU);
		nItems=GetMenuItemCount(hmenu);
		if (nItems>nMaxFileHistory) {
			for (i=nMaxFileHistory;i<nItems;i++)
				DeleteMenu(hmenu,i,MF_BYPOSITION);
		}
	}
}


/*
	t@C̃t@C擾
*/
static int FileHistory_GetCount()
{
	HMENU hmenu;

	hmenu=GetSubMenu(GetSubMenu(hmenuMain,FILE_MENU),FILE_HISTORY_MENU);
	return GetMenuItemCount(hmenu);
}


/*
	t@Cǂݍ
*/
static BOOL FileHistory_Load(LPCTSTR pszFileName)
{
	Settings *pSettings;
	int nFiles,i;
	HMENU hmenu,hmenuPopup;
	TCHAR szName[8];
	TCHAR szFileName[MAX_PATH],szMenu[MAX_PATH+4];

	if (nMaxFileHistory==0)
		return TRUE;
	pSettings=new Settings;
	if (!pSettings->Open(pszFileName,TEXT("RecentFileList"),
														Settings::OPEN_READ)) {
		delete pSettings;
		return FALSE;
	}
	if (!pSettings->ReadInt(TEXT("FileCount"),&nFiles) || nFiles<=0) {
		delete pSettings;
		return FALSE;
	}
	if (nFiles>nMaxFileHistory)
		nFiles=nMaxFileHistory;
	hmenu=GetSubMenu(hmenuMain,0);
	hmenuPopup=GetSubMenu(hmenu,FILE_HISTORY_MENU);
	for (i=0;i<nFiles;i++) {
		wsprintf(szName,TEXT("File%d"),i);
		if (!pSettings->ReadString(szName,szFileName,numberof(szFileName))
														|| szFileName[0]=='\0')
			break;
		wsprintf(szMenu,TEXT("&%c: %s"),GetAccelChar(i),szFileName);
		AppendMenu(hmenuPopup,MFT_STRING,CM_FILEHISTORY_FIRST+i,szMenu);
	}
	delete pSettings;
	EnableMenuItem(hmenu,FILE_HISTORY_MENU,
								MF_BYPOSITION | (i>0?MFS_ENABLED:MFS_GRAYED));
	return TRUE;
}


/*
	t@Cۑ
*/
static BOOL FileHistory_Save(LPCTSTR pszFileName)
{
	Settings HistorySettings;

	if (!HistorySettings.Open(pszFileName,TEXT("RecentFileList"),
														Settings::OPEN_WRITE))
		return FALSE;
	HistorySettings.Clear();
	if (fSaveFileHistory) {
		HMENU hmenu;
		int nItems,i;
		TCHAR szText[MAX_PATH+4],szName[8];

		hmenu=GetSubMenu(GetSubMenu(hmenuMain,0),FILE_HISTORY_MENU);
		nItems=GetMenuItemCount(hmenu);
		HistorySettings.WriteInt(TEXT("FileCount"),nItems);
		for (i=0;i<nItems;i++) {
			GetMenuString(hmenu,i,szText,numberof(szText),MF_BYPOSITION);
			wsprintf(szName,TEXT("File%d"),i);
			HistorySettings.WriteString(szName,szText+4);
		}
	} else
		HistorySettings.WriteInt(TEXT("FileCount"),0);
	return TRUE;
}


/*
	fBNgɃfBNgǉ
*/
static BOOL DirHistory_Add(LPCTSTR pszDirectory)
{
	HMENU hmenu,hmenuPopup;
	int nMenuItems,i;
	TCHAR szText[MAX_PATH+4];

	if (nMaxDirectoryHistory<=0)
		return FALSE;
	hmenu=hmenuMain;
	hmenuPopup=GetSubMenu(GetSubMenu(hmenu,0),DIR_HISTORY_MENU);
	nMenuItems=GetMenuItemCount(hmenuPopup);
	if (nMenuItems==0)
		EnableMenuItem(GetSubMenu(hmenu,0),DIR_HISTORY_MENU,
												MF_BYPOSITION | MFS_ENABLED);
	for (i=0;i<nMenuItems;) {
		GetMenuString(hmenuPopup,i,szText,numberof(szText),MF_BYPOSITION);
		if (lstrcmpi(pszDirectory,szText+4)==0) {
			if (i==0) {
				// ɍŏɂ
				return TRUE;
			} else {
				DeleteMenu(hmenuPopup,i,MF_BYPOSITION);
				nMenuItems--;
				continue;
			}
		}
		szText[1]=GetAccelChar(i+1);
		ModifyMenu(hmenuPopup,i,MF_BYPOSITION | MFT_STRING,
											CM_DIRHISTORY_FIRST+i+1,szText);
		i++;
	}
	if (i>=nMaxDirectoryHistory)
		DeleteMenu(hmenuPopup,nMaxDirectoryHistory-1,MF_BYPOSITION);
	lstrcpy(szText,TEXT("&0: "));
	lstrcat(szText,pszDirectory);
	InsertMenu(hmenuPopup,0,MF_BYPOSITION | MFT_STRING,CM_DIRHISTORY_FIRST,
																	szText);
	return TRUE;
}


/*
	fBNg̃ACe폜
*/
static BOOL DirHistory_Delete(int nNo)
{
	HMENU hmenu,hmenuPopup;
	int nItems,i;
	TCHAR szText[MAX_PATH+4];

	hmenu=GetSubMenu(hmenuMain,0);
	hmenuPopup=GetSubMenu(hmenu,DIR_HISTORY_MENU);
	nItems=GetMenuItemCount(hmenuPopup);
	if (nNo<0 || nNo>=nItems)
		return FALSE;
	DeleteMenu(hmenuPopup,nNo,MF_BYPOSITION);
	nItems--;
	if (nItems>0) {
		for (i=nNo;i<nItems;i++) {
			GetMenuString(hmenuPopup,i,szText,numberof(szText),MF_BYPOSITION);
			szText[1]=GetAccelChar(i);
			ModifyMenu(hmenuPopup,i,MF_BYPOSITION | MFT_STRING,
												CM_DIRHISTORY_FIRST+i,szText);
		}
	} else {
		EnableMenuItem(hmenu,DIR_HISTORY_MENU,MF_BYPOSITION | MFS_GRAYED);
	}
	return TRUE;
}


/*
	fBNgNA
*/
static void DirHistory_Clear()
{
	HMENU hmenu,hmenuPopup;
	int nMenuItems,i;

	hmenu=GetSubMenu(hmenuMain,0);
	hmenuPopup=GetSubMenu(hmenu,DIR_HISTORY_MENU);
	nMenuItems=GetMenuItemCount(hmenuPopup);
	if (nMenuItems==0)
		return;
	for (i=0;i<nMenuItems;i++)
		DeleteMenu(hmenuPopup,0,MF_BYPOSITION);
	EnableMenuItem(hmenu,DIR_HISTORY_MENU,MF_BYPOSITION | MFS_GRAYED);
}


/*
	fBNg̍ő吔ύX
*/
static void DirHistory_ChangeMax()
{
	if (nMaxDirectoryHistory==0) {
		DirHistory_Clear();
	} else {
		HMENU hmenu;
		int nItems,i;

		hmenu=GetSubMenu(GetSubMenu(hmenuMain,0),DIR_HISTORY_MENU);
		nItems=GetMenuItemCount(hmenu);
		if (nItems>nMaxDirectoryHistory) {
			for (i=nMaxDirectoryHistory;i<nItems;i++)
				DeleteMenu(hmenu,i,MF_BYPOSITION);
		}
	}
}


/*
	fBNg̃ACe擾
*/
static int DirHistory_GetCount()
{
	HMENU hmenu;

	hmenu=GetSubMenu(GetSubMenu(hmenuMain,0),DIR_HISTORY_MENU);
	return GetMenuItemCount(hmenu);
}


/*
	fBNgǂݍ
*/
static BOOL DirHistory_Load(LPCTSTR pszFileName)
{
	Settings *pSettings;
	int nDirs,i;
	HMENU hmenu,hmenuPopup;
	TCHAR szName[8];
	TCHAR szDirectory[MAX_PATH],szMenu[MAX_PATH+4];

	if (nMaxDirectoryHistory==0)
		return TRUE;
	pSettings=new Settings;
	if (!pSettings->Open(pszFileName,TEXT("RecentDirList"),
														Settings::OPEN_READ)) {
		delete pSettings;
		return FALSE;
	}
	if (!pSettings->ReadInt(TEXT("DirCount"),&nDirs) || nDirs<=0) {
		delete pSettings;
		return FALSE;
	}
	if (nDirs>nMaxDirectoryHistory)
		nDirs=nMaxDirectoryHistory;
	hmenu=GetSubMenu(hmenuMain,0);
	hmenuPopup=GetSubMenu(hmenu,DIR_HISTORY_MENU);
	for (i=0;i<nDirs;i++) {
		wsprintf(szName,TEXT("Dir%d"),i);
		if (!pSettings->ReadString(szName,szDirectory,numberof(szDirectory))
													|| szDirectory[0]=='\0')
			break;
		wsprintf(szMenu,TEXT("&%c: %s"),GetAccelChar(i),szDirectory);
		AppendMenu(hmenuPopup,MFT_STRING,CM_DIRHISTORY_FIRST+i,szMenu);
	}
	delete pSettings;
	EnableMenuItem(hmenu,DIR_HISTORY_MENU,
								MF_BYPOSITION | (i>0?MFS_ENABLED:MFS_GRAYED));
	return TRUE;
}


/*
	fBNgۑ
*/
static BOOL DirHistory_Save(LPCTSTR pszFileName)
{
	Settings HistorySettings;

	if (!HistorySettings.Open(pszFileName,TEXT("RecentDirList"),
														Settings::OPEN_WRITE))
		return FALSE;
	HistorySettings.Clear();
	if (fSaveDirectoryHistory) {
		HMENU hmenu;
		int nItems,i;
		TCHAR szText[MAX_PATH+4],szName[8];

		hmenu=GetSubMenu(GetSubMenu(hmenuMain,0),DIR_HISTORY_MENU);
		nItems=GetMenuItemCount(hmenu);
		HistorySettings.WriteInt(TEXT("DirCount"),nItems);
		for (i=0;i<nItems;i++) {
			GetMenuString(hmenu,i,szText,numberof(szText),MF_BYPOSITION);
			wsprintf(szName,TEXT("Dir%d"),i);
			HistorySettings.WriteString(szName,szText+4);
		}
	} else
		HistorySettings.WriteInt(TEXT("DirCount"),0);
	return TRUE;
}


#endif


static BOOL GetUserProfileDirectory(LPTSTR pszDirectory)
{
	ITEMIDLIST *pid;
	TCHAR szDir[MAX_PATH];

	if (SHGetSpecialFolderLocation(NULL,CSIDL_APPDATA,&pid)!=NOERROR)
		return FALSE;
	if (!SHGetPathFromIDList(pid,szDir)) {
		CoTaskMemFree(pid);
		return FALSE;
	}
	CoTaskMemFree(pid);
	MergeFileName(pszDirectory,szDir,TEXT("LafPlayer"));
	return TRUE;
}


static BOOL GetUserProfileName(LPTSTR pszFileName)
{
	TCHAR szDirectory[MAX_PATH];

	if (!GetUserProfileDirectory(szDirectory))
		return FALSE;
	MergeFileName(pszFileName,szDirectory,INI_FILE_NAME);
	return TRUE;
}


static void GetCommonProfileName(LPTSTR pszFileName)
{
	GetModuleFileName(hInst,pszFileName,MAX_PATH);
	lstrcpy(GetFileNamePtr(pszFileName),INI_FILE_NAME);
}


/*
	ݒǂݍ
*/
static BOOL LoadSettings(LPCTSTR pszFileName)
{
	Settings *pSettings=new Settings;
	int Value;
	BOOL fValue;

	pSettings->Open(pszFileName,TEXT("Settings"),Settings::OPEN_READ);
	pSettings->ReadInt(TEXT("WindowLeft"),&nWindowLeft);
	pSettings->ReadInt(TEXT("WindowTop"),&nWindowTop);
	pSettings->ReadInt(TEXT("WindowWidth"),&nWindowWidth);
	pSettings->ReadInt(TEXT("WindowHeight"),&nWindowHeight);
	pSettings->ReadBoolean(TEXT("Maximize"),&fWindowMaximized);
	pSettings->ReadInt(TEXT("PlaylistLeft"),&nPlaylistLeft);
	pSettings->ReadInt(TEXT("PlaylistTop"),&nPlaylistTop);
	pSettings->ReadInt(TEXT("PlaylistWidth"),&nPlaylistWidth);
	pSettings->ReadInt(TEXT("PlaylistHeight"),&nPlaylistHeight);
	pSettings->ReadString(TEXT("OpenDirectory"),szFileOpenDirectory,
												numberof(szFileOpenDirectory));
	pSettings->ReadInt(TEXT("OpenFilter"),&nOpenFilterIndex);
	pSettings->ReadBoolean(TEXT("Repeat"),&fPlayRepeat);
	pSettings->ReadBoolean(TEXT("SequentialPlay"),&fSequentialPlay);
	pSettings->ReadBoolean(TEXT("RepeatPlaylist"),&fRepeatPlaylist);
	pSettings->ReadBoolean(TEXT("ShowPlaylist"),&fShowPlaylist);
	pSettings->ReadBoolean(TEXT("ShowMenuBar"),&fShowMenuBar);
	pSettings->ReadBoolean(TEXT("ShowToolbar"),&fShowToolbar);
	pSettings->ReadBoolean(TEXT("ShowStatusBar"),&fShowStatusBar);
	pSettings->ReadBoolean(TEXT("ShowPlayBar"),&fShowPlayBar);
	pSettings->ReadBoolean(TEXT("ShowSeekBar"),&fShowSeekBar);
	pSettings->ReadBoolean(TEXT("KeepZoomRate"),&fKeepZoomRate);
	pSettings->ReadInt(TEXT("ZoomRate"),&nPrevZoomRate);
	nPrevZoomRate=RangeLimit(nPrevZoomRate,MIN_ZOOM,MAX_ZOOM);
	pSettings->ReadBoolean(TEXT("FitToWindow"),&fFitToWindow);
	if (fFitToWindow && fKeepZoomRate)
		fKeepZoomRate=false;
	pSettings->ReadBoolean(TEXT("AlwaysOnTop"),&fAlwaysOnTop);
	if (pSettings->ReadInt(TEXT("SortType"),&Value)
				&& Value>=0 && Value<=DirectoryFileList::SORT_LASTWRITETIME)
		DirSortType=(DirectoryFileList::SortType)Value;
	if (pSettings->ReadInt(TEXT("Volume"),&DefaultVolume))
		DefaultVolume=RangeLimit(DefaultVolume,0,MAX_VOLUME);
	pSettings->ReadBoolean(TEXT("SortDescending"),&fDirSortDescending);
	pSettings->ReadBoolean(TEXT("KeepSingleTask"),&fKeepSingleTask);
	if (pSettings->ReadInt(TEXT("TitleFormat"),&Value)
										&& Value>=TITLE_FORMAT_PROGRAM_FILENAME
										&& Value<=TITLE_FORMAT_PROGRAM)
		TitleBarFormat=(TitleBarFormatType)Value;
	if (pSettings->ReadInt(TEXT("TitleFileName"),&Value)) {
		if (Value>=TITLE_FILEPATH_FILENAME
								&& Value<=TITLE_FILEPATH_DIRECTORY_FILENAME)
			TitleFilePath=(TitleFilePathType)Value;
	} else if (pSettings->ReadBoolean(TEXT("ShowFullPathName"),&fValue))
		TitleFilePath=fValue?TITLE_FILEPATH_FULLPATH:TITLE_FILEPATH_FILENAME;
	pSettings->ReadBoolean(TEXT("ShowToolTips"),&fShowTipHelp);
	pSettings->ReadBoolean(TEXT("SaveWindowPos"),&fSaveWindowPos);
	pSettings->ReadBoolean(TEXT("SaveWindowSize"),&fSaveWindowSize);
	pSettings->ReadBoolean(TEXT("FindSubDir"),&fFindSubDirectory);
	pSettings->ReadString(TEXT("FileMasks"),szFileMasks,numberof(szFileMasks));
	if (pSettings->ReadInt(TEXT("MediaDevice"),&Value))
		MediaDevice=Value==0?MEDIA_DEVICE_MCI:MEDIA_DEVICE_DIRECTSHOW;
	if (pSettings->ReadInt(TEXT("MaxFileHistory"),&nMaxFileHistory))
		nMaxFileHistory=RangeLimit(nMaxFileHistory,0,MAX_FILE_HISTORY);
	FileHistory.SetMax(nMaxFileHistory);
	pSettings->ReadBoolean(TEXT("SaveFileHistory"),&fSaveFileHistory);
	if (pSettings->ReadInt(TEXT("MaxDirHistory"),&nMaxDirectoryHistory))
		nMaxDirectoryHistory=RangeLimit(nMaxDirectoryHistory,0,
														MAX_DIRECTORY_HISTORY);
	DirHistory.SetMax(nMaxDirectoryHistory);
	pSettings->ReadBoolean(TEXT("SaveDirHistory"),&fSaveDirectoryHistory);
	pSettings->ReadBoolean(TEXT("UseDirHistoryMenu"),&fUseDirHistoryMenu);
	if (pSettings->ReadInt(TEXT("MaxFileListMenu"),&MaxFileListMenuFiles))
		MaxFileListMenuFiles=RangeLimit(MaxFileListMenuFiles,1,
													MAX_FILE_LIST_MENU_FILES);
	pSettings->ReadBoolean(TEXT("PlayOnOpen"),&fPlayOnOpen);
	if (pSettings->ReadInt(TEXT("ZoomStep"),&nZoomStep))
		nZoomStep=RangeLimit(nZoomStep,MIN_ZOOM_STEP,MAX_ZOOM_STEP);
	if (pSettings->ReadInt(TEXT("BackwardSpeed"),&nBackwardSpeed))
		nBackwardSpeed=RangeLimit(nBackwardSpeed,MIN_BACKWARD_SPEED,
														MAX_BACKWARD_SPEED);
	if (pSettings->ReadInt(TEXT("ForwardSpeed"),&nForwardSpeed))
		nForwardSpeed=RangeLimit(nForwardSpeed,MIN_FORWARD_SPEED,
															MAX_FORWARD_SPEED);
	pSettings->ReadBoolean(TEXT("FullScreenSeekBar"),&fShowFullScreenSeekBar);
	pSettings->ReadBoolean(TEXT("FullScreenPlayBar"),&fShowFullScreenPlayBar);
	pSettings->ReadBoolean(TEXT("FitToDisplay"),&fFullScreenFitToDisplay);
	pSettings->ReadBoolean(TEXT("FullScreenHideCursor"),
													&fFullScreenHideCursor);
	pSettings->ReadBoolean(TEXT("PlaylistKeepForeground"),
													&fPlaylistKeepForeground);
	pSettings->ReadBoolean(TEXT("PlaylistSingleClickOpen"),
													&fPlaylistSingleClickOpen);
	if (pSettings->ReadInt(TEXT("PlaylistFileName"),&Value)) {
		if (Value>=Playlist::FILENAME_FILENAME
							&& Value<=Playlist::FILENAME_DIRECTORY_FILENAME)
			PlaylistFileNameType=(Playlist::FileNameType)Value;
	} else if (pSettings->ReadBoolean(TEXT("PlaylistFullPath"),&fValue))
		PlaylistFileNameType=fValue?Playlist::FILENAME_FULLPATH:
												Playlist::FILENAME_FILENAME;
	pSettings->ReadBoolean(TEXT("RestorePlaylist"),&fPlaylistRestoreList);
	pSettings->ReadInt(TEXT("MaxPlaylistHistory"),&MaxPlaylistHistory);
	MaxPlaylistHistory=RangeLimit(MaxPlaylistHistory,0,MAX_PLAYLIST_HISTORY);
	pSettings->ReadString(TEXT("CurThemeFile"),szCurThemeFileName,
												numberof(szCurThemeFileName));
	for (int i=0;i<numberof(ColorSchemeList);i++) {
		TCHAR szName[64],szText[32];
		COLORREF cr;

		wsprintf(szName,TEXT("%sColor"),ColorSchemeList[i].pName);
		if (pSettings->ReadString(szName,szText,numberof(szText))
														&& szText[0]!='\0') {
			ColorInfo_Parse(&ColorSchemeList[i].Color,szText);
		} else if ((ColorSchemeType)i==COLOR_SCHEME_PLAYLIST_PLAYINGTEXT) {
			ColorSchemeList[i].Color=
							ColorSchemeList[COLOR_SCHEME_PLAYLISTTEXT].Color;
		} else if ((ColorSchemeType)i==COLOR_SCHEME_PLAYLIST_PLAYINGSELTEXT) {
			ColorSchemeList[i].Color=
						ColorSchemeList[COLOR_SCHEME_PLAYLISTSELTEXT].Color;
		} else if ((ColorSchemeType)i==COLOR_SCHEME_PLAYLIST_PLAYEDTEXT) {
			ColorSchemeList[i].Color=
							ColorSchemeList[COLOR_SCHEME_PLAYLISTTEXT].Color;
		}
		if (ColorInfo_GetType(ColorSchemeList[i].Color)!=COLOR_TYPE_CUSTOM) {
			wsprintf(szName,TEXT("%sCustomColor"),ColorSchemeList[i].pName);
			if (pSettings->ReadColor(szName,&cr))
				ColorInfo_SetCustomColor(&ColorSchemeList[i].Color,
								 GetRValue(cr),GetGValue(cr),GetBValue(cr));
		}
	}
	pSettings->ReadString(TEXT("ThemePlugin"),szThemePluginFileName,
											numberof(szThemePluginFileName));
	if (pSettings->Open(pszFileName,TEXT("Presets"),Settings::OPEN_READ)) {
		int Count;

		if (pSettings->ReadInt(TEXT("Count"),&Count) && Count>0) {
			int i;
			TCHAR szKey[16];

			if (Count>NUM_PRESETS)
				Count=NUM_PRESETS;
			for (i=0;i<Count;i++) {
				wsprintf(szKey,TEXT("Name%d"),i);
				pSettings->ReadString(szKey,PresetList[i].szName,
															MAX_PRESET_NAME);
				wsprintf(szKey,TEXT("Status%d"),i);
				pSettings->ReadUInt(szKey,&PresetList[i].Flags);
			}
		}
	}
	delete pSettings;
	return TRUE;
}


/*
	ݒۑ
*/
static BOOL SaveSettings(LPCTSTR pszFileName)
{
	Settings *pSettings=new Settings;

	pSettings->Open(pszFileName,TEXT("Settings"),Settings::OPEN_WRITE);
	if (fSaveWindowPos) {
		if (nWindowLeft!=CW_USEDEFAULT)
			pSettings->WriteInt(TEXT("WindowLeft"),nWindowLeft);
		if (nWindowTop!=CW_USEDEFAULT)
			pSettings->WriteInt(TEXT("WindowTop"),nWindowTop);
	}
	if (fSaveWindowSize) {
		if (nWindowWidth!=CW_USEDEFAULT)
			pSettings->WriteInt(TEXT("WindowWidth"),nWindowWidth);
		if (nWindowHeight!=CW_USEDEFAULT)
			pSettings->WriteInt(TEXT("WindowHeight"),nWindowHeight);
	}
	pSettings->WriteBoolean(TEXT("Maximize"),fWindowMaximized);
	if (nPlaylistLeft!=CW_USEDEFAULT)
		pSettings->WriteInt(TEXT("PlaylistLeft"),nPlaylistLeft);
	if (nPlaylistTop!=CW_USEDEFAULT)
		pSettings->WriteInt(TEXT("PlaylistTop"),nPlaylistTop);
	if (nPlaylistWidth!=CW_USEDEFAULT)
		pSettings->WriteInt(TEXT("PlaylistWidth"),nPlaylistWidth);
	if (nPlaylistHeight!=CW_USEDEFAULT)
		pSettings->WriteInt(TEXT("PlaylistHeight"),nPlaylistHeight);
	pSettings->WriteString(TEXT("OpenDirectory"),szFileOpenDirectory);
	pSettings->WriteInt(TEXT("OpenFilter"),nOpenFilterIndex);
	pSettings->WriteBoolean(TEXT("Repeat"),fPlayRepeat);
	pSettings->WriteBoolean(TEXT("SequentialPlay"),fSequentialPlay);
	pSettings->WriteBoolean(TEXT("RepeatPlaylist"),fRepeatPlaylist);
	pSettings->WriteBoolean(TEXT("ShowPlaylist"),fShowPlaylist);
	pSettings->WriteBoolean(TEXT("ShowMenuBar"),fShowMenuBar);
	pSettings->WriteBoolean(TEXT("ShowToolbar"),fShowToolbar);
	pSettings->WriteBoolean(TEXT("ShowStatusBar"),fShowStatusBar);
	pSettings->WriteBoolean(TEXT("ShowPlayBar"),fShowPlayBar);
	pSettings->WriteBoolean(TEXT("ShowSeekBar"),fShowSeekBar);
	pSettings->WriteBoolean(TEXT("KeepZoomRate"),fKeepZoomRate);
	pSettings->WriteInt(TEXT("ZoomRate"),nPrevZoomRate);
	pSettings->WriteBoolean(TEXT("FitToWindow"),fFitToWindow);
	pSettings->WriteBoolean(TEXT("AlwaysOnTop"),fAlwaysOnTop);
	pSettings->WriteInt(TEXT("SortType"),(int)DirSortType);
	pSettings->WriteBoolean(TEXT("SortDescending"),fDirSortDescending);
	pSettings->WriteInt(TEXT("Volume"),DefaultVolume);
	pSettings->WriteBoolean(TEXT("KeepSingleTask"),fKeepSingleTask);
	pSettings->WriteInt(TEXT("TitleFormat"),TitleBarFormat);
	pSettings->WriteInt(TEXT("TitleFileName"),TitleFilePath);
	pSettings->WriteBoolean(TEXT("ShowToolTips"),fShowTipHelp);
	pSettings->WriteBoolean(TEXT("SaveWindowPos"),fSaveWindowPos);
	pSettings->WriteBoolean(TEXT("SaveWindowSize"),fSaveWindowSize);
	pSettings->WriteBoolean(TEXT("FindSubDir"),fFindSubDirectory);
	pSettings->WriteString(TEXT("FileMasks"),szFileMasks);
	pSettings->WriteInt(TEXT("MediaDevice"),(int)MediaDevice);
	pSettings->WriteInt(TEXT("MaxFileHistory"),nMaxFileHistory);
	pSettings->WriteBoolean(TEXT("SaveFileHistory"),fSaveFileHistory);
	pSettings->WriteInt(TEXT("MaxDirHistory"),nMaxDirectoryHistory);
	pSettings->WriteBoolean(TEXT("SaveDirHistory"),fSaveDirectoryHistory);
	pSettings->WriteBoolean(TEXT("UseDirHistoryMenu"),fUseDirHistoryMenu);
	pSettings->WriteInt(TEXT("MaxFileListMenu"),MaxFileListMenuFiles);
	pSettings->WriteBoolean(TEXT("PlayOnOpen"),fPlayOnOpen);
	pSettings->WriteInt(TEXT("ZoomStep"),nZoomStep);
	pSettings->WriteInt(TEXT("BackwardSpeed"),nBackwardSpeed);
	pSettings->WriteInt(TEXT("ForwardSpeed"),nForwardSpeed);
	pSettings->WriteBoolean(TEXT("FullScreenSeekBar"),fShowFullScreenSeekBar);
	pSettings->WriteBoolean(TEXT("FullScreenPlayBar"),fShowFullScreenPlayBar);
	pSettings->WriteBoolean(TEXT("FitToDisplay"),fFullScreenFitToDisplay);
	pSettings->WriteBoolean(TEXT("FullScreenHideCursor"),fFullScreenHideCursor);
	pSettings->WriteBoolean(TEXT("PlaylistKeepForeground"),
													fPlaylistKeepForeground);
	pSettings->WriteBoolean(TEXT("PlaylistSingleClickOpen"),
													fPlaylistSingleClickOpen);
	pSettings->WriteInt(TEXT("PlaylistFileName"),PlaylistFileNameType);
	pSettings->WriteBoolean(TEXT("RestorePlaylist"),fPlaylistRestoreList);
	pSettings->WriteInt(TEXT("MaxPlaylistHistory"),MaxPlaylistHistory);
	pSettings->WriteString(TEXT("CurThemeFile"),szCurThemeFileName);
	for (int i=0;i<numberof(ColorSchemeList);i++) {
		TCHAR szName[64],szText[32];

		ColorInfo_GetText(ColorSchemeList[i].Color,szText);
		wsprintf(szName,TEXT("%sColor"),ColorSchemeList[i].pName);
		pSettings->WriteString(szName,szText);
		wsprintf(szName,TEXT("%sCustomColor"),ColorSchemeList[i].pName);
		pSettings->WriteColor(szName,
						ColorInfo_GetCustomColor(ColorSchemeList[i].Color));
	}
	pSettings->WriteString(TEXT("ThemePlugin"),szThemePluginFileName);
	if (pSettings->Open(pszFileName,TEXT("Presets"),Settings::OPEN_WRITE)) {
		int i;
		TCHAR szKey[16];

		pSettings->Clear();
		pSettings->WriteInt(TEXT("Count"),NUM_PRESETS);
		for (i=0;i<NUM_PRESETS;i++) {
			wsprintf(szKey,TEXT("Name%d"),i);
			pSettings->WriteString(szKey,PresetList[i].szName);
			wsprintf(szKey,TEXT("Status%d"),i);
			pSettings->WriteUInt(szKey,PresetList[i].Flags);
		}
	}
	delete pSettings;
	if (!fSaveFileHistory)
		FileHistory.Clear();
	FileHistory.Save(pszFileName,TEXT("RecentFileList"),TEXT("File"));
	if (!fSaveDirectoryHistory)
		DirHistory.Clear();
	DirHistory.Save(pszFileName,TEXT("RecentDirList"),TEXT("Dir"));
	return TRUE;
}


static void DeleteDirFileList()
{
	if (pDirFileList!=NULL) {
		delete pDirFileList;
		pDirFileList=NULL;
	}
}


static bool CreateDirFileList()
{
	if (pDirFileList==NULL) {
		TCHAR szFileName[MAX_PATH],szDirectory[MAX_PATH];

		pDirFileList=new DirectoryFileList;
		pMediaPlayer->GetFileName(szFileName,numberof(szFileName));
		GetFileDirectory(szDirectory,szFileName);
		if (!pDirFileList->FindFiles(szDirectory,szFileMasks)) {
			DeleteDirFileList();
			return false;
		}
		pDirFileList->Sort(DirSortType,fDirSortDescending);
	}
	return true;
}


static bool RefreshDirFileList()
{
	DeleteDirFileList();
	return CreateDirFileList();
}


static void ApplyColorScheme()
{
	COLORREF crTipText,crTipBack;
	HWND hwndToolTips;

	crTipText=ColorInfo_GetColor(ColorSchemeList[COLOR_SCHEME_TIPTEXT].Color);
	crTipBack=ColorInfo_GetColor(ColorSchemeList[COLOR_SCHEME_TIPBG].Color);
	hwndToolTips=(HWND)SendMessage(hwndToolbar,TB_GETTOOLTIPS,0,0);
	SendMessage(hwndToolTips,TTM_SETTIPTEXTCOLOR,crTipText,0);
	SendMessage(hwndToolTips,TTM_SETTIPBKCOLOR,crTipBack,0);
	pMediaPlayer->SetBackgroundColor(
		ColorInfo_GetColor(ColorSchemeList[COLOR_SCHEME_BACKGROUND].Color));
	pMediaPlayer->SetFullScreenBgColor(
		ColorInfo_GetColor(ColorSchemeList[COLOR_SCHEME_FULLSCREENBG].Color));
	pMediaPlayer->SetTipColor(crTipText,crTipBack);
	pMediaPlayer->SetVolumeColor(
		ColorInfo_GetColor(ColorSchemeList[COLOR_SCHEME_VOLUME].Color));
	pMediaPlayer->SetSeekBarBarColor(
		ColorInfo_GetColor(ColorSchemeList[COLOR_SCHEME_SEEKBAR].Color));
	pStatusBar->SetTextColor(
		ColorInfo_GetColor(ColorSchemeList[COLOR_SCHEME_STATUSTEXT].Color));
	pStatusBar->SetBackgroundColor(
		ColorInfo_GetColor(ColorSchemeList[COLOR_SCHEME_STATUSBG].Color));
	pPlaylist->SetColors(
		ColorInfo_GetColor(ColorSchemeList[COLOR_SCHEME_PLAYLISTTEXT].Color),
		ColorInfo_GetColor(ColorSchemeList[COLOR_SCHEME_PLAYLISTBG].Color),
		ColorInfo_GetColor(ColorSchemeList[COLOR_SCHEME_PLAYLISTSELTEXT].Color),
		ColorInfo_GetColor(ColorSchemeList[COLOR_SCHEME_PLAYLISTSELBG].Color),
		ColorInfo_GetColor(
				ColorSchemeList[COLOR_SCHEME_PLAYLIST_PLAYINGTEXT].Color),
		ColorInfo_GetColor(
				ColorSchemeList[COLOR_SCHEME_PLAYLIST_PLAYINGSELTEXT].Color),
		ColorInfo_GetColor(
				ColorSchemeList[COLOR_SCHEME_PLAYLIST_PLAYEDTEXT].Color));
	pPlaylist->SetTipColor(crTipText,crTipBack);
}


/*
	EBhẼ^Cgݒ肷
*/
static void SetWindowTitle(HWND hwnd)
{
	if (pMediaPlayer->IsOpen() && TitleBarFormat!=TITLE_FORMAT_PROGRAM) {
		TCHAR szFileName[MAX_PATH];
		LPTSTR pszFile;
		TCHAR szTitle[numberof(TITLE_TEXT)+3+MAX_PATH];

		pMediaPlayer->GetFileName(szFileName,numberof(szFileName));
		if (TitleFilePath==TITLE_FILEPATH_FILENAME) {
			pszFile=GetFileNamePtr(szFileName);
		} else {
			if (TitleFilePath==TITLE_FILEPATH_DIRECTORY_FILENAME) {
				TCHAR szOrigName[MAX_PATH];

				lstrcpy(szOrigName,szFileName);
				GetFilePathEllipsis(szFileName,szOrigName);
			}
			pszFile=szFileName;
		}
		wsprintf(szTitle,TitleBarFormat==TITLE_FORMAT_PROGRAM_FILENAME?
					TITLE_TEXT TEXT(" - %s"):TEXT("%s - ") TITLE_TEXT,pszFile);
		SetWindowText(hwnd,szTitle);
	} else
		SetWindowText(hwnd,TITLE_TEXT);
}


static void SetPresetMenu()
{
	HMENU hmenu;
	int Count,i;

	hmenu=GetSubMenu(GetSubMenu(hmenuMain,VIEW_MENU),VIEW_PRESET_MENU);
	Count=GetMenuItemCount(hmenu);
	for (i=Count-1;i>=0;i--)
		DeleteMenu(hmenu,i,MF_BYPOSITION);
	for (i=0;i<NUM_PRESETS;i++) {
		TCHAR szText[4+MAX_PRESET_NAME];

		wsprintf(szText,TEXT("&%d: %s"),i+1,PresetList[i].szName);
		AppendMenu(hmenu,MFT_STRING,CM_PRESET_FIRST+i,szText);
	}
}


/*
	EBhETCY𒲐
*/
static void AdjustWindowSize(HWND hwnd)
{
	RECT rc,/*rcClient,*/rcWindow,/*rcNewClient,*/rcWorkArea;

	switch (pMediaPlayer->GetMediaType()) {
	case MEDIA_VIDEO:
		pMediaPlayer->GetDestSize(&rc);
		if (rc.right<320)
			rc.right=320;
		break;
	case MEDIA_SOUND:
		{
/*
			int cxEdge=GetSystemMetrics(SM_CXEDGE);

			GetClientRect(hwnd,&rc);
			rc.left+=cxEdge;
			rc.right-=cxEdge;
*/
			rc.left=rc.top=0;
			rc.right=320;
			rc.bottom=0;
		}
		break;
	default:
		rc.left=rc.top=0;
		rc.right=320;
		rc.bottom=240;
		break;
	}
	pMediaPlayer->AdjustRect(&rc);
	if (fShowToolbar) {
		RECT rcToolbar;

		GetWindowRect(hwndToolbar,&rcToolbar);
		rc.top-=rcToolbar.bottom-rcToolbar.top;
	}
	if (fShowStatusBar) {
		RECT rcStatusBar;

		GetWindowRect(pStatusBar->GetHandle(),&rcStatusBar);
		rc.bottom+=rcStatusBar.bottom-rcStatusBar.top;
	}
	//rcClient=rc;
	AdjustWindowRectEx(&rc,GetWindowStyle(hwnd),fShowMenuBar,
													GetWindowExStyle(hwnd));
	if (rc.right-rc.left<256)
		rc.right=rc.left+256;
	GetWindowRect(hwnd,&rcWindow);
	OffsetRect(&rc,rcWindow.left-rc.left,rcWindow.top-rc.top);
	// fXNgbv͈̔͂擾
	if (!SystemParametersInfo(SPI_GETWORKAREA,0,&rcWorkArea,0)){
		rcWorkArea.left=rcWorkArea.top=0;
		rcWorkArea.right=GetSystemMetrics(SM_CXSCREEN);
		rcWorkArea.bottom=GetSystemMetrics(SM_CYSCREEN);
	}
	// EBhEʓɎ܂悤ɂ
	if (rc.right>rcWorkArea.right) {
		rc.left-=rc.right-rcWorkArea.right;
		rc.right=rcWorkArea.right;
		if (rc.left<rcWorkArea.left) {
			rc.right+=rcWorkArea.left-rc.left;
			rc.left=rcWorkArea.left;
			if (rc.right>rcWorkArea.right)
				rc.right=rcWorkArea.right;
		}
	}
	if (rc.bottom>rcWorkArea.bottom) {
		rc.top-=rc.bottom-rcWorkArea.bottom;
		rc.bottom=rcWorkArea.bottom;
		if (rc.top<rcWorkArea.top) {
			rc.bottom+=rcWorkArea.top-rc.top;
			rc.top=rcWorkArea.top;
			if (rc.bottom>rcWorkArea.bottom)
				rc.bottom=rcWorkArea.bottom;
		}
	}
	MoveWindow(hwnd,rc.left,rc.top,rc.right-rc.left,rc.bottom-rc.top,TRUE);
#if 0
	// j[̒iɂăNCAg̈̃TCYς̂ŁAĐݒs
	OffsetRect(&rcClient,0,-rcClient.top);
	GetClientRect(hwnd,&rcNewClient);
	if (rcNewClient.bottom<rcClient.bottom)
		MoveWindow(hwnd,rc.left,rc.top,rc.right-rc.left,
				(rc.bottom-rc.top)+(rcClient.bottom-rcNewClient.bottom),TRUE);
#endif
}


/*
	c[o[쐬
*/
static HWND CreateMainToolbar(HWND hwndParent)
{
	static TBBUTTON tbb[] = {
		{0,	CM_OPEN,		TBSTATE_ENABLED,
								BTNS_BUTTON | BTNS_DROPDOWN,	{0,0},	0,0},
		{0,	0,				TBSTATE_ENABLED,	BTNS_SEP,		{0,0},	0,0},
		{1,	CM_PLAYLIST,	TBSTATE_ENABLED,	BTNS_CHECK,		{0,0},	0,0},
		{2,	CM_PREVFILE,	0,	BTNS_BUTTON | BTNS_DROPDOWN,	{0,0},	0,0},
		{3,	CM_NEXTFILE,	0,	BTNS_BUTTON | BTNS_DROPDOWN,	{0,0},	0,0},
		{4,	CM_REPEATPLAYLIST,	TBSTATE_ENABLED,	BTNS_CHECK,	{0,0},	0,0},
		{0,	0,				TBSTATE_ENABLED,	BTNS_SEP,		{0,0},	0,0},
		{5,	CM_ZOOMIN,		0,	BTNS_BUTTON | BTNS_DROPDOWN,	{0,0},	0,0},
		{6,	CM_ZOOMOUT,		0,					BTNS_BUTTON,	{0,0},	0,0},
		{0,	0,				TBSTATE_ENABLED,	BTNS_SEP,		{0,0},	0,0},
		{7,	CM_FULLSCREEN,	0,					BTNS_BUTTON,	{0,0},	0,0},
		{8,	CM_ALWAYSONTOP,	TBSTATE_ENABLED,	BTNS_CHECK,		{0,0},	0,0},
		{0,	0,				TBSTATE_ENABLED,	BTNS_SEP,		{0,0},	0,0},
		{9,	CM_OPTIONS,		TBSTATE_ENABLED,	BTNS_BUTTON,	{0,0},	0,0},
	};
	HWND hwnd;
	int IconWidth,IconHeight;

	if (MainToolbarInfo.himl!=NULL) {
		ImageList_GetIconSize(MainToolbarInfo.himl,&IconWidth,&IconHeight);
	} else {
		IconWidth=16;
		IconHeight=16;
	}
	hwnd=CreateToolbarEx(hwndParent,WS_CHILD | (fShowToolbar?WS_VISIBLE:0) |
								TBSTYLE_TOOLTIPS | TBSTYLE_FLAT,IDC_TOOLBAR,
					0,NULL,0,NULL,0,0,0,IconWidth,IconHeight,sizeof(TBBUTTON));
	if (hwnd==NULL)
		return NULL;
	SendMessage(hwnd,TB_SETEXTENDEDSTYLE,0,TBSTYLE_EX_DRAWDDARROWS);
	SendMessage(hwnd,TB_SETIMAGELIST,0,(LPARAM)
		(MainToolbarInfo.himl!=NULL?MainToolbarInfo.himl:
			ImageList_LoadImage(hInst,MAKEINTRESOURCE(IDB_TOOLBAR),16,1,
							RGB(255,0,255),IMAGE_BITMAP,LR_CREATEDIBSECTION)));
	if (MainToolbarInfo.himlHot!=NULL)
		SendMessage(hwnd,TB_SETHOTIMAGELIST,0,(LPARAM)MainToolbarInfo.himlHot);
	if (MainToolbarInfo.himlDisabled!=NULL)
		SendMessage(hwnd,TB_SETDISABLEDIMAGELIST,0,
										(LPARAM)MainToolbarInfo.himlDisabled);
	SendMessage(hwnd,TB_ADDBUTTONS,numberof(tbb),(LPARAM)tbb);
	if (!fShowTipHelp)
		ShowToolbarTipHelp(hwnd,FALSE);
	SendMessage(hwnd,TB_CHECKBUTTON,CM_PLAYLIST,fShowPlaylist);
	SendMessage(hwnd,TB_CHECKBUTTON,CM_REPEATPLAYLIST,fRepeatPlaylist);
	SendMessage(hwnd,TB_CHECKBUTTON,CM_ALWAYSONTOP,fAlwaysOnTop);
	return hwnd;
}


static bool LoadToolbarImages(const ThemePlugin *pPlugin,int Type,
													ToolbarImageInfo *pInfo)
{
	LPThemeToolbarInfo ToolbarInfo;

	ToolbarInfo.Type=Type;
	pPlugin->GetToolbarInfo(&ToolbarInfo);
	pInfo->himl=pPlugin->LoadImageList(Type,ThemePlugin::BUTTON_STAT_NORMAL);
	if ((ToolbarInfo.Flags&LP_TOOLBAR_FLAG_HOT)!=0)
		pInfo->himlHot=
				pPlugin->LoadImageList(Type,ThemePlugin::BUTTON_STAT_HOT);
	if ((ToolbarInfo.Flags&LP_TOOLBAR_FLAG_DISABLED)!=0)
		pInfo->himlDisabled=
				pPlugin->LoadImageList(Type,ThemePlugin::BUTTON_STAT_DISABLED);
	return true;
}


#if 0
/*
	Xe[^Xo[쐬
*/
static HWND CreateMainStatusBar(HWND hwndParent)
{
	static const int nParts[]={80,184,-1};
	HWND hwnd;

	hwnd=CreateStatusWindow(WS_CHILD | (fShowStatusBar?WS_VISIBLE:0) |
							SBARS_SIZEGRIP,TEXT(""),hwndParent,IDC_STATUSBAR);
	if (hwnd==NULL)
		return NULL;
	SendMessage(hwnd,SB_SETPARTS,numberof(nParts),(LPARAM)nParts);
	return hwnd;
}
#endif


static BOOL OnCreate(HWND hwnd,LPCREATESTRUCT /*pcs*/)
{
	// c[o[̍쐬
	hwndToolbar=CreateMainToolbar(hwnd);
	// Xe[^Xo[̍쐬
	//hwndStatusBar=CreateMainStatusBar(hwnd);
	TCHAR szText[64];
	pStatusBar=new LPStatusBar(hwnd,IDC_STATUSBAR);
	pStatusBar->SetSize(LPStatusBar::PART_MODE,TEXT("ꎞ~"));
	pStatusBar->SetSize(LPStatusBar::PART_SIZE,TEXT("1000 x 1000 (1000%%)"));
	FormatTime(szText,0,TRUE);
	lstrcat(szText,TEXT(" / "));
	FormatTime(szText+lstrlen(szText),0,TRUE);
	pStatusBar->SetSize(LPStatusBar::PART_TIME,szText);
	/*
	pStatusBar->SetTextColor(ColorInfo_GetColor(
							ColorSchemeList[COLOR_SCHEME_STATUSTEXT].Color));
	pStatusBar->SetBackgroundColor(ColorInfo_GetColor(
								ColorSchemeList[COLOR_SCHEME_STATUSBG].Color));
	*/
	if (fShowStatusBar)
		pStatusBar->Show(TRUE);
	// TCYpObv̍쐬
	hwndSizeGrip=CreateWindowEx(0,TEXT("SCROLLBAR"),NULL,
		WS_CHILD | (fShowStatusBar?WS_VISIBLE:0) | WS_CLIPSIBLINGS |
									SBS_SIZEGRIP | SBS_SIZEBOXBOTTOMRIGHTALIGN,
										0,0,100,100,hwnd,(HMENU)0,hInst,NULL);
	SetWindowPos(hwndSizeGrip,HWND_BOTTOM,0,0,0,0,SWP_NOMOVE | SWP_NOSIZE);
	// fBAEBhE̍쐬
	pMediaPlayer=new MediaPlayer;
	if (PlayBarToolbarInfo.himl!=NULL)
		pMediaPlayer->SetPlayBarImage(PlayBarToolbarInfo.himl,
				PlayBarToolbarInfo.himlHot,PlayBarToolbarInfo.himlDisabled);
	pMediaPlayer->SetMediaDevice(MediaDevice);
	pMediaPlayer->Create(hwnd,IDC_MEDIA,hInst);
	pMediaPlayer->ShowPlayBar(fShowPlayBar);
	pMediaPlayer->ShowSeekBar(fShowSeekBar);
	pMediaPlayer->SetBackwardSpeed(nBackwardSpeed);
	pMediaPlayer->SetForwardSpeed(nForwardSpeed);
	pMediaPlayer->SetRepeat(fPlayRepeat);
	pMediaPlayer->FitToWindow(fFitToWindow);
	/*
	pMediaPlayer->SetBackgroundColor(
		ColorInfo_GetColor(ColorSchemeList[COLOR_SCHEME_BACKGROUND].Color));
	*/
	pMediaPlayer->FullScreenFitToDisplay(fFullScreenFitToDisplay);
	pMediaPlayer->FullScreenHideCursor(fFullScreenHideCursor);
	pMediaPlayer->SetVolume(DefaultVolume);
	// vCXg̍쐬
	pPlaylist=new Playlist;
	if (PlaylistToolbarInfo.himl!=NULL)
		pPlaylist->SetToolbarImage(PlaylistToolbarInfo.himl,
				PlaylistToolbarInfo.himlHot,PlaylistToolbarInfo.himlDisabled);
	pPlaylist->Create(fPlaylistKeepForeground?hwnd:NULL,hInst);
	pPlaylist->FindSubDirectory(fFindSubDirectory);
	pPlaylist->SetFileMasks(szFileMasks);
	pPlaylist->SetMaxHistory(MaxPlaylistHistory);
	pPlaylist->SetFileNameType(PlaylistFileNameType);
	/*
	pPlaylist->SetColors(
		ColorInfo_GetColor(ColorSchemeList[COLOR_SCHEME_PLAYLISTTEXT].Color),
		ColorInfo_GetColor(ColorSchemeList[COLOR_SCHEME_PLAYLISTBG].Color),
		ColorInfo_GetColor(ColorSchemeList[COLOR_SCHEME_PLAYLISTSELTEXT].Color),
		ColorInfo_GetColor(ColorSchemeList[COLOR_SCHEME_PLAYLISTSELBG].Color));
	*/
	if ((nPlaylistLeft!=CW_USEDEFAULT && nPlaylistTop!=CW_USEDEFAULT)
			|| (nPlaylistWidth!=CW_USEDEFAULT
										&& nPlaylistHeight!=CW_USEDEFAULT)) {
		SetWindowPos(pPlaylist->GetHandle(),NULL,nPlaylistLeft,nPlaylistTop,
			nPlaylistWidth,nPlaylistHeight,
			SWP_NOACTIVATE | SWP_NOZORDER |
			(nPlaylistLeft==CW_USEDEFAULT || nPlaylistTop==CW_USEDEFAULT?
																SWP_NOMOVE:0) |
			(nPlaylistWidth==CW_USEDEFAULT || nPlaylistHeight==CW_USEDEFAULT?
																SWP_NOSIZE:0));
	}
	ApplyColorScheme();
	// EBhETCY𒲐
	if (nWindowWidth==CW_USEDEFAULT || nWindowHeight==CW_USEDEFAULT)
		AdjustWindowSize(hwnd);
	// N̓EBhE𒆐Sɕ\
	if (nWindowLeft==CW_USEDEFAULT || nWindowTop==CW_USEDEFAULT) {
		RECT rcWorkArea,rcWindow;
		int x,y;

		SystemParametersInfo(SPI_GETWORKAREA,0,&rcWorkArea,0);
		GetWindowRect(hwnd,&rcWindow);
		x=rcWindow.left;
		y=rcWindow.top;
		if (nWindowLeft==CW_USEDEFAULT)
			x=rcWorkArea.left+((rcWorkArea.right-rcWorkArea.left)-
											(rcWindow.right-rcWindow.left))/2;
		if (nWindowTop==CW_USEDEFAULT)
			y=rcWorkArea.top+((rcWorkArea.bottom-rcWorkArea.top)-
											(rcWindow.bottom-rcWindow.top))/2;
		MoveWindow(hwnd,x,y,rcWindow.right-rcWindow.left,
										rcWindow.bottom-rcWindow.top,FALSE);
	}
	return TRUE;
}


static void OnSize(HWND /*hwnd*/,UINT /*uState*/,int nWidth,int nHeight)
{
	int nTop,nBottom;
	RECT rc;

	if (fShowToolbar) {
		SendMessage(hwndToolbar,TB_AUTOSIZE,0,0);
		GetWindowRect(hwndToolbar,&rc);
		nTop=rc.bottom-rc.top;
	} else
		nTop=0;
	nBottom=nHeight;
	if (fShowStatusBar) {
		GetWindowRect(hwndSizeGrip,&rc);
		rc.right-=rc.left;
		rc.bottom-=rc.top;
		nBottom-=pStatusBar->Height();
		MoveWindow(pStatusBar->GetHandle(),0,nBottom,
									nWidth-rc.right,pStatusBar->Height(),TRUE);
		MoveWindow(hwndSizeGrip,nWidth-rc.right,nHeight-rc.bottom,
													rc.right,rc.bottom,TRUE);
	}
	MoveWindow(pMediaPlayer->GetHandle(),0,nTop,nWidth,nBottom-nTop,TRUE);
}


static void SetBarMenuStatus(HMENU hmenu)
{
	CheckMenuItem(hmenu,CM_MENUBAR,
					MF_BYCOMMAND | (fShowMenuBar?MFS_CHECKED:MFS_UNCHECKED));
	CheckMenuItem(hmenu,CM_TOOLBAR,
					MF_BYCOMMAND | (fShowToolbar?MFS_CHECKED:MFS_UNCHECKED));
	CheckMenuItem(hmenu,CM_STATUSBAR,
					MF_BYCOMMAND | (fShowStatusBar?MFS_CHECKED:MFS_UNCHECKED));
	CheckMenuItem(hmenu,CM_PLAYBAR,
					MF_BYCOMMAND | (fShowPlayBar?MFS_CHECKED:MFS_UNCHECKED));
	CheckMenuItem(hmenu,CM_SEEKBAR,
					MF_BYCOMMAND | (fShowSeekBar?MFS_CHECKED:MFS_UNCHECKED));
}


static void SetFullScreenBarMenuStatus(HMENU hmenu)
{
	CheckMenuItem(hmenu,CM_PLAYBAR_FULLSCREEN,
			MF_BYCOMMAND | (fShowFullScreenPlayBar?MFS_CHECKED:MFS_UNCHECKED));
	CheckMenuItem(hmenu,CM_SEEKBAR_FULLSCREEN,
			MF_BYCOMMAND | (fShowFullScreenSeekBar?MFS_CHECKED:MFS_UNCHECKED));
}


/*
	|bvAbvj[\
*/
static void PopupMenu(HWND hwnd,int x,int y)
{
	if (!fShowMenuBar) {
		HMENU hmenu;
		int i;
		TCHAR szText[32];

		hmenu=CreatePopupMenu();
		for (i=FILE_MENU;i<=HELP_MENU;i++) {
			GetMenuString(hmenuMain,i,szText,numberof(szText),MF_BYPOSITION);
			AppendMenu(hmenu,MF_POPUP,(UINT_PTR)GetSubMenu(hmenuMain,i),
																	szText);
		}
		TrackPopupMenu(hmenu,TPM_RIGHTBUTTON,x,y,0,hwnd,NULL);
		for (i=HELP_MENU;i>=FILE_MENU;i--)
			RemoveMenu(hmenu,i,MF_BYPOSITION);
		DestroyMenu(hmenu);
		return;
	}
	if (!pMediaPlayer->IsOpen()) {
		HMENU hmenu,hmenuPopup,hmenuFile;
		TCHAR szText[64];
		UINT uState;

		hmenu=LoadMenu(hInst,MAKEINTRESOURCE(IDM_POPUP1));
		if (hmenu==NULL)
			return;
		hmenuPopup=GetSubMenu(hmenu,0);
		SetBarMenuStatus(hmenuPopup);
		SetMenuAccel(hmenuPopup);
		// t@CƃtH_ǉ
		hmenuFile=GetSubMenu(hmenuMain,0);
		GetMenuString(hmenuFile,FILE_HISTORY_MENU,szText,numberof(szText),
															MF_BYPOSITION);
		uState=GetMenuState(hmenuFile,FILE_HISTORY_MENU,MF_BYPOSITION);
		InsertMenu(hmenuPopup,2,MF_BYPOSITION | MF_POPUP | (uState&MFS_GRAYED),
				(UINT_PTR)GetSubMenu(hmenuFile,FILE_HISTORY_MENU),szText);
		GetMenuString(hmenuFile,DIR_HISTORY_MENU,szText,numberof(szText),
																MF_BYPOSITION);
		uState=GetMenuState(hmenuFile,DIR_HISTORY_MENU,MF_BYPOSITION);
		InsertMenu(hmenuPopup,3,MF_BYPOSITION | MF_POPUP | (uState&MFS_GRAYED),
					(UINT_PTR)GetSubMenu(hmenuFile,DIR_HISTORY_MENU),szText);
		TrackPopupMenu(hmenuPopup,TPM_RIGHTBUTTON,x,y,0,hwnd,NULL);
		RemoveMenu(hmenuPopup,3,MF_BYPOSITION);
		RemoveMenu(hmenuPopup,2,MF_BYPOSITION);
		DestroyMenu(hmenu);
	} else {
		HMENU hmenu,hmenuPopup,hmenuBar;
		int i,Count;
		TCHAR szText[64];

		hmenu=LoadMenu(hInst,MAKEINTRESOURCE(IDM_POPUP2));
		if (hmenu==NULL)
			return;
		hmenuPopup=GetSubMenu(hmenu,0);
		Count=GetMenuItemCount(hmenuPopup);
		for (i=0;i<Count;i++) {
			UINT State;

			State=GetMenuState(hmenuMain,GetMenuItemID(hmenuPopup,i),
																MF_BYCOMMAND);
			if ((State&MFT_SEPARATOR)==0 && (State&MF_POPUP)==0) {
				if ((State&MFS_GRAYED)!=0)
					EnableMenuItem(hmenuPopup,i,MF_BYPOSITION | MFS_GRAYED);
				if ((State&MFS_CHECKED)!=0)
					CheckMenuItem(hmenuPopup,i,MF_BYPOSITION | MFS_CHECKED);
			}
		}
		if (!pMediaPlayer->IsFullScreen()) {
			hmenuBar=LoadMenu(hInst,MAKEINTRESOURCE(IDM_BAR));
			SetBarMenuStatus(hmenuBar);
		} else {
			hmenuBar=LoadMenu(hInst,MAKEINTRESOURCE(IDM_BAR_FULLSCREEN));
			SetFullScreenBarMenuStatus(hmenuBar);
		}
		GetMenuString(hmenuBar,0,szText,numberof(szText),MF_BYPOSITION);
		AppendMenu(hmenuPopup,MF_POPUP,(UINT_PTR)GetSubMenu(hmenuBar,0),
																	szText);
		SetMenuAccel(hmenuPopup);
		TrackPopupMenu(hmenuPopup,TPM_RIGHTBUTTON,x,y,0,hwnd,NULL);
		DestroyMenu(hmenu);
	}
}


static void BarPopupMenu()
{
	HMENU hmenu,hmenuPopup;
	POINT pt;

	if (!pMediaPlayer->IsFullScreen()) {
		hmenu=LoadMenu(hInst,MAKEINTRESOURCE(IDM_BAR));
		SetBarMenuStatus(hmenu);
	} else {
		hmenu=LoadMenu(hInst,MAKEINTRESOURCE(IDM_BAR_FULLSCREEN));
		SetFullScreenBarMenuStatus(hmenu);
	}
	hmenuPopup=GetSubMenu(hmenu,0);
	SetMenuAccel(hmenuPopup);
	GetCursorPos(&pt);
	TrackPopupMenu(hmenuPopup,TPM_RIGHTBUTTON,pt.x,pt.y,0,hwndMain,NULL);
	DestroyMenu(hmenu);
}


static LRESULT OnNotify(HWND hwnd,int /*nIDFrom*/,LPNMHDR pnmhdr)
{
	switch (pnmhdr->code) {
	case NM_RCLICK:
		BarPopupMenu();
		break;
	case TTN_NEEDTEXT:
		{
			LPTOOLTIPTEXT pttt=(LPTOOLTIPTEXT)pnmhdr;

			pttt->lpszText=(LPTSTR)(IDS_SHORTCMBASE+pttt->hdr.idFrom);
			pttt->hinst=hInst;
		}
		break;
#if 0	// c[o[̃{^̐\
	case TTN_SHOW:
		if (fShowStatusBar) {
			TCHAR szText[128];

			if (LoadString(hInst,pnmhdr->idFrom,szText,numberof(szText))) {
				SendMessage(hwndStatusBar,SB_SIMPLE,TRUE,0);
				SendMessage(hwndStatusBar,SB_SETTEXT,SBT_NOBORDERS | 255,
															(LPARAM)szText);
			}
		}
		break;
	case TTN_POP:
		if (fShowStatusBar) {
			SendMessage(hwndStatusBar,SB_SETTEXT,255,(LPARAM)TEXT(""));
			SendMessage(hwndStatusBar,SB_SIMPLE,FALSE,0);
		}
		break;
#endif
	case TBN_DROPDOWN:
		{
			LPNMTOOLBAR pnmtb=(LPNMTOOLBAR)pnmhdr;
			RECT rc;
			POINT pt;
			HMENU hmenu;

			SendMessage(pnmtb->hdr.hwndFrom,TB_GETITEMRECT,
				SendMessage(pnmtb->hdr.hwndFrom,TB_COMMANDTOINDEX,pnmtb->iItem,
															0),(LPARAM)&rc);
			pt.x=rc.left;
			pt.y=rc.bottom;
			ClientToScreen(pnmtb->hdr.hwndFrom,&pt);
			switch (pnmtb->iItem) {
			case CM_OPEN:
				hmenu=GetSubMenu(GetSubMenu(hmenuMain,FILE_MENU),
						fUseDirHistoryMenu?DIR_HISTORY_MENU:FILE_HISTORY_MENU);
				if (GetMenuItemCount(hmenu)>0)
					TrackPopupMenu(hmenu,TPM_LEFTBUTTON,pt.x,pt.y,0,hwnd,NULL);
				break;
			case CM_PREVFILE:
			case CM_NEXTFILE:
				{
					int Cur,Max,i;
					bool fEllipsis=false;

					if (fOpenFromPlaylist) {
						Cur=pPlaylist->GetPlaying();
						if ((pnmtb->iItem==CM_PREVFILE && Cur<=0)
								|| (pnmtb->iItem==CM_NEXTFILE
											&& Cur==pPlaylist->GetCount()-1))
							break;
						FileListMenuCurFile=Cur;
						hmenu=CreatePopupMenu();
						if (pnmtb->iItem==CM_PREVFILE) {
							if (Cur<=MaxFileListMenuFiles) {
								Max=Cur;
							} else {
								Max=MaxFileListMenuFiles;
								fEllipsis=true;
							}
							for (i=0;i<Max;i++)
								AppendMenu(hmenu,MF_OWNERDRAW,
									CM_PREVFILE_FIRST+i,(LPCTSTR)(Cur-1-i));
						} else {
							Max=pPlaylist->GetCount()-1-Cur;
							if (Max>MaxFileListMenuFiles) {
								Max=MaxFileListMenuFiles;
								fEllipsis=true;
							}
							for (i=0;i<Max;i++)
								AppendMenu(hmenu,MF_OWNERDRAW,
									CM_NEXTFILE_FIRST+i,(LPCTSTR)(Cur+1+i));
						}
					} else {
						TCHAR szFileName[MAX_PATH],*pszFile;

						if (!CreateDirFileList())
							break;
						pMediaPlayer->GetFileName(szFileName,
														numberof(szFileName));
						pszFile=GetFileNamePtr(szFileName);
						Cur=pDirFileList->GetFileIndex(pszFile);
						if (Cur<0) {
							if (!RefreshDirFileList())
								break;
							Cur=pDirFileList->GetFileIndex(pszFile);
							if (Cur<0) {
								if (pDirFileList->AddFile(pszFile)) {
									pDirFileList->Sort(DirSortType,
														fDirSortDescending);
									Cur=pDirFileList->GetFileIndex(pszFile);
								}
								if (Cur<0)
									break;
							}
						}
						if ((pnmtb->iItem==CM_PREVFILE && Cur==0)
								|| (pnmtb->iItem==CM_NEXTFILE
										&& Cur==pDirFileList->GetCount()-1))
							break;
						FileListMenuCurFile=Cur;
						hmenu=CreatePopupMenu();
						if (pnmtb->iItem==CM_PREVFILE) {
							if (Cur<=MaxFileListMenuFiles) {
								Max=Cur;
							} else {
								Max=MaxFileListMenuFiles;
								fEllipsis=true;
							}
							for (i=0;i<Max;i++)
								AppendMenu(hmenu,MF_OWNERDRAW,
									CM_PREVFILE_FIRST+i,(LPCTSTR)(Cur-1-i));
						} else {
							Max=pDirFileList->GetCount()-1-Cur;
							if (Max>MaxFileListMenuFiles) {
								Max=MaxFileListMenuFiles;
								fEllipsis=true;
							}
							for (i=0;i<Max;i++)
								AppendMenu(hmenu,MF_OWNERDRAW,
									CM_NEXTFILE_FIRST+i,(LPCTSTR)(Cur+1+i));
						}
					}
					if (fEllipsis)
						AppendMenu(hmenu,MF_OWNERDRAW | MFS_GRAYED,0,0);
					NONCLIENTMETRICS ncm;
					ncm.cbSize=sizeof(NONCLIENTMETRICS);
					SystemParametersInfo(SPI_GETNONCLIENTMETRICS,
											sizeof(NONCLIENTMETRICS),&ncm,0);
					hfontMenu=CreateFontIndirect(&ncm.lfMenuFont);
					TrackPopupMenu(hmenu,TPM_LEFTBUTTON,pt.x,pt.y,0,hwnd,NULL);
					DestroyMenu(hmenu);
					DeleteObject(hfontMenu);
				}
				break;
			case CM_ZOOMIN:
				TrackPopupMenu(GetSubMenu(GetSubMenu(hmenuMain,VIEW_MENU),
															VIEW_ZOOM_MENU),
										TPM_LEFTBUTTON,pt.x,pt.y,0,hwnd,NULL);
				break;
			}
		}
		break;
	}
	return 0;
}


#define MENU_MARGIN 2
#define MENU_ICON_TEXT_MARGIN 4


static void OnMeasureItem(HWND hwnd,MEASUREITEMSTRUCT *pmis)
{
	HDC hdc;
	HFONT hfontOld;
	TCHAR szFileName[MAX_PATH],*pszText;
	SIZE sz;
	TEXTMETRIC tm;

	hdc=GetDC(hwnd);
	hfontOld=SelectFont(hdc,hfontMenu);
	if (pmis->itemID!=0) {
		if (fOpenFromPlaylist) {
			pPlaylist->GetDisplayFileName(pmis->itemData,szFileName);
			pszText=szFileName;
		} else {
			pDirFileList->GetFile(pmis->itemData,szFileName);
			pszText=GetFileNamePtr(szFileName);
		}
	} else
		pszText=TEXT("...");
	GetTextExtentPoint32(hdc,pszText,lstrlen(pszText),&sz);
	GetTextMetrics(hdc,&tm);
	SelectFont(hdc,hfontOld);
	ReleaseDC(hwnd,hdc);
	pmis->itemWidth=sz.cx+MENU_MARGIN*2;
	if (pmis->itemID!=0)
		pmis->itemWidth+=GetSystemMetrics(SM_CXSMICON)+MENU_ICON_TEXT_MARGIN;
	pmis->itemHeight=max(GetSystemMetrics(SM_CYSMICON),(int)tm.tmHeight)+
																MENU_MARGIN*2;
}


static void OnDrawItem(HWND /*hwnd*/,const DRAWITEMSTRUCT *pdis)
{
	TCHAR szFileName[MAX_PATH],*pszText;
	HICON hico;
	ColorSchemeType TextColor,BkColor;
	COLORREF crOldText;
	int OldBkMode;
	HBRUSH hbr;
	int IconWidth,IconHeight;
	HFONT hfontOld;
	RECT rc;

	if (pdis->itemID!=0) {
		if (fOpenFromPlaylist) {
			pPlaylist->GetDisplayFileName(pdis->itemData,szFileName);
			pszText=szFileName;
			hico=pPlaylist->GetFileIcon(pdis->itemData);
		} else {
			pDirFileList->GetFile(pdis->itemData,szFileName);
			pszText=GetFileNamePtr(szFileName);
			hico=pDirFileList->GetFileIcon(pdis->itemData);
		}
	} else {
		pszText=TEXT("...");
		hico=NULL;
	}
	if ((pdis->itemState&ODS_SELECTED)==0
										|| (pdis->itemState&ODS_DISABLED)!=0) {
		if (fOpenFromPlaylist && pPlaylist->IsFilePlayed(pdis->itemData))
			TextColor=COLOR_SCHEME_PLAYLIST_PLAYEDTEXT;
		else
			TextColor=COLOR_SCHEME_PLAYLISTTEXT;
		BkColor=COLOR_SCHEME_PLAYLISTBG;
	} else {
		TextColor=COLOR_SCHEME_PLAYLISTSELTEXT;
		BkColor=COLOR_SCHEME_PLAYLISTSELBG;
	}
	crOldText=SetTextColor(pdis->hDC,
						ColorInfo_GetColor(ColorSchemeList[TextColor].Color));
	OldBkMode=SetBkMode(pdis->hDC,TRANSPARENT);
	hbr=CreateSolidBrush(ColorInfo_GetColor(ColorSchemeList[BkColor].Color));
	FillRect(pdis->hDC,&pdis->rcItem,hbr);
	DeleteObject(hbr);
	IconWidth=GetSystemMetrics(SM_CXSMICON);
	IconHeight=GetSystemMetrics(SM_CYSMICON);
	hfontOld=SelectFont(pdis->hDC,hfontMenu);
	rc.left=pdis->rcItem.left+MENU_MARGIN+IconWidth+MENU_ICON_TEXT_MARGIN;
	rc.top=pdis->rcItem.top+MENU_MARGIN;
	rc.bottom=pdis->rcItem.bottom-MENU_MARGIN;
	rc.right=pdis->rcItem.right-MENU_MARGIN;
	DrawText(pdis->hDC,pszText,-1,&rc,
						DT_LEFT | DT_SINGLELINE | DT_VCENTER | DT_NOPREFIX);
	SelectFont(pdis->hDC,hfontOld);
	if (hico!=NULL)
		DrawIconEx(pdis->hDC,pdis->rcItem.left+MENU_MARGIN,
			pdis->rcItem.top+(pdis->rcItem.bottom-pdis->rcItem.top-IconHeight)/2,
								hico,IconWidth,IconHeight,0,NULL,DI_NORMAL);
	SetTextColor(pdis->hDC,crOldText);
	SetBkMode(pdis->hDC,OldBkMode);
}


static void SetZoomMenu()
{
	HMENU hmenu;
	int Zoom;

	hmenu=GetSubMenu(GetSubMenu(hmenuMain,VIEW_MENU),VIEW_ZOOM_MENU);
	Zoom=pMediaPlayer->GetZoom();
	for (int i=0;i<numberof(ZoomRateList);i++) {
		CheckMenuItem(hmenu,CM_ZOOM_FIRST+i,
			MF_BYCOMMAND | (ZoomRateList[i]==Zoom?MFS_CHECKED:MFS_UNCHECKED));
	}
}


/*
	j[̏Ԃݒ肷
*/
static void SetMenuBar()
{
	HMENU hmenu=hmenuMain;
	BOOL fOpen=pMediaPlayer->IsOpen();
	MediaType Media=pMediaPlayer->GetMediaType();

	EnableMenuItem(hmenu,CM_CLOSE,
								MF_BYCOMMAND | (fOpen?MFS_ENABLED:MFS_GRAYED));
	EnableMenuItem(hmenu,CM_REFRESH,
		MF_BYCOMMAND | (fOpen && !fOpenFromPlaylist?MFS_ENABLED:MFS_GRAYED));
	EnableMenuItem(hmenu,CM_ADDTOPLAYLIST,
		MF_BYCOMMAND | (fOpen && !fOpenFromPlaylist?MFS_ENABLED:MFS_GRAYED));
	EnableMenuItem(hmenu,CM_FOLDERPLAYLIST,
		MF_BYCOMMAND | (fOpen && !fOpenFromPlaylist?MFS_ENABLED:MFS_GRAYED));
	EnableMenuItem(hmenu,CM_PLAY_PAUSE,
								MF_BYCOMMAND | (fOpen?MFS_ENABLED:MFS_GRAYED));
	EnableMenuItem(hmenu,CM_STOP,MF_BYCOMMAND | MFS_GRAYED);
	CheckMenuItem(hmenu,CM_STOP,MF_BYCOMMAND | MFS_UNCHECKED);
	EnableMenuItem(hmenu,CM_PREVFRAME,
		MF_BYCOMMAND | (fOpen && Media==MEDIA_VIDEO?MFS_ENABLED:MFS_GRAYED));
	EnableMenuItem(hmenu,CM_NEXTFRAME,
		MF_BYCOMMAND | (fOpen && Media==MEDIA_VIDEO?MFS_ENABLED:MFS_GRAYED));
	EnableMenuItem(hmenu,CM_BACKWARD,
								MF_BYCOMMAND | (fOpen?MFS_ENABLED:MFS_GRAYED));
	EnableMenuItem(hmenu,CM_FORWARD,
								MF_BYCOMMAND | (fOpen?MFS_ENABLED:MFS_GRAYED));
	EnableMenuItem(hmenu,CM_HOME,
								MF_BYCOMMAND | (fOpen?MFS_ENABLED:MFS_GRAYED));
	EnableMenuItem(hmenu,CM_END,MF_BYCOMMAND | (fOpen?MFS_ENABLED:MFS_GRAYED));
	EnableMenuItem(GetSubMenu(hmenu,PLAY_MENU),PLAY_VOLUME_MENU,
		MF_BYPOSITION | (fOpen && pMediaPlayer->HasAudio()?MFS_ENABLED:MFS_GRAYED));
	EnableMenuItem(hmenu,CM_PREVFILE,MF_BYCOMMAND |
						(fOpen/* && fOpenFromPlaylist*/?MFS_ENABLED:MFS_GRAYED));
	SendMessage(hwndToolbar,TB_ENABLEBUTTON,CM_PREVFILE,
												fOpen/* && fOpenFromPlaylist*/);
	EnableMenuItem(hmenu,CM_NEXTFILE,MF_BYCOMMAND |
						(fOpen/* && fOpenFromPlaylist*/?MFS_ENABLED:MFS_GRAYED));
	SendMessage(hwndToolbar,TB_ENABLEBUTTON,CM_NEXTFILE,
												fOpen/* && fOpenFromPlaylist*/);
	EnableMenuItem(hmenu,CM_FIRSTFILE,MF_BYCOMMAND |
						(fOpen/* && fOpenFromPlaylist*/?MFS_ENABLED:MFS_GRAYED));
	EnableMenuItem(hmenu,CM_LASTFILE,MF_BYCOMMAND |
						(fOpen/* && fOpenFromPlaylist*/?MFS_ENABLED:MFS_GRAYED));
	EnableMenuItem(hmenu,CM_ZOOMIN,
		MF_BYCOMMAND | (fOpen && Media==MEDIA_VIDEO?MFS_ENABLED:MFS_GRAYED));
	SendMessage(hwndToolbar,TB_ENABLEBUTTON,CM_ZOOMIN,
												fOpen && Media==MEDIA_VIDEO);
	EnableMenuItem(hmenu,CM_ZOOMOUT,
		MF_BYCOMMAND | (fOpen && Media==MEDIA_VIDEO?MFS_ENABLED:MFS_GRAYED));
	SendMessage(hwndToolbar,TB_ENABLEBUTTON,CM_ZOOMOUT,
												fOpen && Media==MEDIA_VIDEO);
	EnableMenuItem(GetSubMenu(hmenu,VIEW_MENU),VIEW_ZOOM_MENU,
		MF_BYPOSITION | (fOpen && Media==MEDIA_VIDEO?MFS_ENABLED:MFS_GRAYED));
	if (fOpen && Media==MEDIA_VIDEO)
		SetZoomMenu();
	EnableMenuItem(GetSubMenu(hmenu,VIEW_MENU),VIEW_ASPECT_MENU,
		MF_BYPOSITION | (fOpen && Media==MEDIA_VIDEO?MFS_ENABLED:MFS_GRAYED));
	if (fOpen && Media==MEDIA_VIDEO) {
		int XAspect=pMediaPlayer->GetXAspect();
		int YAspect=pMediaPlayer->GetYAspect();
		int i;

		for (i=0;i<numberof(AspectRatioList);i++) {
			if (AspectRatioList[i].x==XAspect && AspectRatioList[i].y==YAspect)
				break;
		}
		if (i==numberof(AspectRatioList))
			i=0;
		CheckMenuRadioItem(hmenu,CM_ASPECT_FIRST,CM_ASPECT_LAST,
											CM_ASPECT_FIRST+i,MF_BYCOMMAND);
	}
	EnableMenuItem(hmenu,CM_FULLSCREEN,
		MF_BYCOMMAND | (fOpen/* && Media==MEDIA_VIDEO*/?MFS_ENABLED:MFS_GRAYED));
	SendMessage(hwndToolbar,TB_ENABLEBUTTON,CM_FULLSCREEN,
												fOpen/* && Media==MEDIA_VIDEO*/);
}


static void SetZoomStatus()
{
	SIZE sz;
	TCHAR szText[32];

	pMediaPlayer->GetSourceSize(&sz);
	wsprintf(szText,TEXT("%d x %d (%d%%)"),sz.cx,sz.cy,pMediaPlayer->GetZoom());
	//SendMessage(hwndStatusBar,SB_SETTEXT,1,(LPARAM)szText);
	pStatusBar->SetText(LPStatusBar::PART_SIZE,szText);
}


static void SetMP3Status()
{
	MP3Info Info;

	if (GetMP3Info(pMediaPlayer->GetFileName(),&Info)) {
		TCHAR szText[32];

		wsprintf(szText,TEXT("%dkbps %dkHz"),
										Info.BitRate/1000,Info.Frequency/1000);
		pStatusBar->SetText(LPStatusBar::PART_SIZE,szText);
	}
}


static void SetTimeStatus(int nPos)
{
	int nLength;
	BOOL fDecimal;
	TCHAR szText[64],szPos[32],szTime[32];

	nLength=pMediaPlayer->GetLength();
	fDecimal=nLength<10000;
	wsprintf(szText,TEXT("%s / %s"),
		FormatTime(szPos,nPos,fDecimal),FormatTime(szTime,nLength,fDecimal));
	//SendMessage(hwndStatusBar,SB_SETTEXT,2,(LPARAM)szText);
	pStatusBar->SetText(LPStatusBar::PART_TIME,szText);
}


static void SetStatusBar()
{
	if (pMediaPlayer->IsOpen()) {
		if (pMediaPlayer->GetMediaType()==MEDIA_VIDEO)
			SetZoomStatus();
		else if (lstrcmpi(GetFileExtensionPtr(pMediaPlayer->GetFileName()),
															TEXT(".mp3"))==0)
			SetMP3Status();
		SetTimeStatus(pMediaPlayer->GetPosition());
	} else {
		int i;

		for (i=0;i<LPStatusBar::NUM_PARTS;i++)
			pStatusBar->SetText(i,TEXT(""));
	}
}


/*
	t@C
*/
static void CloseMediaFile()
{
	if (pMediaPlayer->IsOpen()) {
		PrevMediaType=pMediaPlayer->GetMediaType();
		if (PrevMediaType==MEDIA_VIDEO)
			nPrevZoomRate=pMediaPlayer->GetZoom();
		pMediaPlayer->Close();
		SetWindowTitle(hwndMain);
		SetStatusBar();
		SetMenuBar();
		if (fOpenFromPlaylist)
			pPlaylist->SetPlaying(-1);
	}
}


/*
	t@CJ
*/
static BOOL OpenMediaFile(LPCTSTR pszFileName,bool fPlaylist=false)
{
	HCURSOR hcurOld;
	MediaType Type;

	hcurOld=SetCursor(LoadCursor(NULL,IDC_WAIT));
	GetFileDirectory(szFileOpenDirectory,pszFileName);
	CloseMediaFile();
	if (!pMediaPlayer->Open(pszFileName)) {
		if (pMediaPlayer->IsFullScreen())
			SendMessage(hwndMain,WM_COMMAND,MAKEWPARAM(CM_FULLSCREEN,0),0);
		SetCursor(hcurOld);
		return FALSE;
	}
	SetCursor(hcurOld);
	fOpenFromPlaylist=fPlaylist;
	Type=pMediaPlayer->GetMediaType();
#if 0
	// tXN[ɉt@CJꂽAtXN[
	if (Type==MEDIA_SOUND && pMediaPlayer->IsFullScreen())
		FORWARD_WM_COMMAND(hwndMain,CM_FULLSCREEN,NULL,0,SendMessage);
#endif
	if (fKeepZoomRate && Type==MEDIA_VIDEO)
		pMediaPlayer->SetZoom(nPrevZoomRate);
	if (!pMediaPlayer->IsFullScreen() && ((Type==MEDIA_VIDEO && !fFitToWindow)
						|| (Type==MEDIA_SOUND && PrevMediaType==MEDIA_VIDEO)))
		AdjustWindowSize(hwndMain);
	SetWindowTitle(hwndMain);
	SetStatusBar();
	SetMenuBar();
	FileHistory.Add(pszFileName);
	DirHistory.Add(szFileOpenDirectory);
	if (fPlayOnOpen || fPlaylist)
		pMediaPlayer->Play();
	if (pDirFileList!=NULL
			&& lstrcmpi(pDirFileList->GetDirectory(),szFileOpenDirectory)!=0)
		DeleteDirFileList();
	return TRUE;
}


static void OnDropFiles(HWND hwnd,HDROP hdrop)
{
	int nNumFiles;
	TCHAR szFileName[MAX_PATH];
	DWORD dwAttributes;

	SetForegroundWindow(hwnd);
	nNumFiles=DragQueryFile(hdrop,0xFFFFFFFF,NULL,0);
	if (nNumFiles==1) {
		DragQueryFile(hdrop,0,szFileName,numberof(szFileName));
		dwAttributes=GetFileAttributes(szFileName);
		if (dwAttributes==0xFFFFFFFF) {
			DragFinish(hdrop);
			return;
		}
		// tH_łȂvCXgt@CłȂΊJ
		if ((dwAttributes&FILE_ATTRIBUTE_DIRECTORY)==0
				&& lstrcmpi(GetFileExtensionPtr(szFileName),TEXT(".m3u"))!=0) {
			DragFinish(hdrop);
			OpenMediaFile(szFileName,FALSE);
			return;
		}
	}
	// vCXgɒǉ
	if (!fShowPlaylist)
		FORWARD_WM_COMMAND(hwndMain,CM_PLAYLIST,NULL,0,SendMessage);
	pPlaylist->AddDropFiles(hdrop);
	DragFinish(hdrop);
}


static BOOL OnFileOpen(HWND /*hwnd*/,ATOM atom,int nNo,int nNumFiles)
{
	TCHAR szFileName[MAX_PATH];
	DWORD dwAttributes;
	BOOL fResult;

	if (!GlobalGetAtomName(atom,szFileName,numberof(szFileName)))
		return FALSE;
	dwAttributes=GetFileAttributes(szFileName);
	if (dwAttributes==0xFFFFFFFF)
		return FALSE;
	if (nNumFiles==1 && (dwAttributes&FILE_ATTRIBUTE_DIRECTORY)==0
				&& lstrcmpi(GetFileExtensionPtr(szFileName),TEXT(".m3u"))!=0)
		return OpenMediaFile(szFileName,FALSE);
	if (nNo==0) {
		if (!fShowPlaylist)
			FORWARD_WM_COMMAND(hwndMain,CM_PLAYLIST,NULL,0,SendMessage);
	}
	if ((dwAttributes&FILE_ATTRIBUTE_DIRECTORY)==0)
		fResult=pPlaylist->AddFile(szFileName);
	else
		fResult=pPlaylist->AddDirectoryFiles(szFileName);
	return fResult;
}


static BOOL FileOpenDialog(HWND hwndOwner,LPTSTR pszFileName,
														LPCTSTR pszInitialDir)
{
	OPENFILENAME ofn;
	TCHAR szFilter[1024];
	int Length;

	Length=LoadString(hInst,IDS_OPENFILTER,szFilter,numberof(szFilter)-1);
	if (Length==0)
		return FALSE;
	szFilter[Length+1]='\0';	// for MS\[XRpC
	InitOpenFileName(&ofn);
	ofn.hwndOwner=hwndOwner;
	ofn.lpstrFilter=szFilter;
	ofn.nFilterIndex=nOpenFilterIndex;
	ofn.lpstrFile=pszFileName;
	ofn.nMaxFile=MAX_PATH;
	ofn.lpstrInitialDir=pszInitialDir;
	//ofn.lpstrTitle=pszTitle;
	ofn.Flags=OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_EXPLORER;
	if (!GetOpenFileName(&ofn))
		return FALSE;
	nOpenFilterIndex=ofn.nFilterIndex;
	// _CAO̐Ղ
	UpdateWindow(hwndMain);
	if (fShowPlaylist)
		UpdateWindow(pPlaylist->GetHandle());
	return TRUE;
}


BOOL CALLBACK GeneralOptionsDlgProc(HWND hDlg,UINT uMsg,WPARAM wParam,
																LPARAM lParam)
{
	switch (uMsg) {
	case WM_INITDIALOG:
		{
			int i;
			TCHAR szText[64];

			CheckDlgButton(hDlg,IDC_OPT_KEEPSINGLETASK,
									fKeepSingleTask?BST_CHECKED:BST_UNCHECKED);
			CheckDlgButton(hDlg,IDC_OPT_SAVEWINDOWPOS,
									fSaveWindowPos?BST_CHECKED:BST_UNCHECKED);
			CheckDlgButton(hDlg,IDC_OPT_SAVEWINDOWSIZE,
									fSaveWindowSize?BST_CHECKED:BST_UNCHECKED);
			for (i=0;i<3;i++) {
				LoadString(hInst,IDS_TITLEFORMAT_BASE+i,szText,numberof(szText));
				SendDlgItemMessage(hDlg,IDC_OPT_TITLEFORMAT,CB_ADDSTRING,0,
															(LPARAM)szText);
			}
			SendDlgItemMessage(hDlg,IDC_OPT_TITLEFORMAT,CB_SETCURSEL,
															TitleBarFormat,0);
			for (i=0;i<3;i++) {
				LoadString(hInst,IDS_FILENAMEFORMAT_BASE+i,szText,
															numberof(szText));
				SendDlgItemMessage(hDlg,IDC_OPT_TITLEFILENAME,CB_ADDSTRING,0,
															(LPARAM)szText);
			}
			SendDlgItemMessage(hDlg,IDC_OPT_TITLEFILENAME,CB_SETCURSEL,
															TitleFilePath,0);
			CheckDlgButton(hDlg,IDC_OPT_SHOWTIPHELP,
									fShowTipHelp?BST_CHECKED:BST_UNCHECKED);
			CheckDlgButton(hDlg,IDC_OPT_FINDSUBDIR,
								fFindSubDirectory?BST_CHECKED:BST_UNCHECKED);
			SendDlgItemMessage(hDlg,IDC_OPT_FILEMASKS,EM_LIMITTEXT,
													MAX_FILE_MASK_LENGTH-1,0);
			SetDlgItemText(hDlg,IDC_OPT_FILEMASKS,szFileMasks);
			CheckDlgButton(hDlg,IDC_OPT_USEUSERPROFILE,
									fUseUserProfile?BST_CHECKED:BST_UNCHECKED);
			CheckRadioButton(hDlg,IDC_OPT_DEVICE_MCI,IDC_OPT_DEVICE_DIRECTSHOW,
				MediaDevice==MEDIA_DEVICE_MCI?
								IDC_OPT_DEVICE_MCI:IDC_OPT_DEVICE_DIRECTSHOW);
		}
		return TRUE;

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDC_OPT_DEFAULTFILEMASKS:
			{
				TCHAR szText[MAX_FILE_MASK_LENGTH];

				GetMCIExtensions(szText);
				SetDlgItemText(hDlg,IDC_OPT_FILEMASKS,szText);
			}
			break;
		}
		return TRUE;

	case WM_NOTIFY:
		switch (((NMHDR*)lParam)->code) {
		case PSN_APPLY:
			{
				bool f;
				bool fKeepSingleTaskChanged=false;
				TCHAR szNewFileMasks[MAX_FILE_MASK_LENGTH];

				f=IsDlgButtonChecked(hDlg,IDC_OPT_KEEPSINGLETASK)==
																BST_CHECKED;
				if (fKeepSingleTask!=f) {
					fKeepSingleTask=f;
					fKeepSingleTaskChanged=true;
				}
				fSaveWindowPos=IsDlgButtonChecked(hDlg,IDC_OPT_SAVEWINDOWPOS)==
																BST_CHECKED;
				fSaveWindowSize=IsDlgButtonChecked(hDlg,
										IDC_OPT_SAVEWINDOWSIZE)==BST_CHECKED;
				TitleBarFormat=(TitleBarFormatType)SendDlgItemMessage(hDlg,
										IDC_OPT_TITLEFORMAT,CB_GETCURSEL,0,0);
				TitleFilePath=(TitleFilePathType)SendDlgItemMessage(hDlg,
									IDC_OPT_TITLEFILENAME,CB_GETCURSEL,0,0);
				SetWindowTitle(hwndMain);
				f=IsDlgButtonChecked(hDlg,IDC_OPT_SHOWTIPHELP)==BST_CHECKED;
				if (fShowTipHelp!=f) {
					fShowTipHelp=f;
					ShowToolbarTipHelp(hwndToolbar,fShowTipHelp);
					pMediaPlayer->ShowTipHelp(fShowTipHelp);
					pPlaylist->ShowTipHelp(fShowTipHelp);
				}
				fFindSubDirectory=
					IsDlgButtonChecked(hDlg,IDC_OPT_FINDSUBDIR)==BST_CHECKED;
				pPlaylist->FindSubDirectory(fFindSubDirectory);
				GetDlgItemText(hDlg,IDC_OPT_FILEMASKS,szNewFileMasks,
														MAX_FILE_MASK_LENGTH);
				if (szNewFileMasks[0]=='\0')
					lstrcpy(szNewFileMasks,TEXT("*.*"));
				if (lstrcmp(szNewFileMasks,szFileMasks)!=0) {
					lstrcpy(szFileMasks,szNewFileMasks);
					DeleteDirFileList();
					pPlaylist->SetFileMasks(szFileMasks);
				}
				f=IsDlgButtonChecked(hDlg,IDC_OPT_USEUSERPROFILE)==BST_CHECKED;
				if (f!=fUseUserProfile) {
					TCHAR szUserProfileDir[MAX_PATH];
					TCHAR szUserProfileName[MAX_PATH];
					TCHAR szCommonProfileName[MAX_PATH];
					LPTSTR pszSrcFile,pszDstFile;

					if (!GetUserProfileDirectory(szUserProfileDir)) {
						ResMessageBox(hDlg,IDS_FAIL_GETAPPDATADIR,0,
												MB_OK | MB_ICONEXCLAMATION);
						break;
					}
					MergeFileName(szUserProfileName,szUserProfileDir,
																INI_FILE_NAME);
					GetCommonProfileName(szCommonProfileName);
					if (fUseUserProfile) {
						pszSrcFile=szUserProfileName;
						pszDstFile=szCommonProfileName;
					} else {
						pszSrcFile=szCommonProfileName;
						pszDstFile=szUserProfileName;
						if (!DirectoryExists(szUserProfileDir)
								&& !CreateDirectory(szUserProfileDir,NULL)) {
							ResMessageBox(hDlg,IDS_FAIL_MOVEINIFILE,0,
												MB_OK | MB_ICONEXCLAMATION);
							break;
						}
					}
					if (FileExists(pszSrcFile)) {
						if (
#ifdef LP_WIN2000
							!MoveFileEx(pszSrcFile,pszDstFile,
													MOVEFILE_REPLACE_EXISTING)
#else
							!CopyFile(pszSrcFile,pszDstFile,FALSE)
#endif
								) {
							ResMessageBox(hDlg,IDS_FAIL_MOVEINIFILE,0,
												MB_OK | MB_ICONEXCLAMATION);
							break;
						}
#ifndef LP_WIN2000
						if (!DeleteFile(pszSrcFile) && fUseUserProfile) {
							ResMessageBox(hDlg,IDS_FAIL_MOVEINIFILE,0,
												MB_OK | MB_ICONEXCLAMATION);
							break;
						}
#endif
					}
					if (fUseUserProfile) {
						if (DirectoryExists(szUserProfileDir))
							RemoveDirectory(szUserProfileDir);
					}
					fUseUserProfile=f;
					lstrcpy(szIniFileName,pszDstFile);
				}
				if (fKeepSingleTaskChanged) {
					Settings Setting;

					if (Setting.Open(szIniFileName,TEXT("Settings"),
														Settings::OPEN_WRITE))
						Setting.WriteBoolean(TEXT("KeepSingleTask"),
															fKeepSingleTask);
				}
				MediaDeviceType Device=IsDlgButtonChecked(hDlg,
											IDC_OPT_DEVICE_MCI)==BST_CHECKED?
									MEDIA_DEVICE_MCI:MEDIA_DEVICE_DIRECTSHOW;
				if (Device!=MediaDevice) {
					MediaDevice=Device;
					pMediaPlayer->SetMediaDevice(MediaDevice);
				}
			}
			break;
		case PSN_SETACTIVE:
			OptionsStartPage=0;
			break;
		case PSN_HELP:
			ShowHelp(IDH_OPTIONS_GENERAL);
			break;
		}
		break;
	}
	return FALSE;
}


BOOL CALLBACK HistoryOptionsDlgProc(HWND hDlg,UINT uMsg,WPARAM wParam,
																LPARAM lParam)
{
	switch (uMsg) {
	case WM_INITDIALOG:
		SetDlgItemInt(hDlg,IDC_OPT_MAXFILEHISTORY,nMaxFileHistory,TRUE);
		SendDlgItemMessage(hDlg,IDC_OPT_MAXFILEHISTORY_UD,UDM_SETRANGE,0,
											MAKELPARAM(MAX_FILE_HISTORY,0));
		EnableDlgItem(hDlg,IDC_OPT_CLEARFILEHISTORY,FileHistory.GetCount()>0);
		CheckDlgButton(hDlg,IDC_OPT_NOSAVEFILEHISTORY,
								!fSaveFileHistory?BST_CHECKED:BST_UNCHECKED);
		SetDlgItemInt(hDlg,IDC_OPT_MAXDIRHISTORY,nMaxDirectoryHistory,TRUE);
		SendDlgItemMessage(hDlg,IDC_OPT_MAXDIRHISTORY_UD,UDM_SETRANGE,0,
										MAKELPARAM(MAX_DIRECTORY_HISTORY,0));
		EnableDlgItem(hDlg,IDC_OPT_CLEARDIRHISTORY,DirHistory.GetCount()>0);
		CheckDlgButton(hDlg,IDC_OPT_NOSAVEDIRHISTORY,
							!fSaveDirectoryHistory?BST_CHECKED:BST_UNCHECKED);
		CheckDlgButton(hDlg,IDC_OPT_USEDIRHISTORYMENU,
								fUseDirHistoryMenu?BST_CHECKED:BST_UNCHECKED);
		SetDlgItemInt(hDlg,IDC_FILELIST_MAX,MaxFileListMenuFiles,TRUE);
		SendDlgItemMessage(hDlg,IDC_FILELIST_MAX_UD,UDM_SETRANGE,0,
									MAKELPARAM(MAX_FILE_LIST_MENU_FILES,1));
		return TRUE;

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDC_OPT_CLEARFILEHISTORY:
			FileHistory.Clear();
			EnableDlgItem(hDlg,IDC_OPT_CLEARFILEHISTORY,FALSE);
			break;
		case IDC_OPT_CLEARDIRHISTORY:
			DirHistory.Clear();
			EnableDlgItem(hDlg,IDC_OPT_CLEARDIRHISTORY,FALSE);
			break;
		}
		return TRUE;

	case WM_NOTIFY:
		switch (((NMHDR*)lParam)->code) {
		case PSN_APPLY:
			nMaxFileHistory=RangeLimit(
					(int)GetDlgItemInt(hDlg,IDC_OPT_MAXFILEHISTORY,NULL,TRUE),
														0,MAX_FILE_HISTORY);
			FileHistory.SetMax(nMaxFileHistory);
			fSaveFileHistory=IsDlgButtonChecked(hDlg,
									IDC_OPT_NOSAVEFILEHISTORY)!=BST_CHECKED;
			nMaxDirectoryHistory=RangeLimit(
					(int)GetDlgItemInt(hDlg,IDC_OPT_MAXDIRHISTORY,NULL,TRUE),
													0,MAX_DIRECTORY_HISTORY);
			DirHistory.SetMax(nMaxDirectoryHistory);
			fSaveDirectoryHistory=IsDlgButtonChecked(hDlg,
										IDC_OPT_NOSAVEDIRHISTORY)!=BST_CHECKED;
			fUseDirHistoryMenu=IsDlgButtonChecked(hDlg,
									IDC_OPT_USEDIRHISTORYMENU)==BST_CHECKED;
			MaxFileListMenuFiles=RangeLimit(
						(int)GetDlgItemInt(hDlg,IDC_FILELIST_MAX,NULL,TRUE),
												1,MAX_FILE_LIST_MENU_FILES);
			break;
		case PSN_SETACTIVE:
			OptionsStartPage=1;
			break;
		case PSN_HELP:
			ShowHelp(IDH_OPTIONS_HISTORY);
			break;
		}
		break;
	}
	return FALSE;
}


BOOL CALLBACK PlayerOptionsDlgProc(HWND hDlg,UINT uMsg,WPARAM /*wParam*/,
																LPARAM lParam)
{
	switch (uMsg) {
	case WM_INITDIALOG:
		CheckDlgButton(hDlg,IDC_OPT_PLAYONOPEN,
										fPlayOnOpen?BST_CHECKED:BST_UNCHECKED);
		SetDlgItemInt(hDlg,IDC_OPT_ZOOMSTEP,nZoomStep,TRUE);
		SendDlgItemMessage(hDlg,IDC_OPT_ZOOMSTEP_UD,UDM_SETRANGE,0,
															MAKELPARAM(100,1));
		SetDlgItemInt(hDlg,IDC_OPT_BACKWARDSPEED,nBackwardSpeed/10,TRUE);
		SendDlgItemMessage(hDlg,IDC_OPT_BACKWARDSPEED_UD,UDM_SETRANGE,0,
							MAKELPARAM(MAX_BACKWARD_SPEED,MIN_BACKWARD_SPEED));
		SetDlgItemInt(hDlg,IDC_OPT_FORWARDSPEED,nForwardSpeed/10,TRUE);
		SendDlgItemMessage(hDlg,IDC_OPT_FORWARDSPEED_UD,UDM_SETRANGE,0,
							MAKELPARAM(MAX_FORWARD_SPEED,MIN_FORWARD_SPEED));
		CheckDlgButton(hDlg,IDC_OPT_FULLSCREEN_SEEKBAR,
							fShowFullScreenSeekBar?BST_CHECKED:BST_UNCHECKED);
		CheckDlgButton(hDlg,IDC_OPT_FULLSCREEN_PLAYBAR,
							fShowFullScreenPlayBar?BST_CHECKED:BST_UNCHECKED);
		CheckDlgButton(hDlg,IDC_OPT_FULLSCREEN_FITTODISPLAY,
							fFullScreenFitToDisplay?BST_CHECKED:BST_UNCHECKED);
		CheckDlgButton(hDlg,IDC_OPT_FULLSCREEN_HIDECURSOR,
							fFullScreenHideCursor?BST_CHECKED:BST_UNCHECKED);
		return TRUE;

	case WM_NOTIFY:
		switch (((LPNMHDR)lParam)->code) {
		case PSN_APPLY:
			fPlayOnOpen=
					IsDlgButtonChecked(hDlg,IDC_OPT_PLAYONOPEN)==BST_CHECKED;
			nZoomStep=RangeLimit(
								GetDlgItemInt(hDlg,IDC_OPT_ZOOMSTEP,NULL,TRUE),
												MIN_ZOOM_STEP,MAX_ZOOM_STEP);
			nBackwardSpeed=RangeLimit(
						GetDlgItemInt(hDlg,IDC_OPT_BACKWARDSPEED,NULL,TRUE)*10,
										MIN_BACKWARD_SPEED,MAX_BACKWARD_SPEED);
			pMediaPlayer->SetBackwardSpeed(nBackwardSpeed);
			nForwardSpeed=RangeLimit(
						GetDlgItemInt(hDlg,IDC_OPT_FORWARDSPEED,NULL,TRUE)*10,
										MIN_FORWARD_SPEED,MAX_FORWARD_SPEED);
			pMediaPlayer->SetForwardSpeed(nForwardSpeed);
			fShowFullScreenSeekBar=IsDlgButtonChecked(hDlg,
									IDC_OPT_FULLSCREEN_SEEKBAR)==BST_CHECKED;
			fShowFullScreenPlayBar=IsDlgButtonChecked(hDlg,
									IDC_OPT_FULLSCREEN_PLAYBAR)==BST_CHECKED;
			fFullScreenFitToDisplay=IsDlgButtonChecked(hDlg,
								IDC_OPT_FULLSCREEN_FITTODISPLAY)==BST_CHECKED;
			pMediaPlayer->FullScreenFitToDisplay(fFullScreenFitToDisplay);
			fFullScreenHideCursor=IsDlgButtonChecked(hDlg,
								IDC_OPT_FULLSCREEN_HIDECURSOR)==BST_CHECKED;
			pMediaPlayer->FullScreenHideCursor(fFullScreenHideCursor);
			break;
		case PSN_SETACTIVE:
			OptionsStartPage=2;
			break;
		case PSN_HELP:
			ShowHelp(IDH_OPTIONS_PLAYER);
			break;
		}
		break;
	}
	return FALSE;
}


BOOL CALLBACK PlaylistOptionsDlgProc(HWND hDlg,UINT uMsg,WPARAM wParam,
																LPARAM lParam)
{
	switch (uMsg) {
	case WM_INITDIALOG:
		CheckDlgButton(hDlg,IDC_PLAYLISTOPT_FOREGROUND,
							fPlaylistKeepForeground?BST_CHECKED:BST_UNCHECKED);
		CheckDlgButton(hDlg,IDC_PLAYLISTOPT_SINGLECLICKOPEN,
						fPlaylistSingleClickOpen?BST_CHECKED:BST_UNCHECKED);
		{
			int i;
			TCHAR szText[64];

			for (i=0;i<3;i++) {
				LoadString(hInst,IDS_FILENAMEFORMAT_BASE+i,szText,
															numberof(szText));
				SendDlgItemMessage(hDlg,IDC_PLAYLISTOPT_FILENAMEFORMAT,
												CB_ADDSTRING,0,(LPARAM)szText);
			}
			SendDlgItemMessage(hDlg,IDC_PLAYLISTOPT_FILENAMEFORMAT,
										CB_SETCURSEL,PlaylistFileNameType,0);
		}
		CheckDlgButton(hDlg,IDC_PLAYLISTOPT_RESTORELIST,
							fPlaylistRestoreList?BST_CHECKED:BST_UNCHECKED);
		SetDlgItemInt(hDlg,IDC_PLAYLISTOPT_MAXHISTORY,MaxPlaylistHistory,TRUE);
		SendDlgItemMessage(hDlg,IDC_PLAYLISTOPT_MAXHISTORY_UD,UDM_SETRANGE,0,
										MAKELPARAM(MAX_PLAYLIST_HISTORY,0));
		EnableWindow(GetDlgItem(hDlg,IDC_PLAYLISTOPT_CLEARHISTORY),
											pPlaylist->GetHistoryCount()>0);
		// Presets
		{
			HWND hwndList=GetDlgItem(hDlg,IDC_PRESET_LIST);
			LV_COLUMN lvc;
			int i;

			(void)ListView_SetExtendedListViewStyle(hwndList,
														LVS_EX_FULLROWSELECT);
			lvc.mask=LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
			lvc.fmt=LVCFMT_RIGHT;
			lvc.cx=16;
			lvc.pszText=TEXT("No");
			lvc.iSubItem=0;
			ListView_InsertColumn(hwndList,0,&lvc);
			lvc.fmt=LVCFMT_LEFT;
			lvc.cx=120;
			lvc.pszText=TEXT("O");
			lvc.iSubItem=1;
			ListView_InsertColumn(hwndList,1,&lvc);
			for (i=0;i<NUM_PRESETS;i++) {
				LV_ITEM lvi;
				TCHAR szText[4];

				wsprintf(szText,TEXT("%d"),i+1);
				lvi.mask=LVIF_TEXT | LVIF_PARAM;
				lvi.iItem=i;
				lvi.iSubItem=0;
				lvi.pszText=szText;
				lvi.lParam=PresetList[i].Flags;
				ListView_InsertItem(hwndList,&lvi);
				lvi.mask=LVIF_TEXT;
				lvi.iSubItem=1;
				lvi.pszText=PresetList[i].szName;
				ListView_SetItem(hwndList,&lvi);
			}
			ListView_SetColumnWidth(hwndList,0,LVSCW_AUTOSIZE_USEHEADER);
			ListView_SetColumnWidth(hwndList,1,LVSCW_AUTOSIZE_USEHEADER);
			SendDlgItemMessage(hDlg,IDC_PRESET_NAME,EM_LIMITTEXT,
														MAX_PRESET_NAME-1,0);
		}
		return TRUE;

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDC_PLAYLISTOPT_CLEARHISTORY:
			pPlaylist->ClearHistory();
			EnableDlgItem(hDlg,IDC_PLAYLISTOPT_CLEARHISTORY,FALSE);
			break;
		case IDC_PRESET_NAME:
			if (HIWORD(wParam)==EN_CHANGE) {
				HWND hwndList=GetDlgItem(hDlg,IDC_PRESET_LIST);
				int Sel=ListView_GetNextItem(hwndList,-1,LVNI_SELECTED);

				if (Sel>=0) {
					LV_ITEM lvi;
					TCHAR szName[MAX_PRESET_NAME];

					GetDlgItemText(hDlg,IDC_PRESET_NAME,szName,MAX_PRESET_NAME);
					lvi.mask=LVIF_TEXT;
					lvi.iItem=Sel;
					lvi.iSubItem=1;
					lvi.pszText=szName;
					ListView_SetItem(hwndList,&lvi);
				}
			}
			break;
		case IDC_PRESET_MENUBAR:
		case IDC_PRESET_TOOLBAR:
		case IDC_PRESET_STATUSBAR:
		case IDC_PRESET_PLAYBAR:
		case IDC_PRESET_SEEKBAR:
			{
				HWND hwndList=GetDlgItem(hDlg,IDC_PRESET_LIST);
				int Sel=ListView_GetNextItem(hwndList,-1,LVNI_SELECTED);

				if (Sel>=0) {
					LV_ITEM lvi;
					unsigned int Mask=1<<(LOWORD(wParam)-IDC_PRESET_MENUBAR);

					lvi.mask=LVIF_PARAM;
					lvi.iItem=Sel;
					lvi.iSubItem=0;
					ListView_GetItem(hwndList,&lvi);
					if (IsDlgButtonChecked(hDlg,LOWORD(wParam))==BST_CHECKED)
						lvi.lParam|=Mask;
					else
						lvi.lParam&=~Mask;
					ListView_SetItem(hwndList,&lvi);
				}
			}
			break;
		}
		return TRUE;

	case WM_NOTIFY:
		switch (((LPNMHDR)lParam)->code) {
		case LVN_ITEMCHANGED:
			{
				HWND hwndList=((LPNMHDR)lParam)->hwndFrom;
				int Sel=ListView_GetNextItem(hwndList,-1,LVNI_SELECTED);
				int i;

				if (Sel>=0) {
					LV_ITEM lvi;
					TCHAR szName[MAX_PRESET_NAME];

					lvi.mask=LVIF_TEXT;
					lvi.iItem=Sel;
					lvi.iSubItem=1;
					lvi.pszText=szName;
					lvi.cchTextMax=MAX_PRESET_NAME;
					ListView_GetItem(hwndList,&lvi);
					SetDlgItemText(hDlg,IDC_PRESET_NAME,szName);
					lvi.mask=LVIF_PARAM;
					lvi.iSubItem=0;
					ListView_GetItem(hwndList,&lvi);
					for (i=0;i<5;i++)
						CheckDlgButton(hDlg,IDC_PRESET_MENUBAR+i,
							((lvi.lParam>>i)&1)!=0?MFS_CHECKED:MFS_UNCHECKED);
				} else {
					SetDlgItemText(hDlg,IDC_PRESET_NAME,TEXT(""));
					for (i=IDC_PRESET_MENUBAR;i<=IDC_PRESET_SEEKBAR;i++)
						CheckDlgButton(hDlg,i,MFS_UNCHECKED);
				}
				EnableDlgItems(hDlg,IDC_PRESET_NAME,IDC_PRESET_SEEKBAR,Sel>=0);
			}
			return TRUE;
		case PSN_APPLY:
#if 0
			{
				bool f=IsDlgButtonChecked(hDlg,IDC_PLAYLISTOPT_FOREGROUND)==
																BST_CHECKED;

				if (f!=fPlaylistKeepForeground) {
					fPlaylistKeepForeground=f;
					SetParent(pPlaylist->GetHandle(),
										fPlaylistKeepForeground?hwndMain:NULL);
				}
			}
#else
			fPlaylistKeepForeground=IsDlgButtonChecked(hDlg,
									IDC_PLAYLISTOPT_FOREGROUND)==BST_CHECKED;
#endif
			fPlaylistSingleClickOpen=IsDlgButtonChecked(hDlg,
								IDC_PLAYLISTOPT_SINGLECLICKOPEN)==BST_CHECKED;
			PlaylistFileNameType=(Playlist::FileNameType)
				SendDlgItemMessage(hDlg,IDC_PLAYLISTOPT_FILENAMEFORMAT,
															CB_GETCURSEL,0,0);
			pPlaylist->SetFileNameType(PlaylistFileNameType);
			fPlaylistRestoreList=IsDlgButtonChecked(hDlg,
									IDC_PLAYLISTOPT_RESTORELIST)==BST_CHECKED;
			MaxPlaylistHistory=RangeLimit(
					GetDlgItemInt(hDlg,IDC_PLAYLISTOPT_MAXHISTORY,NULL,TRUE),
													0,MAX_PLAYLIST_HISTORY);
			pPlaylist->SetMaxHistory(MaxPlaylistHistory);
			// Presets
			{
				HWND hwndList=GetDlgItem(hDlg,IDC_PRESET_LIST);
				int i;
				LV_ITEM lvi;

				for (i=0;i<NUM_PRESETS;i++) {
					lvi.mask=LVIF_TEXT;
					lvi.iItem=i;
					lvi.iSubItem=1;
					lvi.pszText=PresetList[i].szName;
					lvi.cchTextMax=MAX_PRESET_NAME;
					ListView_GetItem(hwndList,&lvi);
					lvi.mask=LVIF_PARAM;
					lvi.iSubItem=0;
					ListView_GetItem(hwndList,&lvi);
					PresetList[i].Flags=lvi.lParam;
				}
				SetPresetMenu();
			}
			break;
		case PSN_SETACTIVE:
			OptionsStartPage=3;
			break;
		case PSN_HELP:
			ShowHelp(IDH_OPTIONS_PLAYLIST);
			break;
		}
		break;
	}
	return FALSE;
}


static bool LoadTheme(HWND hDlg,LPCTSTR pFileName)
{
	Settings ThemeSettings;
	int i;
	TCHAR szText[32];
	ColorInfo Color;

	if (!ThemeSettings.Open(pFileName,TEXT("ColorScheme"),Settings::OPEN_READ))
		return false;
	for (i=0;i<numberof(ColorSchemeList);i++) {
		if (ThemeSettings.ReadString(ColorSchemeList[i].pName,szText,
										numberof(szText)) && szText[0]!='\0') {
			Color=(ColorInfo)SendDlgItemMessage(hDlg,IDC_THEME_COLORS_LIST,
														LB_GETITEMDATA,i,0);
			if (ColorInfo_Parse(&Color,szText))
				SendDlgItemMessage(hDlg,IDC_THEME_COLORS_LIST,LB_SETITEMDATA,
															i,(LPARAM)Color);
		} else if ((ColorSchemeType)i==COLOR_SCHEME_PLAYLIST_PLAYINGTEXT) {
			SendDlgItemMessage(hDlg,IDC_THEME_COLORS_LIST,LB_SETITEMDATA,i,
				SendDlgItemMessage(hDlg,IDC_THEME_COLORS_LIST,LB_GETITEMDATA,
									(WPARAM)COLOR_SCHEME_PLAYLISTTEXT,0));
		} else if ((ColorSchemeType)i==COLOR_SCHEME_PLAYLIST_PLAYINGSELTEXT) {
			SendDlgItemMessage(hDlg,IDC_THEME_COLORS_LIST,LB_SETITEMDATA,i,
				SendDlgItemMessage(hDlg,IDC_THEME_COLORS_LIST,LB_GETITEMDATA,
									(WPARAM)COLOR_SCHEME_PLAYLISTSELTEXT,0));
		} else if ((ColorSchemeType)i==COLOR_SCHEME_PLAYLIST_PLAYEDTEXT) {
			SendDlgItemMessage(hDlg,IDC_THEME_COLORS_LIST,LB_SETITEMDATA,i,
				SendDlgItemMessage(hDlg,IDC_THEME_COLORS_LIST,LB_GETITEMDATA,
									(WPARAM)COLOR_SCHEME_PLAYLISTTEXT,0));
		}
	}
	if (ThemeSettings.Open(pFileName,TEXT("Plugin"),Settings::OPEN_READ)) {
		TCHAR szFileName[MAX_PATH];

		if (ThemeSettings.ReadString(TEXT("File"),szFileName,MAX_PATH)
													&& szFileName[0]!='\0') {
			HWND hwndList=GetDlgItem(hDlg,IDC_THEME_TOOLBAR_LIST);

			if (lstrcmpi(szFileName,TEXT("default"))==0) {
				ListView_SetItemState(hwndList,0,LVIS_FOCUSED | LVIS_SELECTED,
												LVIS_FOCUSED | LVIS_SELECTED);
			} else {
				int Count=ListView_GetItemCount(hwndList);
				LV_ITEM lvi;
				const ThemePlugin *pPlugin;

				for (i=1;i<Count;i++) {
					lvi.mask=LVIF_PARAM;
					lvi.iItem=i;
					lvi.iSubItem=0;
					ListView_GetItem(hwndList,&lvi);
					pPlugin=(const ThemePlugin*)lvi.lParam;
					if (lstrcmpi(GetFileNamePtr(pPlugin->FileName()),
															szFileName)==0) {
						ListView_SetItemState(hwndList,i,
												LVIS_FOCUSED | LVIS_SELECTED,
												LVIS_FOCUSED | LVIS_SELECTED);
						break;
					}
				}
				if (i==Count)
					ListView_SetItemState(hwndList,0,
												LVIS_FOCUSED | LVIS_SELECTED,
												LVIS_FOCUSED | LVIS_SELECTED);
			}
		}
	}
	return true;
}


static bool SaveTheme(HWND hDlg,LPCTSTR pFileName)
{
	HANDLE hFile;
	DWORD Size,Write;
	int i;
	TCHAR szColor[32],szText[64];

	hFile=CreateFile(pFileName,GENERIC_WRITE,FILE_SHARE_READ,NULL,
									CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
	if (hFile==INVALID_HANDLE_VALUE)
		return false;
	Size=sizeof("[ColorScheme]\r\n")-1;
	if (!WriteFile(hFile,"[ColorScheme]\r\n",Size,&Write,NULL)
															|| Write!=Size) {
		CloseHandle(hFile);
		return false;
	}
	for (i=0;i<numberof(ColorSchemeList);i++) {
		ColorInfo_GetText((ColorInfo)SendDlgItemMessage(hDlg,
							IDC_THEME_COLORS_LIST,LB_GETITEMDATA,i,0),szColor);
		Size=wsprintf(szText,TEXT("%s=%s\r\n"),
											ColorSchemeList[i].pName,szColor);
#ifndef UNICODE
		if (!WriteFile(hFile,szText,Size,&Write,NULL) || Size!=Write) {
			CloseHandle(hFile);
			return FALSE;
		}
#else
		char szAnsiText[64];

		Size=WideCharToMultiByte(CP_ACP,0,szText,Size,
									szAnsiText,sizeof(szAnsiText),NULL,NULL);
		if (!WriteFile(hFile,szAnsiText,Size,&Write,NULL) || Size!=Write) {
			CloseHandle(hFile);
			return FALSE;
		}
#endif
	}
	int Sel=ListView_GetNextItem(GetDlgItem(hDlg,IDC_THEME_TOOLBAR_LIST),-1,
																LVNI_SELECTED);
	if (Sel>=0) {
		LPCTSTR pFileName;

		if (Sel==0) {
			pFileName=TEXT("default");
		} else {
			LV_ITEM lvi;
			const ThemePlugin *pPlugin;

			lvi.mask=LVIF_PARAM;
			lvi.iItem=Sel;
			lvi.iSubItem=0;
			ListView_GetItem(GetDlgItem(hDlg,IDC_THEME_TOOLBAR_LIST),&lvi);
			pPlugin=(const ThemePlugin*)lvi.lParam;
			pFileName=GetFileNamePtr(pPlugin->FileName());
		}
		Size=sizeof("\r\n[Plugin]\r\nFile=")-1;
		if (!WriteFile(hFile,"\r\n[Plugin]\r\nFile=",Size,&Write,NULL)
															|| Size!=Write) {
			CloseHandle(hFile);
			return FALSE;
		}
		Size=lstrlen(pFileName);
#ifndef UNICODE
		if (!WriteFile(hFile,pFileName,Size,&Write,NULL) || Size!=Write) {
			CloseHandle(hFile);
			return FALSE;
		}
#else
		char szFileName[MAX_PATH];
		Size=WideCharToMultiByte(CP_ACP,0,pFileName,Size,szFileName,MAX_PATH,
																	NULL,NULL);
		if (!WriteFile(hFile,szFileName,Size,&Write,NULL) || Size!=Write) {
			CloseHandle(hFile);
			return FALSE;
		}
#endif
		Size=sizeof("\r\n")-1;
		if (!WriteFile(hFile,"\r\n",Size,&Write,NULL) || Size!=Write) {
			CloseHandle(hFile);
			return FALSE;
		}
	}
	CloseHandle(hFile);
	return true;
}


static void GetThemesDirectory(LPTSTR pszDirectory)
{
	GetModuleFileName(hInst,pszDirectory,MAX_PATH);
	lstrcpy(GetFileNamePtr(pszDirectory),THEMES_DIRECTORY);
}


BOOL CALLBACK ThemeOptionsDlgProc(HWND hDlg,UINT uMsg,WPARAM wParam,
																LPARAM lParam)
{
	static ColorInfo *pOldColorScheme=NULL;
	static ThemePluginList *pThemePlugins;
	static HIMAGELIST himlPreview=NULL;
	static bool fLoading=false;

	switch (uMsg) {
	case WM_INITDIALOG:
		// Themes
		{
			FileList ThemeFiles;
			TCHAR szFileName[MAX_PATH];
			int i;
			int Cur;

			GetThemesDirectory(szFileName);
			ThemeFiles.AddDirectoryFiles(szFileName,TEXT("*.lptheme"),FALSE);
			SendDlgItemMessage(hDlg,IDC_THEME_LIST,CB_ADDSTRING,0,
												(LPARAM)TEXT("(݂̐ݒ)"));
			Cur=-1;
			for (i=0;i<ThemeFiles.GetCount();i++) {
				ThemeFiles.GetFile(i,szFileName);
				if (lstrcmpi(szFileName,szCurThemeFileName)==0)
					Cur=i;
				*GetFileExtensionPtr(szFileName)='\0';
				SendDlgItemMessage(hDlg,IDC_THEME_LIST,CB_ADDSTRING,0,
										(LPARAM)GetFileNamePtr(szFileName));
			}
			SendDlgItemMessage(hDlg,IDC_THEME_LIST,CB_SETCURSEL,Cur+1,0);
			EnableDlgItem(hDlg,IDC_THEME_DELETE,Cur>=0);
		}
		// Color scheme
		{
			int i;
			TCHAR szText[64];

			for (i=0;i<numberof(ColorSchemeList);i++) {
				LoadString(hInst,IDS_COLORSCHEME_BASE+i,szText,
															numberof(szText));
				SendDlgItemMessage(hDlg,IDC_THEME_COLORS_LIST,
												LB_ADDSTRING,0,(LPARAM)szText);
				SendDlgItemMessage(hDlg,IDC_THEME_COLORS_LIST,LB_SETITEMDATA,
										i,(LPARAM)ColorSchemeList[i].Color);
			}
			SendDlgItemMessage(hDlg,IDC_THEME_COLORS_LIST,LB_SETCURSEL,0,0);
			ColorsComboBox_Init(hDlg,IDC_THEME_COLORS_COLORLIST,
													ColorSchemeList[0].Color);
			EnableDlgItem(hDlg,IDC_THEME_COLORS_CHOOSE,
				ColorInfo_GetType(ColorSchemeList[0].Color)==
															COLOR_TYPE_CUSTOM);
		}
		// Toolbar
		{
			HWND hwndList=GetDlgItem(hDlg,IDC_THEME_TOOLBAR_LIST);
			LV_COLUMN lvc;
			TCHAR szDir[MAX_PATH];
			int nNumPlugins,i;
			LV_ITEM lvi;

			(void)ListView_SetExtendedListViewStyle(hwndList,
														LVS_EX_FULLROWSELECT);
			lvc.mask=LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
			lvc.fmt=LVCFMT_LEFT;
			lvc.cx=60;
			lvc.pszText=TEXT("O");
			lvc.iSubItem=0;
			ListView_InsertColumn(hwndList,0,&lvc);
			//lvc.fmt=LVCFMT_LEFT;
			lvc.cx=120;
			lvc.pszText=TEXT("t@C");
			lvc.iSubItem=1;
			ListView_InsertColumn(hwndList,1,&lvc);
			//lvc.fmt=LVCFMT_LEFT;
			lvc.cx=120;
			lvc.pszText=TEXT("쌠");
			lvc.iSubItem=2;
			ListView_InsertColumn(hwndList,2,&lvc);
			GetThemesDirectory(szDir);
			pThemePlugins=new ThemePluginList;
			pThemePlugins->LoadPlugins(szDir);
			lvi.mask=LVIF_TEXT;
			lvi.iItem=0;
			lvi.iSubItem=0;
			lvi.pszText=TEXT("W");
			lvi.iItem=ListView_InsertItem(hwndList,&lvi);
			nNumPlugins=pThemePlugins->NumPlugins();
			for (i=0;i<nNumPlugins;i++) {
				const ThemePlugin *pPlugin=pThemePlugins->GetPlugin(i);
				ThemePluginInfo Info;

				pPlugin->GetInfo(&Info);
				lvi.mask=LVIF_TEXT | LVIF_PARAM;
				lvi.iItem=i+1;
				lvi.iSubItem=0;
				lvi.pszText=Info.Name;
				lvi.lParam=(LPARAM)pPlugin;
				ListView_InsertItem(hwndList,&lvi);
				lvi.mask=LVIF_TEXT;
				lvi.iSubItem=1;
				lvi.pszText=GetFileNamePtr(pPlugin->FileName());
				ListView_SetItem(hwndList,&lvi);
				lvi.iSubItem=2;
				lvi.pszText=Info.Copyright;
				ListView_SetItem(hwndList,&lvi);
				if (szThemePluginFileName[0]!='\0'
						&& lstrcmpi(szThemePluginFileName,
									GetFileNamePtr(pPlugin->FileName()))==0)
					ListView_SetItemState(hwndList,lvi.iItem,
												LVIS_FOCUSED | LVIS_SELECTED,
												LVIS_FOCUSED | LVIS_SELECTED);
			}
			if (szThemePluginFileName[0]=='\0')
				ListView_SetItemState(hwndList,0,
					LVIS_FOCUSED | LVIS_SELECTED,LVIS_FOCUSED | LVIS_SELECTED);
			ListView_SetColumnWidth(hwndList,0,LVSCW_AUTOSIZE_USEHEADER);
			ListView_SetColumnWidth(hwndList,1,LVSCW_AUTOSIZE_USEHEADER);
			ListView_SetColumnWidth(hwndList,2,LVSCW_AUTOSIZE_USEHEADER);
		}
		return TRUE;

	case WM_MEASUREITEM:
		return ColorsComboBox_OnMeasureItem(hDlg,wParam,lParam);

	case WM_DRAWITEM:
		if (wParam==IDC_THEME_COLORS_COLORLIST) {
			return ColorsComboBox_OnDrawItem(hDlg,wParam,lParam);
		} else if (wParam==IDC_THEME_COLORS_LIST) {
			LPDRAWITEMSTRUCT pdis=(LPDRAWITEMSTRUCT)lParam;

			switch (pdis->itemAction) {
			case ODA_DRAWENTIRE:
			case ODA_SELECT:
				{
					HBRUSH hbr,hbrOld;
					HPEN hpenOld;
					RECT rc;
					TCHAR szText[64];
					int nOldBkMode;
					COLORREF crOldTextColor;

					hbr=CreateSolidBrush(GetSysColor(
								(pdis->itemState&ODS_SELECTED)==0?COLOR_WINDOW:
															COLOR_HIGHLIGHT));
					FillRect(pdis->hDC,&pdis->rcItem,hbr);
					DeleteObject(hbr);
					hbr=CreateSolidBrush(
								ColorInfo_GetColor((ColorInfo)pdis->itemData));
					hbrOld=SelectBrush(pdis->hDC,hbr);
					hpenOld=SelectPen(pdis->hDC,
											(HPEN)GetStockObject(BLACK_PEN));
					rc.left=pdis->rcItem.left+2;
					rc.top=pdis->rcItem.top+2;
					rc.bottom=pdis->rcItem.bottom-2;
					rc.right=rc.left+(rc.bottom-rc.top)*2;
					Rectangle(pdis->hDC,rc.left,rc.top,rc.right,rc.bottom);
					SelectPen(pdis->hDC,hpenOld);
					SelectBrush(pdis->hDC,hbrOld);
					DeleteObject(hbr);
					nOldBkMode=SetBkMode(pdis->hDC,TRANSPARENT);
					crOldTextColor=SetTextColor(pdis->hDC,GetSysColor(
						(pdis->itemState&ODS_SELECTED)==0?COLOR_WINDOWTEXT:
														COLOR_HIGHLIGHTTEXT));
					SendDlgItemMessage(hDlg,IDC_THEME_COLORS_LIST,LB_GETTEXT,
												pdis->itemID,(LPARAM)szText);
					rc.left=rc.right+2;
					rc.top=pdis->rcItem.top;
					rc.right=pdis->rcItem.right;
					rc.bottom=pdis->rcItem.bottom;
					DrawText(pdis->hDC,szText,-1,&rc,
										DT_LEFT | DT_SINGLELINE | DT_VCENTER);
					SetTextColor(pdis->hDC,crOldTextColor);
					SetBkMode(pdis->hDC,nOldBkMode);
					if ((pdis->itemState&ODS_FOCUS)==0)
						break;
				}
			case ODA_FOCUS:
				DrawFocusRect(pdis->hDC,&pdis->rcItem);
				break;
			}
		} else {
			LPDRAWITEMSTRUCT pdis=(LPDRAWITEMSTRUCT)lParam;
			HBRUSH hbr;

			hbr=CreateSolidBrush(GetSysColor(COLOR_3DFACE));
			FillRect(pdis->hDC,&pdis->rcItem,hbr);
			DeleteObject(hbr);
			if (himlPreview!=NULL) {
				int IconWidth,IconHeight;
				int Count,i,x;

				ImageList_GetIconSize(himlPreview,&IconWidth,&IconHeight);
				Count=ImageList_GetImageCount(himlPreview);
				x=pdis->rcItem.left;
				for (i=0;i<Count && x<pdis->rcItem.right;i++) {
					ImageList_Draw(himlPreview,i,pdis->hDC,x,pdis->rcItem.top,
															ILD_TRANSPARENT);
					x+=IconWidth;
				}
			}
		}
		return TRUE;

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDC_THEME_LIST:
			if (HIWORD(wParam)==CBN_SELCHANGE) {
				int Sel;
				TCHAR szFileName[MAX_PATH],*p;
				ColorInfo Color;

				Sel=SendDlgItemMessage(hDlg,IDC_THEME_LIST,CB_GETCURSEL,0,0);
				EnableDlgItem(hDlg,IDC_THEME_DELETE,Sel>0);
				if (Sel<1)
					return TRUE;
				GetThemesDirectory(szFileName);
				p=szFileName+lstrlen(szFileName);
				*p++='\\';
				SendDlgItemMessage(hDlg,IDC_THEME_LIST,CB_GETLBTEXT,Sel,
																	(LPARAM)p);
				lstrcat(p,TEXT(".lptheme"));
				fLoading=true;
				if (!LoadTheme(hDlg,szFileName)) {
					fLoading=false;
					ResMessageBox(hDlg,IDS_FAIL_LOADFILE,0,
												MB_OK | MB_ICONEXCLAMATION);
					SendDlgItemMessage(hDlg,IDC_THEME_LIST,CB_SETCURSEL,0,0);
					EnableDlgItem(hDlg,IDC_THEME_DELETE,FALSE);
					return TRUE;
				}
				fLoading=false;
				InvalidateDlgItem(hDlg,IDC_THEME_COLORS_LIST);
				Sel=SendDlgItemMessage(hDlg,IDC_THEME_COLORS_LIST,
															LB_GETCURSEL,0,0);
				Color=(ColorInfo)SendDlgItemMessage(hDlg,IDC_THEME_COLORS_LIST,
														LB_GETITEMDATA,Sel,0);
				ColorsComboBox_SetColor(hDlg,IDC_THEME_COLORS_COLORLIST,Color);
				EnableDlgItem(hDlg,IDC_THEME_COLORS_CHOOSE,
								ColorInfo_GetType(Color)==COLOR_TYPE_CUSTOM);
			}
			return TRUE;
		case IDC_THEME_DELETE:
			{
				int Sel=SendDlgItemMessage(hDlg,IDC_THEME_LIST,CB_GETCURSEL,
																		0,0);

				if (Sel>0 && ResMessageBox(hDlg,IDS_THEME_DELETE,
													IDS_THEME_DELETE_CAPTION,
										MB_OKCANCEL | MB_ICONQUESTION)==IDOK) {
					TCHAR szFileName[MAX_PATH],*p;

					GetThemesDirectory(szFileName);
					p=szFileName+lstrlen(szFileName);
					*p++='\\';
					SendDlgItemMessage(hDlg,IDC_THEME_LIST,CB_GETLBTEXT,Sel,
																	(LPARAM)p);
					lstrcat(p,TEXT(".lptheme"));
					if (!DeleteFile(szFileName)) {
						ResMessageBox(hDlg,IDS_THEME_DELETE_FAIL,0,
												MB_OK | MB_ICONEXCLAMATION);
						return TRUE;
					}
					SendDlgItemMessage(hDlg,IDC_THEME_LIST,CB_DELETESTRING,Sel,
																			0);
					SendDlgItemMessage(hDlg,IDC_THEME_LIST,CB_SETCURSEL,0,0);
					EnableDlgItem(hDlg,IDC_THEME_DELETE,FALSE);
				}
			}
			return TRUE;
		case IDC_THEME_COLORS_LIST:
			if (HIWORD(wParam)==LBN_SELCHANGE) {
				int nSel=SendDlgItemMessage(hDlg,IDC_THEME_COLORS_LIST,
															LB_GETCURSEL,0,0);
				ColorInfo Color;

				Color=(ColorInfo)SendDlgItemMessage(hDlg,IDC_THEME_COLORS_LIST,
														LB_GETITEMDATA,nSel,0);
				ColorsComboBox_SetColor(hDlg,IDC_THEME_COLORS_COLORLIST,Color);
				EnableDlgItem(hDlg,IDC_THEME_COLORS_CHOOSE,
								ColorInfo_GetType(Color)==COLOR_TYPE_CUSTOM);
			}
			return TRUE;
		case IDC_THEME_COLORS_COLORLIST:
			if (HIWORD(wParam)==CBN_SELCHANGE) {
				EnableDlgItem(hDlg,IDC_THEME_COLORS_CHOOSE,
					SendDlgItemMessage(hDlg,IDC_THEME_COLORS_COLORLIST,
														CB_GETCURSEL,0,0)==0);
				SendDlgItemMessage(hDlg,IDC_THEME_COLORS_LIST,LB_SETITEMDATA,
					SendDlgItemMessage(hDlg,IDC_THEME_COLORS_LIST,LB_GETCURSEL,
																		0,0),
					ColorsComboBox_GetColor(hDlg,IDC_THEME_COLORS_COLORLIST));
				InvalidateDlgItem(hDlg,IDC_THEME_COLORS_LIST);
				if ((int)SendDlgItemMessage(hDlg,IDC_THEME_LIST,CB_GETCURSEL,
																	0,0)>0) {
					SendDlgItemMessage(hDlg,IDC_THEME_LIST,CB_SETCURSEL,0,0);
					EnableDlgItem(hDlg,IDC_THEME_DELETE,FALSE);
				}
			}
			return TRUE;
		case IDC_THEME_COLORS_CHOOSE:
			if (ColorsComboBox_ChooseColor(hDlg,IDC_THEME_COLORS_COLORLIST)) {
				SendDlgItemMessage(hDlg,IDC_THEME_COLORS_LIST,LB_SETITEMDATA,
					SendDlgItemMessage(hDlg,IDC_THEME_COLORS_LIST,LB_GETCURSEL,
																		0,0),
					ColorsComboBox_GetColor(hDlg,IDC_THEME_COLORS_COLORLIST));
				InvalidateDlgItem(hDlg,IDC_THEME_COLORS_LIST);
				if ((int)SendDlgItemMessage(hDlg,IDC_THEME_LIST,CB_GETCURSEL,
																	0,0)>0) {
					SendDlgItemMessage(hDlg,IDC_THEME_LIST,CB_SETCURSEL,0,0);
					EnableDlgItem(hDlg,IDC_THEME_DELETE,FALSE);
				}
			}
			return TRUE;
		case IDC_THEME_PREVIEW:
			{
				int i;

				if (pOldColorScheme==NULL) {
					pOldColorScheme=new ColorInfo[numberof(ColorSchemeList)];
					for (i=0;i<numberof(ColorSchemeList);i++)
						pOldColorScheme[i]=ColorSchemeList[i].Color;
				}
				for (i=0;i<numberof(ColorSchemeList);i++)
					ColorSchemeList[i].Color=(ColorInfo)
						SendDlgItemMessage(hDlg,IDC_THEME_COLORS_LIST,
														LB_GETITEMDATA,i,0);
				ApplyColorScheme();
			}
			return 0;
#if 0
		case IDC_THEME_LOAD:
			{
				TCHAR szFileName[MAX_PATH],szFilter[128],szInitDir[MAX_PATH];
				int Length;
				OPENFILENAME ofn;

				szFileName[0]='\0';
				GetModuleFileName(NULL,szInitDir,MAX_PATH);
				lstrcpy(GetFileNamePtr(szInitDir),THEMES_DIRECTORY);
				Length=LoadString(hInst,IDS_THEMEFILEFILTER,
												szFilter,numberof(szFilter)-1);
				szFilter[Length+1]='\0';	// for MS\[XRpC
				InitOpenFileName(&ofn);
				ofn.hwndOwner=hDlg;
				ofn.lpstrFilter=szFilter;
				ofn.nFilterIndex=1;
				ofn.lpstrFile=szFileName;
				ofn.nMaxFile=MAX_PATH;
				ofn.lpstrInitialDir=szInitDir;
				ofn.Flags=OFN_HIDEREADONLY | OFN_FILEMUSTEXIST | OFN_EXPLORER;
				if (GetOpenFileName(&ofn)) {
					int Sel;
					ColorInfo Color;

					if (!LoadTheme(hDlg,szFileName)) {
						ResMessageBox(hDlg,IDS_FAIL_LOADFILE,0,
												MB_OK | MB_ICONEXCLAMATION);
						return TRUE;
					}
					InvalidateDlgItem(hDlg,IDC_THEME_COLORS_LIST);
					Sel=SendDlgItemMessage(hDlg,IDC_THEME_COLORS_LIST,
															LB_GETCURSEL,0,0);
					Color=(ColorInfo)SendDlgItemMessage(hDlg,
								IDC_THEME_COLORS_LIST,LB_GETITEMDATA,Sel,0);
					ColorsComboBox_SetColor(hDlg,
											IDC_THEME_COLORS_COLORLIST,Color);
					EnableDlgItem(hDlg,IDC_THEME_COLORS_CHOOSE,
								ColorInfo_GetType(Color)==COLOR_TYPE_CUSTOM);
				}
			}
			return TRUE;
#endif
		case IDC_THEME_SAVE:
			{
				TCHAR szFileName[MAX_PATH],szFilter[128],szThemesDir[MAX_PATH];
				int Length;
				OPENFILENAME ofn;

				szFileName[0]='\0';
				GetThemesDirectory(szThemesDir);
				Length=LoadString(hInst,IDS_THEMEFILEFILTER,
												szFilter,numberof(szFilter)-1);
				szFilter[Length+1]='\0';	// for MS\[XRpC
				InitOpenFileName(&ofn);
				ofn.hwndOwner=hDlg;
				ofn.lpstrFilter=szFilter;
				ofn.nFilterIndex=1;
				ofn.lpstrFile=szFileName;
				ofn.nMaxFile=MAX_PATH;
				ofn.lpstrInitialDir=szThemesDir;
				ofn.Flags=OFN_HIDEREADONLY | OFN_EXPLORER |
														OFN_OVERWRITEPROMPT;
				ofn.lpstrDefExt=TEXT("lptheme");
				if (GetSaveFileName(&ofn)) {
					TCHAR szDir[MAX_PATH];

					if (!SaveTheme(hDlg,szFileName)) {
						ResMessageBox(hDlg,IDS_FAIL_SAVEFILE,0,
												MB_OK | MB_ICONEXCLAMATION);
						return TRUE;
					}
					GetFileDirectory(szDir,szFileName);
					if (lstrcmpi(szDir,szThemesDir)==0) {
						int Count,i;
						TCHAR szSaveFile[MAX_PATH],szFile[MAX_PATH];
						int Cmp;

						Count=SendDlgItemMessage(hDlg,IDC_THEME_LIST,
															CB_GETCOUNT,0,0);
						lstrcpy(szSaveFile,GetFileNamePtr(szFileName));
						*GetFileExtensionPtr(szSaveFile)='\0';
						for (i=1;i<Count;i++) {
							SendDlgItemMessage(hDlg,IDC_THEME_LIST,
												CB_GETLBTEXT,i,(LPARAM)szFile);
							Cmp=lstrcmpi(szFile,szSaveFile);
							if (Cmp>=0)
								break;
						}
						if (i==Count || Cmp>0)
							SendDlgItemMessage(hDlg,IDC_THEME_LIST,
										CB_INSERTSTRING,i,(LPARAM)szSaveFile);
						SendDlgItemMessage(hDlg,IDC_THEME_LIST,
															CB_SETCURSEL,i,0);
						EnableDlgItem(hDlg,IDC_THEME_DELETE,TRUE);
					}
				}
			}
			return TRUE;
		}
		return TRUE;

	case WM_NOTIFY:
		switch (((LPNMHDR)lParam)->code) {
		case LVN_ITEMCHANGED:
			{
				int Sel=ListView_GetNextItem(((LPNMHDR)lParam)->hwndFrom,-1,
																LVNI_SELECTED);

				if (himlPreview!=NULL) {
					ImageList_Destroy(himlPreview);
					himlPreview=NULL;
				}
				if (Sel==0) {
					himlPreview=ImageList_LoadImage(hInst,
											MAKEINTRESOURCE(IDB_TOOLBAR),16,1,
							RGB(255,0,255),IMAGE_BITMAP,LR_CREATEDIBSECTION);
				} else if (Sel>0) {
					const ThemePlugin *pPlugin=pThemePlugins->GetPlugin(Sel-1);

					himlPreview=pPlugin->LoadImageList(LP_TOOLBAR_TYPE_TOOLBAR,
											ThemePlugin::BUTTON_STAT_NORMAL);
				}
				InvalidateDlgItem(hDlg,IDC_THEME_TOOLBAR_PREVIEW);
				if (!fLoading
						&& (int)SendDlgItemMessage(hDlg,IDC_THEME_LIST,
														CB_GETCURSEL,0,0)>0) {
					SendDlgItemMessage(hDlg,IDC_THEME_LIST,CB_SETCURSEL,0,0);
					EnableDlgItem(hDlg,IDC_THEME_DELETE,FALSE);
				}
			}
			return TRUE;
		case PSN_APPLY:
			// Themes
			{
				int Sel=SendDlgItemMessage(hDlg,IDC_THEME_LIST,CB_GETCURSEL,0,0);

				if (Sel>0) {
					SendDlgItemMessage(hDlg,IDC_THEME_LIST,CB_GETLBTEXT,Sel,
												(LPARAM)szCurThemeFileName);
					lstrcat(szCurThemeFileName,TEXT(".lptheme"));
				} else
					szCurThemeFileName[0]='\0';
			}
			// Color scheme
			{
				int i;

				for (i=0;i<numberof(ColorSchemeList);i++)
					ColorSchemeList[i].Color=(ColorInfo)SendDlgItemMessage(
								hDlg,IDC_THEME_COLORS_LIST,LB_GETITEMDATA,i,0);
				ApplyColorScheme();
			}
			// Toolbar
			{
				int Sel=ListView_GetNextItem(
					GetDlgItem(hDlg,IDC_THEME_TOOLBAR_LIST),-1,LVNI_SELECTED);

				if (Sel>=0 && (Sel>0 || MainToolbarInfo.himl!=NULL)) {
					HBITMAP hbmLogo;

					MainToolbarInfo.himl=NULL;
					MainToolbarInfo.himlHot=NULL;
					MainToolbarInfo.himlDisabled=NULL;
					PlayBarToolbarInfo.himl=NULL;
					PlayBarToolbarInfo.himlHot=NULL;
					PlayBarToolbarInfo.himlDisabled=NULL;
					PlaylistToolbarInfo.himl=NULL;
					PlaylistToolbarInfo.himlHot=NULL;
					PlaylistToolbarInfo.himlDisabled=NULL;
					if (Sel==0) {
						hbmLogo=NULL;
						szThemePluginFileName[0]='\0';
					} else {
						const ThemePlugin *pPlugin=
											pThemePlugins->GetPlugin(Sel-1);
						ThemePluginInfo PluginInfo;

						pPlugin->GetInfo(&PluginInfo);
						if ((PluginInfo.Flags&LP_THEME_FLAG_TOOLBAR)!=0)
							LoadToolbarImages(pPlugin,LP_TOOLBAR_TYPE_TOOLBAR,
															&MainToolbarInfo);
						if ((PluginInfo.Flags&LP_THEME_FLAG_PLAYBAR)!=0)
							LoadToolbarImages(pPlugin,LP_TOOLBAR_TYPE_PLAYBAR,
														&PlayBarToolbarInfo);
						if ((PluginInfo.Flags&LP_THEME_FLAG_PLAYLIST)!=0)
							LoadToolbarImages(pPlugin,LP_TOOLBAR_TYPE_PLAYLIST,
														&PlaylistToolbarInfo);
						if ((PluginInfo.Flags&LP_THEME_FLAG_LOGO)!=0)
							hbmLogo=pPlugin->LoadBitmap(LP_IMAGE_TYPE_LOGO);
						lstrcpy(szThemePluginFileName,
										GetFileNamePtr(pPlugin->FileName()));
					}
					HWND hwnd=CreateMainToolbar(hwndMain);
					if (hwnd!=NULL) {
						DestroyWindow(hwndToolbar);
						hwndToolbar=hwnd;
						SetMenuBar();
						if (fShowToolbar) {
							RECT rc;

							GetClientRect(hwndMain,&rc);
							SendMessage(hwndMain,WM_SIZE,0,
											MAKELPARAM(rc.right,rc.bottom));
						}
					}
					pMediaPlayer->SetPlayBarImage(
											PlayBarToolbarInfo.himl,
											PlayBarToolbarInfo.himlHot,
											PlayBarToolbarInfo.himlDisabled);
					pPlaylist->SetToolbarImage(
											PlaylistToolbarInfo.himl,
											PlaylistToolbarInfo.himlHot,
											PlaylistToolbarInfo.himlDisabled);
					pMediaPlayer->SetLogo(hbmLogo);
				}
			}
			break;
		case PSN_RESET:
			if (pOldColorScheme!=NULL) {
				int i;

				for (i=0;i<numberof(ColorSchemeList);i++)
					ColorSchemeList[i].Color=pOldColorScheme[i];
				ApplyColorScheme();
			}
			break;
		case PSN_SETACTIVE:
			OptionsStartPage=4;
			break;
		case PSN_HELP:
			ShowHelp(IDH_OPTIONS_THEME);
			break;
		}
		break;

	case WM_DESTROY:
		if (pOldColorScheme!=NULL) {
			delete [] pOldColorScheme;
			pOldColorScheme=NULL;
		}
		delete pThemePlugins;
		if (himlPreview!=NULL) {
			ImageList_Destroy(himlPreview);
			himlPreview=NULL;
		}
		return TRUE;
	}
	return FALSE;
}


/*
	ݒ_CAO\
*/
static BOOL OptionsDialog(HWND hwndOwner)
{
	PROPSHEETPAGE psp[6];
	PROPSHEETHEADER psh;
	int i;

	// vpeBV[gy[W̐ݒ
	for (i=0;i<numberof(psp);i++) {
		psp[i].dwSize=sizeof(PROPSHEETPAGE);
		psp[i].dwFlags=PSP_DEFAULT | PSP_HASHELP;
		psp[i].hInstance=hInst;
	}
	psp[0].UNION_MEMBER(u) pszTemplate=MAKEINTRESOURCE(IDD_OPTIONS_GENERAL);
	psp[0].pfnDlgProc=GeneralOptionsDlgProc;
	psp[1].UNION_MEMBER(u) pszTemplate=MAKEINTRESOURCE(IDD_OPTIONS_HISTORY);
	psp[1].pfnDlgProc=HistoryOptionsDlgProc;
	psp[2].UNION_MEMBER(u) pszTemplate=MAKEINTRESOURCE(IDD_OPTIONS_PLAYER);
	psp[2].pfnDlgProc=PlayerOptionsDlgProc;
	psp[3].UNION_MEMBER(u) pszTemplate=MAKEINTRESOURCE(IDD_OPTIONS_PLAYLIST);
	psp[3].pfnDlgProc=PlaylistOptionsDlgProc;
	psp[4].UNION_MEMBER(u) pszTemplate=MAKEINTRESOURCE(IDD_OPTIONS_THEME);
	psp[4].pfnDlgProc=ThemeOptionsDlgProc;
	psp[5].UNION_MEMBER(u) pszTemplate=MAKEINTRESOURCE(IDD_OPTIONS_ACCEL);
	psp[5].pfnDlgProc=AccelOptionsDlgProc;
	// vpeBV[g̐ݒ
	psh.dwSize=sizeof(PROPSHEETHEADER);
	psh.dwFlags=PSH_NOAPPLYNOW | PSH_PROPSHEETPAGE | PSH_NOCONTEXTHELP;
	psh.hwndParent=hwndOwner;
	psh.hInstance=hInst;
	psh.pszCaption=TEXT("ݒ");
	psh.nPages=numberof(psp);
	psh.UNION_MEMBER(u2) nStartPage=OptionsStartPage;
	psh.UNION_MEMBER(u3) ppsp=psp;
	PropertySheet(&psh);
	return TRUE;
}


static WNDPROC pfnOldWebLinkProc;

LRESULT CALLBACK WebLinkProc(HWND hwnd,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	if (uMsg==WM_SETCURSOR) {
		SetCursor(LoadCursor(NULL,IDC_HAND));
		return 0;
	}
	return CallWindowProc(pfnOldWebLinkProc,hwnd,uMsg,wParam,lParam);
}


BOOL CALLBACK AboutDlgProc(HWND hDlg,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	static HFONT hfontLink;

	switch (uMsg) {
	case WM_INITDIALOG:
		{
			LOGFONT lf;

			GetObject((HFONT)SendMessage(hDlg,WM_GETFONT,0,0),
														sizeof(LOGFONT),&lf);
			lf.lfUnderline=1;
			hfontLink=CreateFontIndirect(&lf);
			SendDlgItemMessage(hDlg,IDC_ABOUT_WEB,WM_SETFONT,
													(WPARAM)hfontLink,FALSE);
			pfnOldWebLinkProc=(WNDPROC)SetWindowLongPtr(
								GetDlgItem(hDlg,IDC_ABOUT_WEB),GWLP_WNDPROC,
														(LONG_PTR)WebLinkProc);
		}
		return TRUE;

	case WM_CTLCOLORSTATIC:
		if ((HWND)lParam==GetDlgItem(hDlg,IDC_ABOUT_WEB)) {
			SetTextColor((HDC)wParam,RGB(0,0,255));
			SetBkMode((HDC)wParam,TRANSPARENT);
			return (BOOL)(HBRUSH)GetStockObject(NULL_BRUSH);
		}
		break;

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDC_ABOUT_WEB:
			if (HIWORD(wParam)==STN_CLICKED)
				ShellExecute(NULL,TEXT("open"),
							TEXT("http://www.geocities.jp/iooiau/"),NULL,NULL,
																SW_SHOWNORMAL);
			return TRUE;
		case IDOK:
		case IDCANCEL:
			EndDialog(hDlg,LOWORD(wParam));
		}
		return TRUE;

	case WM_DESTROY:
		SetWindowLongPtr(GetDlgItem(hDlg,IDC_ABOUT_WEB),GWLP_WNDPROC,
												(LONG_PTR)pfnOldWebLinkProc);
		DeleteObject(hfontLink);
		return TRUE;
	}
	return FALSE;
}


static int GetDirFileListIndex(LPCTSTR pszFileName)
{
	int Index;

	if (!CreateDirFileList())
		return -1;
	pszFileName=GetFileNamePtr(pszFileName);
	Index=pDirFileList->GetFileIndex(pszFileName);
	if (Index<0) {
		if (!RefreshDirFileList())
			return -1;
		Index=pDirFileList->GetFileIndex(pszFileName);
	}
	return Index;
}


static void OnCommand(HWND hwnd,int nID,HWND hwndCtrl,UINT uNotifyCode)
{
	switch (nID) {
	case CM_OPEN:
		{
			TCHAR szFileName[MAX_PATH];

			if (pMediaPlayer->IsFullScreen())
				OnCommand(hwndMain,CM_FULLSCREEN,NULL,0);
			szFileName[0]='\0';
			if (FileOpenDialog(hwnd,szFileName,szFileOpenDirectory))
				OpenMediaFile(szFileName,FALSE);
		}
		break;
	case CM_CLOSE:
		CloseMediaFile();
		if (pMediaPlayer->IsFullScreen())
			OnCommand(hwndMain,CM_FULLSCREEN,NULL,0);
		break;
	case CM_SORT_NAME:
	case CM_SORT_NAME_NUMBER:
	case CM_SORT_TYPE:
	case CM_SORT_EXTENSION:
	case CM_SORT_SIZE:
	case CM_SORT_LASTWRITETIME:
		DirSortType=(DirectoryFileList::SortType)(nID-CM_SORT_FIRST);
		if (pDirFileList!=NULL)
			pDirFileList->Sort(DirSortType,fDirSortDescending);
		CheckMenuRadioItem(hmenuMain,CM_SORT_FIRST,CM_SORT_LAST,nID,
																MF_BYCOMMAND);
		break;
	case CM_SORTDESCENDING:
		fDirSortDescending=!fDirSortDescending;
		if (pDirFileList!=NULL)
			pDirFileList->Sort(DirSortType,fDirSortDescending);
		CheckMenuItem(hmenuMain,CM_SORTDESCENDING,
				MF_BYCOMMAND | (fDirSortDescending?MFS_CHECKED:MFS_UNCHECKED));
		break;
	case CM_PREVFILE:
		if (fOpenFromPlaylist) {
			int nPlaying,nCount,nIndex;
			TCHAR szFileName[MAX_PATH];

			nPlaying=pPlaylist->GetPlaying();
			nCount=pPlaylist->GetCount();
			if (nPlaying<0 || nCount<1)
				break;
			if (nPlaying>0)
				nIndex=nPlaying-1;
			else
				nIndex=nCount-1;
			if (pPlaylist->GetFile(nIndex,szFileName)) {
				if (OpenMediaFile(szFileName,TRUE))
					pPlaylist->SetPlaying(nIndex);
			}
		} else {
			TCHAR szFileName[MAX_PATH];
			int Index;

			pMediaPlayer->GetFileName(szFileName,numberof(szFileName));
			Index=GetDirFileListIndex(szFileName);
			if (Index<0)
				break;
			if (Index>0)
				Index--;
			else
				Index=pDirFileList->GetCount()-1;
			pDirFileList->GetFile(Index,szFileName);
			OpenMediaFile(szFileName);
		}
		break;
	case CM_NEXTFILE:
		if (fOpenFromPlaylist) {
			int nPlaying,nCount,nIndex;
			TCHAR szFileName[MAX_PATH];

			nPlaying=pPlaylist->GetPlaying();
			nCount=pPlaylist->GetCount();
			if (nPlaying<0 || nCount<1)
				break;
			nIndex=(nPlaying+1)%nCount;
			if (pPlaylist->GetFile(nIndex,szFileName)) {
				if (OpenMediaFile(szFileName,TRUE))
					pPlaylist->SetPlaying(nIndex);
			}
		} else {
			TCHAR szFileName[MAX_PATH];
			int Index;

			pMediaPlayer->GetFileName(szFileName,numberof(szFileName));
			Index=GetDirFileListIndex(szFileName);
			if (Index<0)
				break;
			if (Index<pDirFileList->GetCount()-1)
				Index++;
			else
				Index=0;
			pDirFileList->GetFile(Index,szFileName);
			OpenMediaFile(szFileName);
		}
		break;
	case CM_FIRSTFILE:
	case CM_LASTFILE:
		if (fOpenFromPlaylist) {
			int Index;
			TCHAR szFileName[MAX_PATH];

			if (nID==CM_FIRSTFILE)
				Index=0;
			else
				Index=pPlaylist->GetCount()-1;
			if (pPlaylist->GetFile(Index,szFileName)) {
				if (OpenMediaFile(szFileName,TRUE))
					pPlaylist->SetPlaying(Index);
			}
		} else {
			TCHAR szFileName[MAX_PATH];

			if (!CreateDirFileList())
				break;
			pDirFileList->GetFile(
					nID==CM_FIRSTFILE?0:pDirFileList->GetCount()-1,szFileName);
			OpenMediaFile(szFileName);
		}
		break;
	case CM_ADDTOPLAYLIST:
		{
			TCHAR szFileName[MAX_PATH];
			int Index;

			pMediaPlayer->GetFileName(szFileName,numberof(szFileName));
			Index=pPlaylist->GetCount();
			pPlaylist->AddFile(szFileName);
			pPlaylist->SetSel(Index);
			pPlaylist->VisibleFile(Index);
		}
		break;
	case CM_FOLDERPLAYLIST:
		if (CreateDirFileList()) {
			int i;
			TCHAR szFileName[MAX_PATH];

			pPlaylist->Clear();
			if (!fShowPlaylist)
				OnCommand(hwnd,CM_PLAYLIST,NULL,0);
			for (i=0;i<pDirFileList->GetCount();i++) {
				pDirFileList->GetFile(i,szFileName);
				pPlaylist->AddFile(szFileName);
			}
			pMediaPlayer->GetFileName(szFileName,numberof(szFileName));
			i=pPlaylist->FindFile(szFileName);
			pPlaylist->SetSel(i);
			pPlaylist->SetPlaying(i);
			if (i>=0) {
				pPlaylist->VisibleFile(i);
				fOpenFromPlaylist=true;
				SetMenuBar();
			}
		}
		break;
	case CM_REFRESH:
		DeleteDirFileList();
		break;
	case CM_EXIT:
		SendMessage(hwnd,WM_CLOSE,0,0);
		break;
/*
	case CM_PLAY:
	case CM_PAUSE:
*/
	case CM_PLAY_PAUSE:
	case CM_STOP:
	case CM_PREVFRAME:
	case CM_NEXTFRAME:
	case CM_BACKWARD:
	case CM_FORWARD:
	case CM_HOME:
	case CM_END:
		FORWARD_WM_COMMAND(pMediaPlayer->GetHandle(),nID,NULL,0,SendMessage);
		break;
	case CM_REPEAT:
		fPlayRepeat=!fPlayRepeat;
		CheckMenuItem(hmenuMain,CM_REPEAT,
					MF_BYCOMMAND | (fPlayRepeat?MFS_CHECKED:MFS_UNCHECKED));
		pMediaPlayer->SetRepeat(fPlayRepeat);
		break;
	case CM_VOLUMEUP:
		{
			int nVolume=pMediaPlayer->GetVolume();

			if (nVolume<1000)
				pMediaPlayer->SetVolume(min(nVolume+100,1000));
		}
		break;
	case CM_VOLUMEDOWN:
		{
			int nVolume=pMediaPlayer->GetVolume();

			if (nVolume>0)
				pMediaPlayer->SetVolume(max(nVolume-100,0));
		}
		break;
	case CM_MUTE:
		pMediaPlayer->Mute(!pMediaPlayer->IsMute());
		CheckMenuItem(hmenuMain,CM_MUTE,
			MF_BYCOMMAND | (pMediaPlayer->IsMute()?MFS_CHECKED:MFS_UNCHECKED));
		break;
	case CM_SEQUENTIALPLAY:
		fSequentialPlay=!fSequentialPlay;
		CheckMenuItem(hmenuMain,CM_SEQUENTIALPLAY,
				MF_BYCOMMAND | (fSequentialPlay?MFS_CHECKED:MFS_UNCHECKED));
		break;
	case CM_REPEATPLAYLIST:
		fRepeatPlaylist=!fRepeatPlaylist;
		CheckMenuItem(hmenuMain,CM_REPEATPLAYLIST,
				MF_BYCOMMAND | (fRepeatPlaylist?MFS_CHECKED:MFS_UNCHECKED));
		SendMessage(hwndToolbar,TB_CHECKBUTTON,CM_REPEATPLAYLIST,
															fRepeatPlaylist);
		break;
	case CM_ZOOMIN:
		if (!pMediaPlayer->IsFullScreen()) {
			int nZoom;

			if (fFitToWindow)
				OnCommand(hwnd,CM_FITTOWINDOW,NULL,0);
			nZoom=pMediaPlayer->GetZoom()+nZoomStep;
			pMediaPlayer->SetZoom(min(nZoom,MAX_ZOOM));
			AdjustWindowSize(hwnd);
			SetZoomStatus();
			SetZoomMenu();
		}
		break;
	case CM_ZOOMOUT:
		if (!pMediaPlayer->IsFullScreen()) {
			int nZoom;

			nZoom=pMediaPlayer->GetZoom()-nZoomStep;
			if (fFitToWindow)
				OnCommand(hwnd,CM_FITTOWINDOW,NULL,0);
			pMediaPlayer->SetZoom(max(nZoom,MIN_ZOOM));
			AdjustWindowSize(hwnd);
			SetZoomStatus();
			SetZoomMenu();
		}
		break;
	case CM_ZOOM50:
	case CM_ZOOM75:
	case CM_ZOOM100:
	case CM_ZOOM150:
	case CM_ZOOM200:
	case CM_ZOOM300:
		if (!pMediaPlayer->IsFullScreen()) {
			if (fFitToWindow)
				OnCommand(hwnd,CM_FITTOWINDOW,NULL,0);
			pMediaPlayer->SetZoom(ZoomRateList[nID-CM_ZOOM_FIRST]);
			AdjustWindowSize(hwnd);
			SetZoomStatus();
			SetZoomMenu();
		}
		break;
	case CM_ASPECT_DEFAULT:
	case CM_ASPECT_4X3:
	case CM_ASPECT_16X9:
		{
			int i=nID-CM_ASPECT_FIRST;

			pMediaPlayer->SetAspectRatio(
									AspectRatioList[i].x,AspectRatioList[i].y);
			AdjustWindowSize(hwnd);
			CheckMenuRadioItem(hmenuMain,CM_ASPECT_FIRST,CM_ASPECT_LAST,nID,
																MF_BYCOMMAND);
		}
		break;
	case CM_KEEPZOOMRATE:
		fKeepZoomRate=!fKeepZoomRate;
		CheckMenuItem(hmenuMain,CM_KEEPZOOMRATE,
				MF_BYCOMMAND | (fKeepZoomRate?MFS_CHECKED:MFS_UNCHECKED));
		if (/*fKeepZoomRate && */fFitToWindow) {
			fFitToWindow=false;
			pMediaPlayer->FitToWindow(FALSE);
			CheckMenuItem(hmenuMain,CM_FITTOWINDOW,
												MF_BYCOMMAND | MFS_UNCHECKED);
		}
		break;
	case CM_FITTOWINDOW:
		fFitToWindow=!fFitToWindow;
		pMediaPlayer->FitToWindow(fFitToWindow);
		CheckMenuItem(hmenuMain,CM_FITTOWINDOW,
					MF_BYCOMMAND | (fFitToWindow?MFS_CHECKED:MFS_UNCHECKED));
		if (/*fFitToWindow && */fKeepZoomRate) {
			fKeepZoomRate=false;
			CheckMenuItem(hmenuMain,CM_KEEPZOOMRATE,
												MF_BYCOMMAND | MFS_UNCHECKED);
		}
		break;
	case CM_FULLSCREEN:
		{
			bool fFullScreen=pMediaPlayer->IsFullScreen();

			if (fShowPlayBar && !fShowFullScreenPlayBar)
				pMediaPlayer->ShowPlayBar(fFullScreen);
			if (fShowSeekBar && !fShowFullScreenSeekBar)
				pMediaPlayer->ShowSeekBar(fFullScreen);
			pMediaPlayer->FullScreen(!fFullScreen);
			if (fFullScreen) {
				if (!fFitToWindow)
					AdjustWindowSize(hwndMain);
			}
			ShowWindow(hwndMain,fFullScreen?SW_SHOW:SW_HIDE);
			CheckMenuItem(hmenuMain,CM_FULLSCREEN,MF_BYCOMMAND |
									(!fFullScreen?MFS_CHECKED:MFS_UNCHECKED));
		}
		break;
	case CM_ALWAYSONTOP:
		fAlwaysOnTop=!fAlwaysOnTop;
		SetWindowPos(hwnd,fAlwaysOnTop?HWND_TOPMOST:HWND_NOTOPMOST,
											0,0,0,0,SWP_NOMOVE | SWP_NOSIZE);
		CheckMenuItem(hmenuMain,CM_ALWAYSONTOP,
					MF_BYCOMMAND | (fAlwaysOnTop?MFS_CHECKED:MFS_UNCHECKED));
		SendMessage(hwndToolbar,TB_CHECKBUTTON,CM_ALWAYSONTOP,fAlwaysOnTop);
		break;
	case CM_PLAYLIST:
		fShowPlaylist=!fShowPlaylist;
		CheckMenuItem(hmenuMain,CM_PLAYLIST,
					MF_BYCOMMAND | (fShowPlaylist?MFS_CHECKED:MFS_UNCHECKED));
		SendMessage(hwndToolbar,TB_CHECKBUTTON,CM_PLAYLIST,fShowPlaylist);
		pPlaylist->Show(fShowPlaylist);
		break;
	case CM_MENUBAR:
		fShowMenuBar=!fShowMenuBar;
		SetMenu(hwnd,fShowMenuBar?hmenuMain:NULL);
		CheckMenuItem(hmenuMain,CM_MENUBAR,
					MF_BYCOMMAND | (fShowMenuBar?MFS_CHECKED:MFS_UNCHECKED));
		break;
	case CM_TOOLBAR:
		{
			RECT rc;

			fShowToolbar=!fShowToolbar;
			CheckMenuItem(hmenuMain,CM_TOOLBAR,
					MF_BYCOMMAND | (fShowToolbar?MFS_CHECKED:MFS_UNCHECKED));
			ShowWindow(hwndToolbar,fShowToolbar?SW_SHOW:SW_HIDE);
			GetClientRect(hwnd,&rc);
			OnSize(hwnd,0,rc.right,rc.bottom);
		}
		break;
	case CM_STATUSBAR:
		{
			RECT rc;

			fShowStatusBar=!fShowStatusBar;
			CheckMenuItem(hmenuMain,CM_STATUSBAR,
					MF_BYCOMMAND | (fShowStatusBar?MFS_CHECKED:MFS_UNCHECKED));
			//ShowWindow(hwndStatusBar,fShowStatusBar?SW_SHOW:SW_HIDE);
			pStatusBar->Show(fShowStatusBar);
			ShowWindow(hwndSizeGrip,fShowStatusBar?SW_SHOW:SW_HIDE);
			GetClientRect(hwnd,&rc);
			OnSize(hwnd,0,rc.right,rc.bottom);
		}
		break;
	case CM_PLAYBAR:
		if (!pMediaPlayer->IsFullScreen()) {
			fShowPlayBar=!fShowPlayBar;
			CheckMenuItem(hmenuMain,CM_PLAYBAR,
					MF_BYCOMMAND | (fShowPlayBar?MFS_CHECKED:MFS_UNCHECKED));
			pMediaPlayer->ShowPlayBar(fShowPlayBar);
		}
		break;
	case CM_SEEKBAR:
		if (!pMediaPlayer->IsFullScreen()) {
			fShowSeekBar=!fShowSeekBar;
			CheckMenuItem(hmenuMain,CM_SEEKBAR,
					MF_BYCOMMAND | (fShowSeekBar?MFS_CHECKED:MFS_UNCHECKED));
			pMediaPlayer->ShowSeekBar(fShowSeekBar);
		}
		break;
	case CM_PLAYBAR_FULLSCREEN:
		if (pMediaPlayer->IsFullScreen()) {
			fShowFullScreenPlayBar=!fShowFullScreenPlayBar;
			pMediaPlayer->ShowPlayBar(fShowFullScreenPlayBar);
		}
		break;
	case CM_SEEKBAR_FULLSCREEN:
		if (pMediaPlayer->IsFullScreen()) {
			fShowFullScreenSeekBar=!fShowFullScreenSeekBar;
			pMediaPlayer->ShowSeekBar(fShowFullScreenSeekBar);
		}
		break;
	case CM_OPTIONS:
		if (pMediaPlayer->IsFullScreen())
			OnCommand(hwndMain,CM_FULLSCREEN,NULL,0);
		OptionsDialog(hwnd);
		break;
	case CM_HELP:
		if (pMediaPlayer->IsFullScreen())
			OnCommand(hwndMain,CM_FULLSCREEN,NULL,0);
#ifndef USE_HTML_HELP
		WinHelp(hwnd,szHelpFileName,HELP_FINDER,0);
#else
		if (LoadHtmlHelpLibrary())
			(*pfnHtmlHelp)(NULL,szHelpFileName,HH_DISPLAY_TOC,0);
#endif
		break;
	case CM_ABOUT:
		if (pMediaPlayer->IsFullScreen())
			OnCommand(hwndMain,CM_FULLSCREEN,NULL,0);
		DialogBox(hInst,MAKEINTRESOURCE(IDD_ABOUT),hwnd,AboutDlgProc);
		break;
	case IDC_MEDIA:
		switch (uNotifyCode) {
		case MEDIAPLAYER_NOTIFY_ZOOMCHANGE:
			SetZoomStatus();
			SetZoomMenu();
			break;
		case MEDIAPLAYER_NOTIFY_END:
			if (!fOpenFromPlaylist) {
				if (fSequentialPlay) {
					TCHAR szFileName[MAX_PATH];
					int Index;

					pMediaPlayer->GetFileName(szFileName,numberof(szFileName));
					Index=GetDirFileListIndex(szFileName);
					if (Index<0)
						break;
					if (Index+1<pDirFileList->GetCount())
						OnCommand(hwnd,CM_NEXTFILE,NULL,0);
				}
			} else if (fRepeatPlaylist
						|| pPlaylist->GetPlaying()+1<pPlaylist->GetCount()) {
				// vCXg̎̃t@CJ
				OnCommand(hwnd,CM_NEXTFILE,NULL,0);
			}
			break;
		case MEDIAPLAYER_NOTIFY_REPEAT:
			fPlayRepeat=pMediaPlayer->GetRepeat();
			CheckMenuItem(hmenuMain,CM_REPEAT,
					MF_BYCOMMAND | (fPlayRepeat?MFS_CHECKED:MFS_UNCHECKED));
			break;
		case MEDIAPLAYER_NOTIFY_LBUTTONDOWN:
			if (fOpenFromPlaylist)
				OnCommand(hwnd,CM_NEXTFILE,NULL,0);
			else if (pMediaPlayer->IsFullScreen())
				OnCommand(hwnd,CM_FULLSCREEN,NULL,0);
			break;
		case MEDIAPLAYER_NOTIFY_RBUTTONDOWN:
			{
				POINT pt;

				GetCursorPos(&pt);
				PopupMenu(hwnd,pt.x,pt.y);
			}
			break;
		case MEDIAPLAYER_NOTIFY_PLAYBAR_RBUTTONDOWN:
		case MEDIAPLAYER_NOTIFY_SEEKBAR_RBUTTONDOWN:
			BarPopupMenu();
			break;
		case MEDIAPLAYER_NOTIFY_FULLSCREENEND:
			ShowWindow(hwndMain,SW_SHOW);
			if (fShowPlayBar && !fShowFullScreenPlayBar)
				pMediaPlayer->ShowPlayBar(TRUE);
			if (fShowSeekBar && !fShowFullScreenSeekBar)
				pMediaPlayer->ShowSeekBar(TRUE);
			CheckMenuItem(hmenuMain,CM_FULLSCREEN,
												MF_BYCOMMAND | MFS_UNCHECKED);
			break;
		}
		break;
	case IDC_STATUSBAR:
		if (uNotifyCode==LPStatusBar::NOTIFY_RBUTTONDOWN)
			BarPopupMenu();
		break;
	case CM_PLAYLIST_OPEN:
		{
			OPENFILENAME ofn;
			TCHAR szFileName[MAX_PATH];
			TCHAR szFilter[256];
			int Length;

			szFileName[0]='\0';
			Length=LoadString(hInst,IDS_FILELISTFILTER,szFilter,
														numberof(szFilter)-1);
			if (Length==0)
				break;
			szFilter[Length+1]='\0';	// for MS\[XRpC
			InitOpenFileName(&ofn);
			ofn.hwndOwner=pPlaylist->GetHandle();
			ofn.lpstrFilter=szFilter;
			ofn.lpstrFile=szFileName;
			ofn.nMaxFile=numberof(szFileName);
			ofn.lpstrInitialDir=szFileOpenDirectory;
			ofn.Flags=OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_EXPLORER;
			if (GetOpenFileName(&ofn)) {
				HCURSOR hcurOld;
				BOOL fResult;

				GetFileDirectory(szFileOpenDirectory,szFileName);
				hcurOld=SetCursor(LoadCursor(NULL,IDC_WAIT));
				fResult=pPlaylist->Load(szFileName);
				SetCursor(hcurOld);
				if (!fResult)
					ResMessageBox(hwnd,IDS_FAIL_LOADFILE,0,
												MB_OK | MB_ICONEXCLAMATION);
			}
		}
		break;
	case CM_PLAYLIST_SAVE:
		{
			OPENFILENAME ofn;
			TCHAR szFileName[MAX_PATH];
			TCHAR szFilter[256];
			int Length;

			szFileName[0]='\0';
			Length=LoadString(hInst,IDS_FILELISTFILTER,szFilter,
														numberof(szFilter)-1);
			if (Length==0)
				break;
			szFilter[Length+1]='\0';	// for MS\[XRpC
			InitOpenFileName(&ofn);
			ofn.hwndOwner=pPlaylist->GetHandle();
			ofn.lpstrFilter=szFilter;
			ofn.lpstrFile=szFileName;
			ofn.nMaxFile=numberof(szFileName);
			ofn.lpstrInitialDir=szFileOpenDirectory;
			ofn.Flags=OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT | OFN_EXPLORER;
			ofn.lpstrDefExt=TEXT("m3u");
			if (GetSaveFileName(&ofn)) {
				HCURSOR hcurOld;
				BOOL fResult;

				GetFileDirectory(szFileOpenDirectory,szFileName);
				hcurOld=SetCursor(LoadCursor(NULL,IDC_WAIT));
				fResult=pPlaylist->Save(szFileName);
				SetCursor(hcurOld);
				if (!fResult)
					ResMessageBox(hwnd,IDS_FAIL_SAVEFILE,0,
												MB_OK | MB_ICONEXCLAMATION);
			}
		}
		break;
	case CM_PLAYLIST_ADD:
		{
			TCHAR szFiles[MAX_PATH*100];
			OPENFILENAME ofn;
			TCHAR szFilter[1024];
			int Length;

			szFiles[0]='\0';
			Length=LoadString(hInst,IDS_OPENFILTER,
												szFilter,numberof(szFilter)-1);
			if (Length==0)
				return;
			szFilter[Length+1]='\0';	// for MS\[XRpC
			InitOpenFileName(&ofn);
			ofn.hwndOwner=pPlaylist->GetHandle();
			ofn.lpstrFilter=szFilter;
			ofn.nFilterIndex=nOpenFilterIndex;
			ofn.lpstrFile=szFiles;
			ofn.nMaxFile=numberof(szFiles);
			ofn.lpstrInitialDir=szFileOpenDirectory;
			ofn.lpstrTitle=TEXT("ǉt@C");
			ofn.Flags=OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_EXPLORER |
														OFN_ALLOWMULTISELECT;
			if (GetOpenFileName(&ofn)) {
				LPTSTR p;

				nOpenFilterIndex=ofn.nFilterIndex;
				p=szFiles+lstrlen(szFiles)+1;
				if (*p=='\0') {
					GetFileDirectory(szFileOpenDirectory,szFiles);
					pPlaylist->AddFile(szFiles);
				} else {
					TCHAR szFilePath[MAX_PATH];

					lstrcpy(szFileOpenDirectory,szFiles);
					for (;*p!='\0';p+=lstrlen(p)+1) {
						MergeFileName(szFilePath,szFiles,p);
						pPlaylist->AddFile(szFilePath);
					}
				}
			}
		}
		break;
	default:
		if (hwndCtrl==pPlaylist->GetHandle()) {
			switch (uNotifyCode) {
			case Playlist::NOTIFY_SELCHANGE:
				if (!fPlaylistSingleClickOpen ||
								pPlaylist->GetSel()==pPlaylist->GetPlaying())
					break;
				// Fall through
			case Playlist::NOTIFY_PLAY:
				// vCXg̃t@CJ
				{
					int nSel;
					TCHAR szFileName[MAX_PATH];

					nSel=pPlaylist->GetSel();
					if (pPlaylist->GetFile(nSel,szFileName)) {
						if (OpenMediaFile(szFileName,TRUE))
							pPlaylist->SetPlaying(nSel);
					}
				}
				break;
			case Playlist::NOTIFY_FILEDELETE:
				// JĂt@CvCXg폜ꂽ
				fOpenFromPlaylist=false;
				SetMenuBar();
				break;
			case Playlist::NOTIFY_CLOSE:
				FORWARD_WM_COMMAND(hwnd,CM_PLAYLIST,NULL,0,SendMessage);
				break;
			}
			return;
		}
		if (nID>=CM_FILEHISTORY_FIRST && nID<=CM_FILEHISTORY_LAST) {
			// t@CJ
			TCHAR szFileName[MAX_PATH];

			FileHistory.Get(nID-CM_FILEHISTORY_FIRST,szFileName);
			if (FileExists(szFileName)) {
				OpenMediaFile(szFileName,FALSE);
			} else {
				TCHAR szText[MAX_MESSAGE_LENGTH];
				TCHAR szMessage[MAX_MESSAGE_LENGTH+MAX_PATH];

				if (!LoadString(hInst,IDS_FILENOTFOUND,szText,
															numberof(szText)))
					break;
				wsprintf(szMessage,szText,szFileName);
				if (MessageBox(hwnd,szMessage,NULL,
						MB_YESNO | MB_ICONEXCLAMATION | MB_DEFBUTTON2)==IDYES)
					FileHistory.Remove(nID-CM_FILEHISTORY_FIRST);
			}
		} else if (nID>=CM_DIRHISTORY_FIRST && nID<=CM_DIRHISTORY_LAST) {
			// tH_J
			TCHAR szDirectory[MAX_PATH];

			DirHistory.Get(nID-CM_DIRHISTORY_FIRST,szDirectory);
			if (DirectoryExists(szDirectory)) {
				TCHAR szFileName[MAX_PATH];

				szFileName[0]='\0';
				if (FileOpenDialog(hwnd,szFileName,szDirectory))
					OpenMediaFile(szFileName,FALSE);
			} else {
				TCHAR szText[MAX_MESSAGE_LENGTH];
				TCHAR szMessage[MAX_MESSAGE_LENGTH+MAX_PATH];

				if (!LoadString(hInst,IDS_DIRNOTFOUND,szText,
															numberof(szText)))
					break;
				wsprintf(szMessage,szText,szDirectory);
				if (MessageBox(hwnd,szMessage,NULL,
						MB_YESNO | MB_ICONEXCLAMATION | MB_DEFBUTTON2)==IDYES)
					DirHistory.Remove(nID-CM_DIRHISTORY_FIRST);
			}
		} else if (nID>=CM_PRESET_FIRST && nID<=CM_PRESET_LAST) {
			unsigned int Flags=PresetList[nID-CM_PRESET_FIRST].Flags;
			bool f;

			if (pMediaPlayer->IsFullScreen())
				OnCommand(hwndMain,CM_FULLSCREEN,NULL,0);
			f=(Flags&PRESET_FLAG_MENUBAR)!=0;
			if (fShowMenuBar!=f)
				OnCommand(hwndMain,CM_MENUBAR,NULL,0);
			f=(Flags&PRESET_FLAG_TOOLBAR)!=0;
			if (fShowToolbar!=f)
				OnCommand(hwndMain,CM_TOOLBAR,NULL,0);
			f=(Flags&PRESET_FLAG_STATUSBAR)!=0;
			if (fShowStatusBar!=f)
				OnCommand(hwndMain,CM_STATUSBAR,NULL,0);
			f=(Flags&PRESET_FLAG_PLAYBAR)!=0;
			if (fShowPlayBar!=f)
				OnCommand(hwndMain,CM_PLAYBAR,NULL,0);
			f=(Flags&PRESET_FLAG_SEEKBAR)!=0;
			if (fShowSeekBar!=f)
				OnCommand(hwndMain,CM_SEEKBAR,NULL,0);
		} else if ((nID>=CM_PREVFILE_FIRST && nID<=CM_PREVFILE_LAST)
				|| (nID>=CM_NEXTFILE_FIRST && nID<=CM_NEXTFILE_LAST)) {
			int Index;
			TCHAR szFileName[MAX_PATH];

			if (nID>=CM_PREVFILE_FIRST && nID<=CM_PREVFILE_LAST)
				Index=FileListMenuCurFile-1-(nID-CM_PREVFILE_FIRST);
			else
				Index=FileListMenuCurFile+1+(nID-CM_NEXTFILE_FIRST);
			if (fOpenFromPlaylist)
				pPlaylist->GetFile(Index,szFileName);
			else
				pDirFileList->GetFile(Index,szFileName);
			if (OpenMediaFile(szFileName,fOpenFromPlaylist))
				if (fOpenFromPlaylist)
					pPlaylist->SetPlaying(Index);
		}
	}
}


static BOOL OnAppCommand(HWND hwnd,HWND /*hwndFrom*/,int nCommand,
											UINT /*uDevice*/,DWORD /*dwKeys*/)
{
#ifndef APPCOMMAND_OPEN
#define APPCOMMAND_OPEN					30
#define APPCOMMAND_CLOSE				31
#endif
#ifndef APPCOMMAND_MEDIA_PLAY
#define APPCOMMAND_MEDIA_PLAY			46
#define APPCOMMAND_MEDIA_PAUSE			47
#define APPCOMMAND_MEDIA_FAST_FORWARD	49
#define APPCOMMAND_MEDIA_REWIND			50
#endif

	static const struct {
		WORD AppCommand;
		WORD Command;
	} CommandTable [] = {
		{APPCOMMAND_VOLUME_MUTE,			CM_MUTE},
		{APPCOMMAND_VOLUME_DOWN,			CM_VOLUMEDOWN},
		{APPCOMMAND_VOLUME_UP,				CM_VOLUMEUP},
		{APPCOMMAND_MEDIA_NEXTTRACK,		CM_NEXTFILE},
		{APPCOMMAND_MEDIA_PREVIOUSTRACK,	CM_PREVFILE},
		{APPCOMMAND_MEDIA_STOP,				CM_STOP},
		{APPCOMMAND_MEDIA_PLAY_PAUSE,		CM_PLAY_PAUSE},
		{APPCOMMAND_OPEN,					CM_OPEN},
		{APPCOMMAND_CLOSE,					CM_CLOSE},
		{APPCOMMAND_MEDIA_PLAY,				CM_PLAY},
		{APPCOMMAND_MEDIA_PAUSE,			CM_PAUSE},
		{APPCOMMAND_MEDIA_FAST_FORWARD,		CM_FORWARD},
		{APPCOMMAND_MEDIA_REWIND,			CM_BACKWARD},
	};
	int i;

	for (i=0;i<numberof(CommandTable);i++) {
		if (nCommand==(int)CommandTable[i].AppCommand) {
			FORWARD_WM_COMMAND(hwnd,CommandTable[i].Command,NULL,0,
																SendMessage);
			break;
		}
	}
	return TRUE;
}


static void OnNotifyMode(HWND /*hwnd*/,MediaMode Mode)
{
	HMENU hmenu=hmenuMain;

	switch (Mode) {
	case MEDIA_MODE_UNDEFINED:
	case MEDIA_MODE_READY:
		/*
		SendMessage(hwndStatusBar,SB_SETTEXT,0,
											(LPARAM)TEXT("JĂ܂..."));
		*/
		//SendMessage(hwndStatusBar,SB_SETTEXT,0,(LPARAM)TEXT(""));
		pStatusBar->SetText(LPStatusBar::PART_MODE,TEXT(""));
		return;
	case MEDIA_MODE_PLAY:
		EnableMenuItem(hmenu,CM_STOP,MF_BYCOMMAND | MFS_ENABLED);
		if (pMediaPlayer->GetMediaType()==MEDIA_VIDEO) {
			EnableMenuItem(hmenu,CM_PREVFRAME,MF_BYCOMMAND | MFS_GRAYED);
			EnableMenuItem(hmenu,CM_NEXTFRAME,MF_BYCOMMAND | MFS_GRAYED);
		}
		//EnableMenuItem(hmenu,CM_BACKWARD,MF_BYCOMMAND | MFS_ENABLED);
		//EnableMenuItem(hmenu,CM_FORWARD,MF_BYCOMMAND | MFS_ENABLED);
		//SendMessage(hwndStatusBar,SB_SETTEXT,0,(LPARAM)TEXT("Đ"));
		pStatusBar->SetText(LPStatusBar::PART_MODE,TEXT("Đ"));
		break;
	case MEDIA_MODE_STOP:
	case MEDIA_MODE_PAUSE:
		EnableMenuItem(hmenu,CM_STOP,
				MF_BYCOMMAND | (Mode==MEDIA_MODE_STOP?MFS_GRAYED:MFS_ENABLED));
		if (pMediaPlayer->GetMediaType()==MEDIA_VIDEO) {
			EnableMenuItem(hmenu,CM_PREVFRAME,MF_BYCOMMAND | MFS_ENABLED);
			EnableMenuItem(hmenu,CM_NEXTFRAME,MF_BYCOMMAND | MFS_ENABLED);
		}
		//EnableMenuItem(hmenu,CM_BACKWARD,MF_BYCOMMAND | MFS_GRAYED);
		//EnableMenuItem(hmenu,CM_FORWARD,MF_BYCOMMAND | MFS_GRAYED);
#if 0
		SendMessage(hwndStatusBar,SB_SETTEXT,0,
			(LPARAM)(nMode==MCI_MODE_STOP?TEXT("~"):TEXT("ꎞ~")));
#endif
		pStatusBar->SetText(LPStatusBar::PART_MODE,
					Mode==MEDIA_MODE_STOP?TEXT("~"):TEXT("ꎞ~"));
		break;
	default:
		return;
	}
	CheckMenuItem(hmenu,CM_STOP,
			MF_BYCOMMAND | (Mode==MEDIA_MODE_STOP?MFS_CHECKED:MFS_UNCHECKED));
}


static void OnClose(HWND hwnd)
{
	// EBhËʒuƑ傫擾
	WINDOWPLACEMENT wp;
	wp.length=sizeof(WINDOWPLACEMENT);
	if (GetWindowPlacement(hwnd,&wp)) {
		nWindowLeft=wp.rcNormalPosition.left;
		nWindowTop=wp.rcNormalPosition.top;
		nWindowWidth=wp.rcNormalPosition.right-wp.rcNormalPosition.left;
		nWindowHeight=wp.rcNormalPosition.bottom-wp.rcNormalPosition.top;
		fWindowMaximized=wp.showCmd==SW_SHOWMAXIMIZED;
	}
	RECT rc;
	if (GetWindowRect(pPlaylist->GetHandle(),&rc)) {
		nPlaylistLeft=rc.left;
		nPlaylistTop=rc.top;
		nPlaylistWidth=rc.right-rc.left;
		nPlaylistHeight=rc.bottom-rc.top;
	}
	if (fPlaylistRestoreList)
		pPlaylist->SaveList();
	else
		pPlaylist->ClearSaveList();
	DefaultVolume=pMediaPlayer->GetVolume();
	DestroyWindow(hwndMain);
}


static void OnDestroy(HWND hwnd)
{
#ifndef USE_HTML_HELP
	WinHelp(hwnd,szHelpFileName,HELP_QUIT,0);
#else
	if (pfnHtmlHelp!=NULL) {
		(*pfnHtmlHelp)(NULL,NULL,HH_CLOSE_ALL,0);
		(*pfnHtmlHelp)(NULL,NULL,HH_UNINITIALIZE,dwHtmlHelpCookie);
		FreeLibrary(hHtmlHelpLib);
		hHtmlHelpLib=NULL;
		pfnHtmlHelp=NULL;
	}
	(void)hwnd;	// For warning
#endif
	// ݒ̕ۑ
	if (fUseUserProfile) {
		TCHAR szDir[MAX_PATH];

		GetUserProfileDirectory(szDir);
		if (!DirectoryExists(szDir))
			CreateDirectory(szDir,NULL);
	}
	SaveSettings(szIniFileName);
	pPlaylist->SaveHistory(szIniFileName);
	// n
	delete pStatusBar;
	delete pMediaPlayer;
	delete pPlaylist;
	// I
	PostQuitMessage(0);
}


LRESULT CALLBACK WndProc(HWND hwnd,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	switch (uMsg) {
	HANDLE_MSG(hwnd,WM_CREATE,OnCreate);
	HANDLE_MSG(hwnd,WM_SIZE,OnSize);
	HANDLE_MSG(hwnd,WM_NOTIFY,OnNotify);
	HANDLE_MSG(hwnd,WM_DROPFILES,OnDropFiles);
	HANDLE_MSG(hwnd,LAFPLAYER_MSG_FILEOPEN,OnFileOpen);
	// HANDLE_MSG(hwnd,WM_MENUSELECT,OnMenuSelect);
#if 0
	case WM_MENUSELECT:
		if (fShowStatusBar) {
			UINT uIDs[2]={0,0};

			if (LOWORD(wParam)>=CM_FILEHISTORY_FIRST
										&& LOWORD(wParam)<=CM_FILEHISTORY_LAST)
				wParam=MAKEWPARAM(CM_FILEHISTORY_FIRST,HIWORD(wParam));
			else if (LOWORD(wParam)>=CM_DIRHISTORY_FIRST
										&& LOWORD(wParam)<=CM_DIRHISTORY_LAST)
				wParam=MAKEWPARAM(CM_DIRHISTORY_FIRST,HIWORD(wParam));
			MenuHelp(uMsg,wParam,lParam,hmenuMain,hInst,hwndStatusBar,uIDs);
		}
		return 0;
#endif
	case WM_CONTEXTMENU:
		if (lParam==(LPARAM)0xFFFFFFFF) {
			RECT rc;

			pMediaPlayer->GetCanvasRect(&rc);
			PopupMenu(hwnd,rc.left,rc.top);
		}
		return 0;
	HANDLE_MSG(hwnd,WM_MEASUREITEM,OnMeasureItem);
	HANDLE_MSG(hwnd,WM_DRAWITEM,OnDrawItem);
	case WM_SYSCOLORCHANGE:
		ApplyColorScheme();
		return 0;
	HANDLE_MSG(hwnd,WM_COMMAND,OnCommand);
	case WM_APPCOMMAND:
		return OnAppCommand(hwnd,(HWND)wParam,GET_APPCOMMAND_LPARAM(lParam),
						GET_DEVICE_LPARAM(lParam),GET_KEYSTATE_LPARAM(lParam));
	case WM_MEDIA_NOTIFY:
		switch (wParam) {
		case MEDIA_NOTIFY_MODE:
			OnNotifyMode(hwnd,(MediaMode)lParam);
			break;
		case MEDIA_NOTIFY_POS:
			if (pMediaPlayer->IsOpen())
				SetTimeStatus(lParam);
			break;
		}
		return 0;
	HANDLE_MSG(hwnd,WM_CLOSE,OnClose);
	HANDLE_MSG(hwnd,WM_DESTROY,OnDestroy);
	}
	return DefWindowProc(hwnd,uMsg,wParam,lParam);
}


/*
	R}hC񂩂IvṼXg쐬

	:
		(i)pszCmdLine	R}hCւ̃|C^B
	߂l:
		쐬IvṼXgւ̃|C^ԂB
		eIvViŋ؂AŌɃi2t`ƂȂB
*/
static LPTSTR MakeCommandLineList(LPCTSTR pszCmdLine)
{
	LPTSTR pszCmdLineList,q;
	LPCTSTR p;

	pszCmdLineList=new TCHAR[lstrlen(pszCmdLine)+2];
	p=pszCmdLine;
	q=pszCmdLineList;
	while (*p!='\0') {
		if (*p==' ') {
			p++;
		} else {
			if (*p=='\"') {
				p++;
				while (*p!='\0' && *p!='\"')
					*q++=*p++;
				*q++='\0';
				if (*p=='\"')
					p++;
			} else {
				while (*p!='\0' && *p!=' ')
					*q++=*p++;
				*q++='\0';
			}
		}
	}
	if (q==pszCmdLineList)
		*q++='\0';
	*q='\0';
	return pszCmdLineList;
}


class AppMutexClass {
	HANDLE m_hMutex;
	bool m_fOwner;
public:
	AppMutexClass()
	{
		m_hMutex=CreateMutex(NULL,TRUE,TEXT("LafPlayer mutex"));
		m_fOwner=m_hMutex!=NULL && GetLastError()!=ERROR_ALREADY_EXISTS;
	}
	~AppMutexClass()
	{
		if (m_hMutex!=NULL) {
			if (m_fOwner)
				ReleaseMutex(m_hMutex);
			CloseHandle(m_hMutex);
		}
	}
	bool IsOwner() const { return m_fOwner; }
};


int APIENTRY _tWinMain(HINSTANCE hInstance,HINSTANCE /*hPrevInstance*/,
												LPTSTR pszCmdLine,int nCmdShow)
{
	AppMutexClass AppMutex;
	BOOL fFullScreen=FALSE;
	FileList *pCmdLineFiles=NULL;

	/* ϐ̏ */
	hInst=hInstance;
	// wvt@C̎擾
	GetModuleFileName(hInst,szHelpFileName,MAX_PATH);
	lstrcpy(GetFileNamePtr(szHelpFileName),
#ifndef USE_HTML_HELP
			TEXT("LafPlayer.hlp")
#else
			TEXT("LafPlayer.chm")
#endif
		);
	// }ChLgtH_擾
	ITEMIDLIST *pid;
	if (SHGetSpecialFolderLocation(NULL,CSIDL_PERSONAL,&pid)==NOERROR) {
		if (!SHGetPathFromIDList(pid,szFileOpenDirectory))
			GetCurrentDirectory(MAX_PATH,szFileOpenDirectory);
		CoTaskMemFree(pid);
	}
	if (szFileOpenDirectory[0]=='\0')
		GetCurrentDirectory(MAX_PATH,szFileOpenDirectory);

	/* INIt@C̏ꏊ擾 */
	if (GetUserProfileName(szIniFileName)) {
		if (FileExists(szIniFileName)) {
			fUseUserProfile=true;
		} else {
			TCHAR szCommonProfileName[MAX_PATH];

			GetCommonProfileName(szCommonProfileName);
			if (FileExists(szCommonProfileName)) {
				lstrcpy(szIniFileName,szCommonProfileName);
			} else {
				switch (ResMessageBox(NULL,IDS_CONFIRM_SETTINGS_DIR,
											IDS_CONFIRM_SETTINGS_DIR_CAPTION,
						MB_ICONQUESTION | MB_YESNOCANCEL | MB_DEFBUTTON2)) {
				case IDYES:
					fUseUserProfile=true;
					break;
				case IDNO:
					lstrcpy(szIniFileName,szCommonProfileName);
					break;
				default:
					return FALSE;
				}
			}
		}
	} else
		GetCommonProfileName(szIniFileName);

	/* ݒ̓ǂݍ */
	if (FileExists(szIniFileName))
		LoadSettings(szIniFileName);

	/* R}hC̎擾 */
	if (pszCmdLine[0]!='\0') {
		LPTSTR pszCmdLineList=MakeCommandLineList(pszCmdLine),p;

		pCmdLineFiles=new FileList;
		p=pszCmdLineList;
		while (*p!='\0') {
			if (*p=='-' || *p=='/') {
				switch ((TCHAR)CharUpper((LPTSTR)p[1])) {
				case 'F':
					fFullScreen=TRUE;
					break;
				case 'M':
					fWindowMaximized=true;
					break;
				}
			} else {
				TCHAR szFileName[MAX_PATH];

				GetLongPathName(p,szFileName,MAX_PATH);
				pCmdLineFiles->AddFile(szFileName);
			}
			p+=lstrlen(p)+1;
		}
		delete [] pszCmdLineList;
	}

	/* Ñ`FbN */
	if (fKeepSingleTask && !AppMutex.IsOwner()) {
		HWND hwnd=FindWindow(MAIN_WINDOW_CLASS,NULL);

		if (hwnd!=NULL) {
			if (IsIconic(hwnd))
				ShowWindow(hwnd,SW_RESTORE);
			else
				SetForegroundWindow(hwnd);
			if (pCmdLineFiles!=NULL) {
				int nFiles,i;
				ATOM atom;
				TCHAR szFileName[MAX_PATH];

				nFiles=pCmdLineFiles->GetCount();
				for (i=0;i<nFiles;i++) {
					pCmdLineFiles->GetFile(i,szFileName);
					atom=GlobalAddAtom(szFileName);
					if (atom==0)
						break;
					FORWARD_LAFPLAYER_MSG_FILEOPEN(hwnd,atom,i,nFiles,
																SendMessage);
					GlobalDeleteAtom(atom);
				}
				delete [] pCmdLineFiles;
			}
		}
		return 0;
	}

	/* EBhENX̓o^ */
	WNDCLASS wc;
	wc.style=0;
	wc.lpfnWndProc=WndProc;
	wc.cbClsExtra=0;
	wc.cbWndExtra=0;
	wc.hInstance=hInst;
	wc.hIcon=LoadIcon(hInst,MAKEINTRESOURCE(IDI_ICON));
	wc.hCursor=LoadCursor(NULL,IDC_ARROW);
	wc.hbrBackground=(HBRUSH)(COLOR_3DFACE+1);
	wc.lpszMenuName=NULL;
	wc.lpszClassName=MAIN_WINDOW_CLASS;
	if (RegisterClass(&wc)==0 || !LPStatusBar::Init()
			|| !MediaPlayer::Init(hInst) || !Playlist::Init(hInst)) {
		ResMessageBox(NULL,IDS_FAIL_REGISTERCLASS,0,MB_OK | MB_ICONSTOP);
		delete pCmdLineFiles;
		return FALSE;
	}

	CoInitialize(NULL);

	/* RRg[̏ */
#if _WIN32_IE<0x0300
	InitCommonControls();
#else
	INITCOMMONCONTROLSEX icc;

	icc.dwSize=sizeof(INITCOMMONCONTROLSEX);
	icc.dwICC=ICC_BAR_CLASSES | ICC_UPDOWN_CLASS | ICC_TREEVIEW_CLASSES |
														ICC_LISTVIEW_CLASSES;
	InitCommonControlsEx(&icc);
#endif

	/* Theme */
	if (szThemePluginFileName[0]!='\0') {
		TCHAR szFilePath[MAX_PATH];
		ThemePlugin Plugin;

		GetModuleFileName(hInst,szFilePath,MAX_PATH);
		wsprintf(GetFileNamePtr(szFilePath),THEMES_DIRECTORY TEXT("\\%s"),
														szThemePluginFileName);
		if (Plugin.Load(szFilePath)) {
			ThemePluginInfo PluginInfo;

			Plugin.GetInfo(&PluginInfo);
			if ((PluginInfo.Flags&LP_THEME_FLAG_TOOLBAR)!=0)
				LoadToolbarImages(&Plugin,LP_TOOLBAR_TYPE_TOOLBAR,
															&MainToolbarInfo);
			if ((PluginInfo.Flags&LP_THEME_FLAG_PLAYBAR)!=0)
				LoadToolbarImages(&Plugin,LP_TOOLBAR_TYPE_PLAYBAR,
														&PlayBarToolbarInfo);
			if ((PluginInfo.Flags&LP_THEME_FLAG_PLAYLIST)!=0)
				LoadToolbarImages(&Plugin,LP_TOOLBAR_TYPE_PLAYLIST,
														&PlaylistToolbarInfo);
		}
	}

	hmenuMain=LoadMenu(hInst,MAKEINTRESOURCE(IDM_MAIN));

	/* CEBhE̍쐬 */
	hwndMain=CreateWindowEx(WS_EX_ACCEPTFILES | (fAlwaysOnTop?WS_EX_TOPMOST:0),
							MAIN_WINDOW_CLASS,TITLE_TEXT,
							WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN,
							nWindowLeft,nWindowTop,nWindowWidth,nWindowHeight,
								NULL,fShowMenuBar?hmenuMain:NULL,hInst,NULL);
	if (hwndMain==NULL) {
		ResMessageBox(NULL,IDS_FAIL_CREATEWINDOW,0,MB_OK | MB_ICONSTOP);
		delete pCmdLineFiles;
		return FALSE;
	}
	ShowWindow(hwndMain,fWindowMaximized?SW_SHOWMAXIMIZED:nCmdShow);
	UpdateWindow(hwndMain);
	if (fShowPlaylist) {
		pPlaylist->Show(TRUE);
		UpdateWindow(pPlaylist->GetHandle());
		SetForegroundWindow(hwndMain);
	}

	/* j[̐ݒ */
	HMENU hmenu=hmenuMain;
	DeleteMenu(GetSubMenu(GetSubMenu(hmenu,FILE_MENU),FILE_HISTORY_MENU),0,
																MF_BYPOSITION);
	FileHistory.SetMenu(GetSubMenu(hmenu,FILE_MENU),FILE_HISTORY_MENU,
														CM_FILEHISTORY_FIRST);
	DeleteMenu(GetSubMenu(GetSubMenu(hmenu,FILE_MENU),DIR_HISTORY_MENU),0,
																MF_BYPOSITION);
	DirHistory.SetMenu(GetSubMenu(hmenu,FILE_MENU),DIR_HISTORY_MENU,
														CM_DIRHISTORY_FIRST);
	CheckMenuRadioItem(hmenu,CM_SORT_FIRST,CM_SORT_LAST,
									CM_SORT_FIRST+DirSortType,MF_BYCOMMAND);
	CheckMenuItem(hmenu,CM_SORTDESCENDING,
				MF_BYCOMMAND | (fDirSortDescending?MFS_CHECKED:MFS_UNCHECKED));
	CheckMenuItem(hmenu,CM_REPEAT,
					MF_BYCOMMAND | (fPlayRepeat?MFS_CHECKED:MFS_UNCHECKED));
	CheckMenuItem(hmenu,CM_SEQUENTIALPLAY,
				MF_BYCOMMAND | (fSequentialPlay?MFS_CHECKED:MFS_UNCHECKED));
	CheckMenuItem(hmenu,CM_REPEATPLAYLIST,
				MF_BYCOMMAND | (fRepeatPlaylist?MFS_CHECKED:MFS_UNCHECKED));
	//SendMessage(hwndToolbar,TB_CHECKBUTTON,CM_REPEATPLAYLIST,fRepeatPlaylist);
	CheckMenuItem(hmenu,CM_PLAYLIST,
					MF_BYCOMMAND | (fShowPlaylist?MFS_CHECKED:MFS_UNCHECKED));
	//SendMessage(hwndToolbar,TB_CHECKBUTTON,CM_PLAYLIST,fShowPlaylist);
	SetBarMenuStatus(hmenu);
	CheckMenuItem(hmenu,CM_KEEPZOOMRATE,
					MF_BYCOMMAND | (fKeepZoomRate?MFS_CHECKED:MFS_UNCHECKED));
	CheckMenuItem(hmenu,CM_FITTOWINDOW,
					MF_BYCOMMAND | (fFitToWindow?MFS_CHECKED:MFS_UNCHECKED));
	CheckMenuItem(hmenu,CM_ALWAYSONTOP,
					MF_BYCOMMAND | (fAlwaysOnTop?MFS_CHECKED:MFS_UNCHECKED));
	//SendMessage(hwndToolbar,TB_CHECKBUTTON,CM_ALWAYSONTOP,fAlwaysOnTop);
	SetPresetMenu();

	/* t@CEfBNg̓ǂݍ */
	if (FileExists(szIniFileName)) {
		FileHistory.Load(szIniFileName,TEXT("RecentFileList"),TEXT("File"));
		DirHistory.Load(szIniFileName,TEXT("RecentDirList"),TEXT("Dir"));
	}

	/* MCI̊gq擾 */
	if (szFileMasks[0]=='\0')
		GetMCIExtensions(szFileMasks);

	/* vCXg̗Ɠe̕ */
	pPlaylist->LoadHistory(szIniFileName);
	if (fPlaylistRestoreList)
		pPlaylist->LoadList();

	InitAccel(hmenuMain);

	/* R}hC̃t@CJ */
	if (pCmdLineFiles!=NULL) {
		TCHAR szFileName[MAX_PATH];

		if (pCmdLineFiles->GetCount()==1) {
			DWORD dwAttributes;

			pCmdLineFiles->GetFile(0,szFileName);
			dwAttributes=GetFileAttributes(szFileName);
			if (dwAttributes==0xFFFFFFFF)
				goto OpenCmdLineFileEnd;
			if ((dwAttributes&FILE_ATTRIBUTE_DIRECTORY)==0
					&& lstrcmpi(GetFileExtensionPtr(szFileName),TEXT(".m3u"))
																		!=0) {
				OpenMediaFile(szFileName,FALSE);
				goto OpenCmdLineFileEnd;
			}
		}
		if (!fShowPlaylist)
			OnCommand(hwndMain,CM_PLAYLIST,NULL,0);
		for (int i=0;i<pCmdLineFiles->GetCount();i++) {
			pCmdLineFiles->GetFile(i,szFileName);
			pPlaylist->AddFileOrDirectoryFiles(szFileName);
		}
	OpenCmdLineFileEnd:
		delete pCmdLineFiles;
	}

	/* R}hC -F w肳ĂtXN[ɂ */
	if (fFullScreen && pMediaPlayer->IsOpen())
		OnCommand(hwndMain,CM_FULLSCREEN,NULL,0);

	/* bZ[W[v */
	MSG msg;
	while (GetMessage(&msg,NULL,0,0)) {
#ifdef USE_HTML_HELP
		if (pfnHtmlHelp!=NULL && (*pfnHtmlHelp)(NULL,NULL,
										HH_PRETRANSLATEMESSAGE,(DWORD)&msg))
			continue;
#endif
		if (!TranslateAccel(hwndMain,&msg)) {
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	if (!fShowMenuBar)
		DestroyMenu(hmenuMain);
	FreeAccel();
	delete pDirFileList;

	CoUninitialize();

	return msg.wParam;
}
