/******************************************************************************
*                                                                             *
*    MP3.cpp                                Copyright(c) 2008-2009 itow,y.    *
*                                                                             *
******************************************************************************/

/*
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <windows.h>
#include <tchar.h>
#include "MP3.h"


class FileReader {
	HANDLE m_hFile;
public:
	enum {EOF=-1};
	FileReader() {m_hFile=INVALID_HANDLE_VALUE;}
	~FileReader() {Close();}
	void Close();
	bool Open(LPCTSTR pszFileName);
	DWORD Read(void *pBuffer,DWORD Size);
	int ReadByte();
	bool Seek(LONG Pos,DWORD Type);
	DWORD GetPos();
	DWORD GetSize();
};


void FileReader::Close()
{
	if (m_hFile!=INVALID_HANDLE_VALUE) {
		CloseHandle(m_hFile);
		m_hFile=INVALID_HANDLE_VALUE;
	}
}


bool FileReader::Open(LPCTSTR pszFileName)
{
	Close();
	m_hFile=CreateFile(pszFileName,GENERIC_READ,FILE_SHARE_READ,NULL,
									OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
	return m_hFile!=INVALID_HANDLE_VALUE;
}


DWORD FileReader::Read(void *pBuffer,DWORD Size)
{
	DWORD Read;

	if (!ReadFile(m_hFile,pBuffer,Size,&Read,NULL))
		return 0;
	return Read;
}


int FileReader::ReadByte()
{
	BYTE b;

	if (Read(&b,1)!=1)
		return EOF;
	return b;
}


bool FileReader::Seek(LONG Pos,DWORD Type)
{
	return SetFilePointer(m_hFile,Pos,NULL,Type)!=INVALID_SET_FILE_POINTER;
}


DWORD FileReader::GetPos()
{
	return SetFilePointer(m_hFile,0,NULL,FILE_CURRENT);
}


DWORD FileReader::GetSize()
{
	return GetFileSize(m_hFile,NULL);
}




bool GetMP3Info(LPCTSTR pszFileName,MP3Info *pInfo)
{
	FileReader File;
	BYTE Buffer[4];
	int b;
	DWORD HeaderPos;
	int Version,Layer;
	int Frequency;
	int Padding;
	MpegChannelMode ChannelMode;
	int Channels,BitRate,SamplesPerFrame,FrameSize,NumFrames;
	static const short BitRateTable1[3][16] = {
		{  0, 32, 40, 48, 56, 64, 80, 96,112,128,160,192,224,256,320,  0},
		{  0, 32, 48, 56, 64, 80, 96,112,128,160,192,224,256,320,384,  0},
		{  0, 32, 64, 96,128,160,192,224,256,288,320,352,384,416,448,  0},
	};
	static const short BitRateTable2[3][16] = {
		{  0,  8, 16, 24, 32, 40, 48, 56, 64, 80, 96,112,128,144,160,  0},
		{  0,  8, 16, 24, 32, 40, 48, 56, 64, 80, 96,112,128,144,160,  0},
		{  0, 32, 48, 56, 64, 80, 96,112,128,144,160,176,192,224,256,  0},
	};
	static const int FrequencyTable[4][4] = {
		{11025,12000, 8000,   0},	// MPEG 2.5
		{    0,    0,    0,   0},	// Reserved
		{22050,24000,16000,   0},	// MPEG 2
		{44100,48000,32000,   0},	// MPEG 1
	};
	static const short SamplesPerFrameTable[4][3] = {
		{ 576,1152, 384},	// MPEG 2.5
		{   0,   0,   0},	// Reserved
		{ 576,1152, 384},	// MPEG 2
		{1152,1152, 384},	// MPEG 1
	};

	if (!File.Open(pszFileName))
		return false;
	pInfo->Flags=0;
	if (File.Read(Buffer,3)!=3)
		return false;
	if (Buffer[0]=='I' && Buffer[1]=='D' && Buffer[2]=='3') {
		DWORD Size;

		if (File.Read(Buffer,3)!=3 || File.Read(Buffer,4)!=4)
			return false;
		Size=(Buffer[0]<<(7*3))|(Buffer[1]<<(7*2))|(Buffer[2]<<7)|Buffer[3];
		if (!File.Seek(Size,FILE_CURRENT))
			return false;
	} else
		File.Seek(0,FILE_BEGIN);
	bool fFindFrame=false;
	do {
		b=File.ReadByte();
		if (b==FileReader::EOF)
			break;
		if (b==0xFF) {
			b=File.ReadByte();
			if (b==FileReader::EOF)
				break;
			if ((b&0xE0)==0xE0) {
				Version=(b&0x18)>>3;
				Layer=(b&0x03)>>1;
				if (Version==MPEG_NOT || Layer==MPEG_LAYER_NOT)
					break;
				fFindFrame=true;
			}
		}
	} while (!fFindFrame);
	if (!fFindFrame)
		return false;
	HeaderPos=File.GetPos()-2;
	if ((b&1)==0)
		pInfo->Flags|=MPEG_FLAG_CRC;
	b=File.ReadByte();
	if (b==FileReader::EOF)
		return false;
	if (Version==MPEG_1)
		BitRate=BitRateTable1[Layer-1][b>>4];
	else
		BitRate=BitRateTable2[Layer-1][b>>4];
	BitRate*=1000;
	if ((b&0x0C)==0x0C)
		return false;
	Frequency=FrequencyTable[Version][(b&0x0C)>>2];
	Padding=(b&0x02)>>1;
	if (Padding==1)
		pInfo->Flags|=MPEG_FLAG_PADDING;
	if ((b&0x01)==1)
		pInfo->Flags|=MPEG_FLAG_PRIVATE;
	b=File.ReadByte();
	ChannelMode=(MpegChannelMode)((b&0xC0)>>6);
	Channels=ChannelMode==MPEG_CHANNEL_MONO?1:2;
	if((b&0x08)!=0)
		pInfo->Flags|=MPEG_FLAG_COPYRIGHTED;
	if ((b&0x04)!=0)
		pInfo->Flags|=MPEG_FLAG_ORIGINAL;
	/*
	// Emphasis
	switch (b&0x03){
	case 0:	printf("none\n");		break;
	case 1:	printf("50/15 ms\n");	break;
	case 2:	printf("reserved\n");	break;
	case 3:	printf("CCITT J.17\n");	break;
	}
	*/
	SamplesPerFrame=SamplesPerFrameTable[Version][Layer-1];
	FrameSize=(SamplesPerFrame/8*BitRate)/Frequency+Padding;
	NumFrames=(File.GetSize()-HeaderPos)/FrameSize;
	// Read Xing header
	while (true) {
		b=File.ReadByte();
	Next:
		if (b==FileReader::EOF)
			break;
		if (b==0xFF) {
			b=File.ReadByte();
			if (b==FileReader::EOF || (b&0xE0)==0xE0)
				break;
		}
		if (b=='X') {
			b=File.ReadByte();
			if (b!='i')
				goto Next;
			b=File.ReadByte();
			if (b!='n')
				goto Next;
			b=File.ReadByte();
			if (b!='g')
				goto Next;
			if (File.Read(Buffer,4)!=4)
				break;
			if ((Buffer[3]&0x03)==0x03) {
#define MSBFirst4(a) (((DWORD)a[0]<<24)|(a[1]<<16)|(a[2]<<8)|a[3])
				DWORD Bytes,Sec;

				if (File.Read(Buffer,4)!=4)
					break;
				NumFrames=MSBFirst4(Buffer);
				if (File.Read(Buffer,4)!=4)
					break;
				Bytes=MSBFirst4(Buffer);
				Sec=NumFrames*SamplesPerFrame/Frequency;
				if (Sec!=0)
					BitRate=Bytes*8/Sec;
				else
					BitRate=0;
			}
			break;
		}
	}
	// song length...
	/*
	double temp,msec;
	int Allsec,min,sec;
	temp   = (double)(n_Frame * smps) / (double)mp3Freq;
	Allsec = (n_Frame * smps) / mp3Freq;
	min    = Allsec/60;
	sec    = Allsec%60;
	msec   = (temp - Allsec)*1000;
	if(msec > 500) sec += 1;
	printf("        Length : (%d:%d)\n",min,sec);
	*/
	pInfo->Version=(MpegVersion)Version;
	pInfo->Layer=(MpegLayer)Layer;
	pInfo->ChannelMode=ChannelMode;
	pInfo->Channels=Channels;
	pInfo->BitRate=BitRate;
	pInfo->Frequency=Frequency;
	pInfo->FrameSize=FrameSize;
	pInfo->NumFrames=NumFrames;
	return true;
}
