/******************************************************************************
*                                                                             *
*    MediaDirectShow.cpp                    Copyright(c) 2008-2009 itow,y.    *
*                                                                             *
******************************************************************************/

/*
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <windows.h>
#include <windowsx.h>
#include <tchar.h>
#include <math.h>	// for log10()
#include "LafPlayer.h"
#include "MediaDirectShow.h"
#include "Util.h"
#include "resource.h"


#define DIRECTSHOW_MEDIA_FORM_WINDOW_CLASS TEXT("LafPlayer DirectShow Form")

#define WM_MEDIA_EVENT (WM_APP+1)

#define POS_TIMER_ID 1
#define POS_TIMER_INTERVAL 200

#define RELEASE(p) if ((p)!=NULL) { (p)->Release(); (p)=NULL; }

#define MediaPtr(hwnd) ((DirectShowMedia*)GetWindowLongPtr(hwnd,0))




bool DirectShowMedia::Init(HINSTANCE hinst)
{
	static bool fInitialized=false;
	WNDCLASS wc;

	if (fInitialized)
		return true;
	wc.style=0;
	wc.lpfnWndProc=FormWndProc;
	wc.cbClsExtra=0;
	wc.cbWndExtra=sizeof(DirectShowMedia*);
	wc.hInstance=hinst;
	wc.hIcon=NULL;
	wc.hCursor=NULL;//LoadCursor(NULL,IDC_ARROW);
	wc.hbrBackground=CreateSolidBrush(RGB(0,0,0));
	wc.lpszMenuName=NULL;
	wc.lpszClassName=DIRECTSHOW_MEDIA_FORM_WINDOW_CLASS;
	if (RegisterClass(&wc)==0)
		return false;
	fInitialized=true;
	return true;
}


bool DirectShowMedia::SetRate(int Rate)
{
#if 0
	IMediaPosition *pMediaPosition;
	double CurRate,NewRate;
	HRESULT hr;

	if (FAILED(m_pGraphBuilder->QueryInterface(IID_IMediaPosition,
													(void**)&pMediaPosition)))
		return false;
	pMediaPosition->get_Rate(&CurRate);
	NewRate=(double)Rate/1000.0;
	if (CurRate==NewRate)
		return true;
	hr=pMediaPosition->put_Rate(NewRate);
	pMediaPosition->Release();
	return SUCCEEDED(hr);
#else
	double CurRate,NewRate;

	m_pMediaSeeking->GetRate(&CurRate);
	NewRate=(double)Rate/1000.0;
	if (CurRate==NewRate)
		return true;
	return SUCCEEDED(m_pMediaSeeking->SetRate(NewRate));
#endif
}


bool DirectShowMedia::GetRange(MediaPosition *pStart,MediaPosition *pEnd)
{
	LONGLONG Earliest,Latest;

	if (FAILED(m_pMediaSeeking->GetAvailable(&Earliest,&Latest)))
		return false;
	*pStart=(MediaPosition)(Earliest/10000);
	*pEnd=(MediaPosition)(Latest/10000);
	return true;
}


LRESULT CALLBACK DirectShowMedia::FormWndProc(HWND hwnd,UINT uMsg,
												WPARAM wParam,LPARAM lParam)
{
	switch (uMsg) {
	case WM_CREATE:
		{
			LPCREATESTRUCT pcs=(LPCREATESTRUCT)lParam;
			DirectShowMedia *pMedia;

			pMedia=(DirectShowMedia*)pcs->lpCreateParams;
			SetWindowLongPtr(hwnd,0,(LONG_PTR)pMedia);
		}
		return 0;

	case WM_SIZE:
		{
			DirectShowMedia *pMedia=MediaPtr(hwnd);

			if (pMedia->m_pVideoWindow!=NULL) {
				IBasicVideo *pBasicVideo;

				/*
				pMedia->m_pVideoWindow->put_Left(0);
				pMedia->m_pVideoWindow->put_Top(0);
				pMedia->m_pVideoWindow->put_Width(LOWORD(lParam));
				pMedia->m_pVideoWindow->put_Height(HIWORD(lParam));
				*/
				pMedia->m_pVideoWindow->SetWindowPosition(0,0,
												LOWORD(lParam),HIWORD(lParam));
				if (SUCCEEDED(pMedia->m_pGraphBuilder->QueryInterface(
									IID_IBasicVideo,(void**)&pBasicVideo))) {
					pBasicVideo->SetDestinationPosition(0,0,
												LOWORD(lParam),HIWORD(lParam));
					pBasicVideo->Release();
				}
			}
		}
		return 0;

	case WM_TIMER:
		{
			DirectShowMedia *pMedia=MediaPtr(hwnd);

			pMedia->SendNotify(MEDIA_NOTIFY_POS,pMedia->GetPosition());
		}
		return 0;

	case WM_SETCURSOR:
		if (LOWORD(lParam)==HTCLIENT)
			return SendMessage(GetParent(hwnd),WM_SETCURSOR,wParam,lParam);
		break;

	case WM_LBUTTONDOWN:
		MediaPtr(hwnd)->SendNotify(MEDIA_NOTIFY_LBUTTONDOWN,0);
		return 0;

	case WM_RBUTTONDOWN:
		MediaPtr(hwnd)->SendNotify(MEDIA_NOTIFY_RBUTTONDOWN,0);
		return 0;

	case WM_MOUSEMOVE:
		{
			DirectShowMedia *pMedia=MediaPtr(hwnd);
			POINT pt;

			pt.x=GET_X_LPARAM(lParam);
			pt.y=GET_Y_LPARAM(lParam);
			MapWindowPoints(hwnd,GetParent(hwnd),&pt,1);
			SendMessage(GetParent(hwnd),WM_MOUSEMOVE,0,MAKELPARAM(pt.x,pt.y));
		}
		return 0;

	case WM_MEDIA_EVENT:
		{
			DirectShowMedia *pMedia=MediaPtr(hwnd);
			long evCode;
			LONG_PTR Param1,Param2;

			while (SUCCEEDED(pMedia->m_pMediaEvent->GetEvent(&evCode,
														&Param1,&Param2,0))) {
				switch (evCode) {
				case EC_OPENING_FILE:
					pMedia->m_Mode=Param1!=0?MEDIA_MODE_READY:MEDIA_MODE_STOP;
					pMedia->SendNotify(MEDIA_NOTIFY_MODE,
													(LPARAM)pMedia->m_Mode);
					break;
				case EC_COMPLETE:
					if (pMedia->m_Mode!=MEDIA_MODE_STOP) {
						pMedia->m_pMediaControl->Pause();
						pMedia->EndTimer();
						pMedia->m_Mode=MEDIA_MODE_STOP;
						pMedia->SendNotify(MEDIA_NOTIFY_POS,
														pMedia->GetPosition());
						pMedia->SendNotify(MEDIA_NOTIFY_MODE,
													(LPARAM)MEDIA_MODE_STOP);
					}
					break;
				case EC_STEP_COMPLETE:
					pMedia->SendNotify(MEDIA_NOTIFY_POS,pMedia->GetPosition());
					break;
#if 0
				/*
				case EC_PAUSED:
					pMedia->m_Mode=MEDIA_MODE_PAUSE;
					pMedia->SendNotify(MEDIA_NOTIFY_MODE,
													(LPARAM)MEDIA_MODE_PAUSE);
					break;
				*/
				case EC_STATE_CHANGE:
					switch (Param1) {
					case State_Stopped:	pMedia->m_Mode=MEDIA_MODE_STOP;	break;
					case State_Paused:	pMedia->m_Mode=MEDIA_MODE_PAUSE;	break;
					case State_Running:	pMedia->m_Mode=MEDIA_MODE_PLAY;	break;
					}
					pMedia->SendNotify(MEDIA_NOTIFY_MODE,
													(LPARAM)pMedia->m_Mode);
					break;
#endif
				/*
				case EC_STATE_CHANGE:
					switch (Param1) {
					case State_Running:	pMedia->BeginTimer();	break;
					default:			pMedia->EndTimer();		break;
					}
					break;
				*/
				}
				pMedia->m_pMediaEvent->FreeEventParams(evCode,Param1,Param2);
			}
		}
		return NOERROR;

	case WM_DESTROY:
		{
			DirectShowMedia *pMedia=MediaPtr(hwnd);

			pMedia->Close();
			pMedia->m_hwnd=NULL;
			SetWindowLongPtr(hwnd,0,(LONG_PTR)(LPVOID)NULL);
		}
		return 0;
	}
	return DefWindowProc(hwnd,uMsg,wParam,lParam);
}


bool DirectShowMedia::BeginTimer()
{
	if ((m_TimerID&POS_TIMER_ID)==0)
		m_TimerID|=SetTimer(m_hwnd,POS_TIMER_ID,POS_TIMER_INTERVAL,NULL);
	return true;
}


void DirectShowMedia::EndTimer()
{
	if ((m_TimerID&POS_TIMER_ID)!=0) {
		KillTimer(m_hwnd,POS_TIMER_ID);
		m_TimerID&=~POS_TIMER_ID;
	}
}


DirectShowMedia::DirectShowMedia()
{
	m_pGraphBuilder=NULL;
	m_pMediaControl=NULL;
	m_pMediaEvent=NULL;
	m_pMediaSeeking=NULL;
	m_pVideoWindow=NULL;
	m_TimerID=0;
}


DirectShowMedia::~DirectShowMedia()
{
	Destroy();
}


bool DirectShowMedia::Create(HWND hwndParent,int ID,HINSTANCE hinst)
{
	if (m_hwnd!=NULL)
		return true;
	m_hwnd=CreateWindowEx(0,DIRECTSHOW_MEDIA_FORM_WINDOW_CLASS,NULL,
		WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN,0,0,0,0,hwndParent,(HMENU)ID,
																hinst,this);
	return m_hwnd!=NULL;
}


void DirectShowMedia::Destroy()
{
	if (m_hwnd!=NULL)
		DestroyWindow(m_hwnd);
}


bool DirectShowMedia::Open(LPCTSTR pszFileName)
{
	HRESULT hr;
	bool fHasVideo,fHasAudio;

	if (m_fOpened)
		Close();
	//CoInitialize(NULL);
	if (FAILED(CoCreateInstance(CLSID_FilterGraph,NULL,CLSCTX_INPROC_SERVER,
								IID_IGraphBuilder,(void**)&m_pGraphBuilder))) {
		CoUninitialize();
		return false;
	}
	m_pGraphBuilder->QueryInterface(IID_IMediaControl,(void**)&m_pMediaControl);
	m_pGraphBuilder->QueryInterface(IID_IMediaEventEx,(void**)&m_pMediaEvent);
	m_pGraphBuilder->QueryInterface(IID_IMediaSeeking,(void**)&m_pMediaSeeking);
	m_pGraphBuilder->QueryInterface(IID_IVideoWindow,(void**)&m_pVideoWindow);
	m_pMediaEvent->SetNotifyWindow((OAHWND)m_hwnd,WM_MEDIA_EVENT,NULL);
#ifndef UNICODE
	WCHAR szFileName[MAX_PATH];

	MultiByteToWideChar(CP_ACP,0,pszFileName,-1,szFileName,MAX_PATH);
	hr=m_pGraphBuilder->RenderFile(szFileName,NULL);
#else
	hr=m_pGraphBuilder->RenderFile(pszFileName,NULL);
#endif
	if (FAILED(hr)) {
		int Message;

		Close();
		switch (hr) {
		case VFW_E_CANNOT_CONNECT:
		case VFW_E_CANNOT_LOAD_SOURCE_FILTER:
		case VFW_E_CANNOT_RENDER:
		case VFW_E_UNKNOWN_FILE_TYPE:
		case VFW_E_UNSUPPORTED_STREAM:
			Message=IDS_ERR_UNSUPPORTED;
			break;
		case VFW_E_INVALID_FILE_FORMAT:
			Message=IDS_ERR_INVALIDFORMAT;
			break;
		default:
			Message=IDS_ERR_OTHER;
		}
		ResMessageBox(GetMainWindow(),Message,0,MB_OK | MB_ICONEXCLAMATION);
		return false;
	}
	m_fOpened=true;
	m_Mode=MEDIA_MODE_READY;
	CheckMediaType(&fHasVideo,&fHasAudio);
	m_pVideoWindow->put_Owner((OAHWND)m_hwnd);
	m_pVideoWindow->put_WindowStyle(WS_CHILD);
	m_pVideoWindow->put_MessageDrain((OAHWND)m_hwnd);
	//m_pVideoWindow->put_AutoShow(OAFALSE);
	if (!fHasVideo)
		m_pVideoWindow->put_Visible(FALSE);
	m_pMediaSeeking->SetTimeFormat(&TIME_FORMAT_MEDIA_TIME);
	m_MediaType=fHasVideo?MEDIA_VIDEO:MEDIA_SOUND;
	if (fHasAudio)
		SetMediaVolume(m_fMute?0:m_Volume);
	long evCode;
	m_pMediaEvent->WaitForCompletion(INFINITE,&evCode);
	if (m_MediaType==MEDIA_VIDEO) {
		SIZE sz;

		GetSourceSize(&sz);
		//m_pVideoWindow->put_Left(0);
		//m_pVideoWindow->put_Top(0);
		//SetWindowPos(m_hwnd,NULL,0,0,sz.cx,sz.cy,SWP_NOMOVE | SWP_NOZORDER);
		SetSize(sz.cx,sz.cy);
		// ŏ̃t[\
		LONGLONG Start,End;
		m_pMediaSeeking->GetAvailable(&Start,&End);
		m_pMediaSeeking->SetPositions(&Start,AM_SEEKING_AbsolutePositioning,
										&Start,AM_SEEKING_AbsolutePositioning);
		m_pMediaControl->Run();
		m_pMediaEvent->WaitForCompletion(INFINITE,&evCode);
		m_pMediaControl->Pause();
		m_pMediaSeeking->SetPositions(&Start,AM_SEEKING_AbsolutePositioning,
										&End,AM_SEEKING_AbsolutePositioning);
	}
	return true;
}


void DirectShowMedia::Close()
{
	if (m_pGraphBuilder!=NULL) {
		m_pMediaControl->Stop();
		EndTimer();
		RELEASE(m_pVideoWindow);
		RELEASE(m_pMediaSeeking);
		RELEASE(m_pMediaEvent);
		RELEASE(m_pMediaControl);
		RELEASE(m_pGraphBuilder);
		//CoUninitialize();
		m_fOpened=false;
		m_MediaType=MEDIA_UNDEFINED;
		m_Mode=MEDIA_MODE_UNDEFINED;
	}
}


bool DirectShowMedia::Run()
{
	if (FAILED(m_pMediaControl->Run()))
		return false;
	m_Mode=MEDIA_MODE_PLAY;
	SendNotify(MEDIA_NOTIFY_MODE,(LPARAM)MEDIA_MODE_PLAY);
	BeginTimer();
	return true;
}


bool DirectShowMedia::Play()
{
	if (!m_fOpened)
		return false;
	SetRate(1000);
	return Run();
}


bool DirectShowMedia::Pause()
{
	if (!m_fOpened)
		return false;
	if (m_Mode==MEDIA_MODE_PAUSE) {
		Play();
		//m_Mode=MEDIA_MODE_PLAY;
	} else if (m_Mode==MEDIA_MODE_PLAY) {
		long evCode;

		m_pMediaControl->Pause();
		m_pMediaEvent->WaitForCompletion(INFINITE,&evCode);
		m_Mode=MEDIA_MODE_PAUSE;
		EndTimer();
	} else
		return false;
	SendNotify(MEDIA_NOTIFY_POS,GetPosition());
	SendNotify(MEDIA_NOTIFY_MODE,(LPARAM)m_Mode);
	return true;
}


bool DirectShowMedia::Stop()
{
	if (!m_fOpened)
		return false;
	if (m_Mode!=MEDIA_MODE_STOP) {
		long evCode;

		if (FAILED(m_pMediaControl->Pause()))
			return false;
		m_pMediaEvent->WaitForCompletion(INFINITE,&evCode);
		m_Mode=MEDIA_MODE_STOP;
		EndTimer();
		Home();
		SendNotify(MEDIA_NOTIFY_MODE,(LPARAM)MEDIA_MODE_STOP);
	}
	return true;
}


MediaPosition DirectShowMedia::GetTimePerFrame()
{
	IBasicVideo *pBasicVideo;
	REFTIME rt;
	HRESULT hr;

	if (m_MediaType!=MEDIA_VIDEO)
		return 0;
	m_pGraphBuilder->QueryInterface(IID_IBasicVideo,(void**)&pBasicVideo);
	hr=pBasicVideo->get_AvgTimePerFrame(&rt);
	pBasicVideo->Release();
	if (FAILED(hr))
		return 0;
	return (MediaPosition)(rt*1000.0+0.5);
}


bool DirectShowMedia::PrevFrame()
{
	if (m_MediaType==MEDIA_VIDEO) {
		MediaPosition Pos,Start,TimePerFrame;

		Pos=GetPosition();
		Start=GetStart();
		if (Pos>Start) {
			TimePerFrame=GetTimePerFrame();
			if (TimePerFrame==0)
				return false;
			Pos-=(TimePerFrame*2-1);
			if (Pos<Start)
				Pos=Start;
			SetPosition(Pos);
		}
	}
	return true;
}


bool DirectShowMedia::NextFrame()
{
	if (m_MediaType==MEDIA_VIDEO) {
		MediaPosition Pos,End;

		Pos=GetPosition();
		End=GetEnd();
		if (Pos<End) {
			IVideoFrameStep *pFrameStep;

			if (SUCCEEDED(m_pGraphBuilder->QueryInterface(IID_IVideoFrameStep,
													(void**)&pFrameStep))) {
				if (pFrameStep->CanStep(1,NULL)==S_OK)
					pFrameStep->Step(1,NULL);
				pFrameStep->Release();
			} else {
				MediaPosition TimePerFrame=GetTimePerFrame();

				if (TimePerFrame==0)
					return false;
				Pos+=(TimePerFrame*2-1);
				if (Pos>End)
					Pos=End;
				SetPosition(Pos);
			}
		}
	}
	return true;
}


bool DirectShowMedia::Backward()
{
	if (!m_fOpened || GetPosition()<=GetStart())
		return false;
	if (!SetRate(-m_BackwardSpeed))
		return false;
	return Run();
}


bool DirectShowMedia::Forward()
{
	if (!m_fOpened || GetPosition()>=GetEnd())
		return false;
	if (!SetRate(m_ForwardSpeed))
		return false;
	return Run();
}


bool DirectShowMedia::SetBackwardSpeed(int Speed)
{
	if (Speed<1)
		return false;
	m_BackwardSpeed=Speed;
	return true;
}


bool DirectShowMedia::SetForwardSpeed(int Speed)
{
	if (Speed<1)
		return false;
	m_ForwardSpeed=Speed;
	return true;
}


bool DirectShowMedia::Home()
{
	SetPosition(GetStart());
	return true;
}


bool DirectShowMedia::End()
{
	SetPosition(GetEnd());
	return true;
}


#if 0
bool DirectShowMedia::SetZoom(int Zoom)
{
	SIZE sz;
	//IBasicVideo *pBasicVideo;

	if (m_MediaType!=MEDIA_VIDEO)
		return false;
	if (Zoom<1)
		Zoom=1;
	m_Zoom=Zoom;
	GetSourceSize(&sz);
	sz.cx=sz.cx*Zoom/100;
	if (sz.cx==0)
		sz.cx=1;
	sz.cy=sz.cy*Zoom/100;
	if (sz.cy==0)
		sz.cy=1;
	m_pVideoWindow->put_Width(sz.cx);
	m_pVideoWindow->put_Height(sz.cy);
	return true;
}
#endif


bool DirectShowMedia::SetMediaVolume(unsigned int Volume)
{
	IBasicAudio *pBasicAudio;
	HRESULT hr;
	int Vol;

	if (FAILED(m_pGraphBuilder->QueryInterface(IID_IBasicAudio,
														(void**)&pBasicAudio)))
		return false;
	if (Volume==0)
		Vol=-10000;
	else
		Vol=(int)(2000.0*log10((double)Volume/1000.0));
	hr=pBasicAudio->put_Volume(Vol);
	pBasicAudio->Release();
	return SUCCEEDED(hr);
}


bool DirectShowMedia::SetVolume(unsigned int Volume)
{
	if (Volume>1000)
		return false;
	if (m_fMute)
		m_fMute=false;
	if (m_fOpened)
		SetMediaVolume(Volume);
	m_Volume=Volume;
	return true;
}


bool DirectShowMedia::Mute(bool fMute)
{
	if (m_fMute==fMute)
		return true;
	if (m_fOpened)
		SetMediaVolume(m_fMute?0:m_Volume);
	m_fMute=fMute;
	return true;
}


bool DirectShowMedia::GetSourceSize(LPSIZE psz)
{
	IBasicVideo *pBasicVideo;

	if (m_MediaType!=MEDIA_VIDEO)
		return false;
	m_pGraphBuilder->QueryInterface(IID_IBasicVideo,(void**)&pBasicVideo);
	pBasicVideo->get_SourceWidth(&psz->cx);
	pBasicVideo->get_SourceHeight(&psz->cy);
	pBasicVideo->Release();
	return true;
}


bool DirectShowMedia::GetDestSize(LPSIZE psz)
{
	IBasicVideo *pBasicVideo;

	if (m_MediaType!=MEDIA_VIDEO)
		return false;
	m_pGraphBuilder->QueryInterface(IID_IBasicVideo,(void**)&pBasicVideo);
	pBasicVideo->get_DestinationWidth(&psz->cx);
	pBasicVideo->get_DestinationHeight(&psz->cy);
	pBasicVideo->Release();
	return true;
}


MediaPosition DirectShowMedia::GetLength()
{
	LONGLONG Earliest,Latest;

	if (!m_fOpened)
		return 0;
	m_pMediaSeeking->GetAvailable(&Earliest,&Latest);
	return (MediaPosition)((Latest-Earliest)/10000);
}


MediaPosition DirectShowMedia::GetStart()
{
	LONGLONG Earliest,Latest;

	if (!m_fOpened)
		return -1;
	m_pMediaSeeking->GetAvailable(&Earliest,&Latest);
	return (MediaPosition)(Earliest/10000);
}


MediaPosition DirectShowMedia::GetEnd()
{
	LONGLONG Earliest,Latest;

	if (!m_fOpened)
		return -1;
	m_pMediaSeeking->GetAvailable(&Earliest,&Latest);
	return (MediaPosition)(Latest/10000);
}


MediaPosition DirectShowMedia::GetPosition()
{
	LONGLONG Pos;

	if (!m_fOpened)
		return -1;
	m_pMediaSeeking->GetCurrentPosition(&Pos);
	return (MediaPosition)(Pos/10000);
}


bool DirectShowMedia::SetPosition(MediaPosition Position)
{
	LONGLONG Pos=(LONGLONG)Position*10000;

	if (FAILED(m_pMediaSeeking->SetPositions(
										&Pos,AM_SEEKING_AbsolutePositioning,
										NULL,AM_SEEKING_NoPositioning)))
		return false;
	SendNotify(MEDIA_NOTIFY_POS,Position);
	return true;
}


bool DirectShowMedia::CheckMediaType(bool *pfHasVideo,bool *pfHasAudio)
{
	if (!m_fOpened)
		return false;
	bool fHasVideo,fHasAudio;
	IEnumFilters *pEnumFilters;
	IBaseFilter *pFilter;
	ULONG cFetched;

	if (FAILED(m_pGraphBuilder->EnumFilters(&pEnumFilters)))
		return false;
	fHasVideo=false;
	fHasAudio=false;
	while (pEnumFilters->Next(1,&pFilter,&cFetched)==S_OK) {
		IEnumPins *pEnumPins;

		if (SUCCEEDED(pFilter->EnumPins(&pEnumPins))) {
			IPin *pPin;

			while (pEnumPins->Next(1,&pPin,NULL)==S_OK) {
				PIN_DIRECTION Dir;

				pPin->QueryDirection(&Dir);
				if (Dir==PINDIR_OUTPUT) {
					IEnumMediaTypes *pEnumMediaTypes;
					AM_MEDIA_TYPE *pamt;

					if (SUCCEEDED(pPin->EnumMediaTypes(&pEnumMediaTypes))) {
						while (pEnumMediaTypes->Next(1,&pamt,NULL)==S_OK) {
							if (pamt->majortype==MEDIATYPE_Video)
								fHasVideo=true;
							if (pamt->majortype==MEDIATYPE_Audio)
								fHasAudio=true;
							if (pamt->cbFormat!=0)
								CoTaskMemFree(pamt->pbFormat);
							RELEASE(pamt->pUnk);
							CoTaskMemFree(pamt);
							if (fHasVideo && fHasAudio)
								break;
						}
						pEnumMediaTypes->Release();
					}
					if (fHasVideo && fHasAudio)
						break;
				}
				pPin->Release();
			}
			pEnumPins->Release();
			if (fHasVideo && fHasAudio)
				break;
		}
		pFilter->Release();
		if (fHasVideo && fHasAudio)
			break;
	}
	pEnumFilters->Release();
	if (pfHasVideo!=NULL)
		*pfHasVideo=fHasVideo;
	if (pfHasAudio!=NULL)
		*pfHasAudio=fHasAudio;
	return true;
}


bool DirectShowMedia::HasVideo()
{
	bool fHasVideo;

	if (!CheckMediaType(&fHasVideo,NULL))
		return false;
	return fHasVideo;
}


bool DirectShowMedia::HasAudio()
{
	bool fHasAudio;

	if (!CheckMediaType(NULL,&fHasAudio))
		return false;
	return fHasAudio;
}


void DirectShowMedia::Move(int x,int y,int Width,int Height)
{
	MoveWindow(m_hwnd,x,y,Width,Height,TRUE);
}


void DirectShowMedia::SetSize(int Width,int Height)
{
	SetWindowPos(m_hwnd,NULL,0,0,Width,Height,SWP_NOZORDER | SWP_NOMOVE);
}
