/******************************************************************************
*                                                                             *
*    Playlist.cpp                           Copyright(c) 2005-2009 itow,y.    *
*                                                                             *
******************************************************************************/

/*
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#include <shlobj.h>
#include <tchar.h>
#include "LafPlayer.h"
#include "Playlist.h"
#include "Util.h"
#include "resource.h"


#define PLAYLIST_WINDOW_CLASS TEXT("LafPlayer Playlist Window")
#define TITLE_TEXT TEXT("LafPlayer Playlist")

#define IDC_PLAYLIST_LIST 400

#define TIMER_ID_UP		1
#define TIMER_ID_DOWN	2

#define PlaylistPtr(hwnd) ((Playlist*)GetWindowLongPtr(hwnd,0))




/*
	vCXg̃ACe
*/
class PlaylistItem {
	TCHAR m_szFileName[MAX_PATH];
	HICON m_hico;
	bool m_fPlayed;
public:
	PlaylistItem(LPCTSTR pszFileName);
	~PlaylistItem();
	LPCTSTR GetFileName() const { return m_szFileName; }
	void GetFileName(LPTSTR pszFileName) const
	{
		lstrcpy(pszFileName,m_szFileName);
	}
	HICON GetIcon() const { return m_hico; }
	bool IsPlayed() const { return m_fPlayed; }
	void SetPlayed() { m_fPlayed=true; }
};


PlaylistItem::PlaylistItem(LPCTSTR pszFileName)
{
	SHFILEINFO shfi;

	lstrcpy(m_szFileName,pszFileName);
	if (SHGetFileInfo(pszFileName,0,&shfi,sizeof(SHFILEINFO),
					SHGFI_ICON | SHGFI_SMALLICON | SHGFI_USEFILEATTRIBUTES))
		m_hico=shfi.hIcon;
	else
		m_hico=NULL;
	m_fPlayed=false;
}


PlaylistItem::~PlaylistItem()
{
	if (m_hico!=NULL)
		DestroyIcon(m_hico);
}




/*
	\p̃t@C擾
*/
LPCTSTR Playlist::GetDisplayFileName(LPCTSTR pszFilePath,LPTSTR pszFileName)
{
	if (m_FileNameType==FILENAME_FILENAME)
		return GetFileNamePtr(pszFilePath);
	if (m_FileNameType==FILENAME_FULLPATH)
		return pszFilePath;
	return GetFilePathEllipsis(pszFileName,pszFilePath);
}


/*
	Xg̕vZ
*/
int Playlist::CalcListHExtent()
{
	int nCount,nMaxWidth;

	nMaxWidth=0;
	nCount=ListBox_GetCount(m_hwndList);
	if (nCount>=1) {
		HDC hdc;
		HFONT hfontOld;
		int i;
		PlaylistItem *pItem;
		LPCTSTR pszFileName;
		TCHAR szTemp[MAX_PATH];
		SIZE sz;

		hdc=GetDC(m_hwndList);
		if (hdc==NULL)
			return 0;
		hfontOld=SelectFont(hdc,m_hfontPlaying);
		for (i=0;i<nCount;i++) {
			pItem=(PlaylistItem*)ListBox_GetItemData(m_hwndList,i);
			pszFileName=GetDisplayFileName(pItem->GetFileName(),szTemp);
			GetTextExtentPoint32(hdc,pszFileName,lstrlen(pszFileName),&sz);
			if (sz.cx>nMaxWidth)
				nMaxWidth=sz.cx;
		}
		SelectFont(hdc,hfontOld);
		ReleaseDC(m_hwndList,hdc);
		nMaxWidth+=3+GetSystemMetrics(SM_CXSMICON);
	}
	return nMaxWidth;
}


/*
	eLXg̕vZ
*/
int Playlist::CalcFileNameHExtent(LPCTSTR pszFileName)
{
	HDC hdc;
	HFONT hfontOld;
	LPCTSTR pszName;
	TCHAR szTemp[MAX_PATH];
	SIZE sz;

	hdc=GetDC(m_hwndList);
	if (hdc==NULL)
		return 0;
	hfontOld=SelectFont(hdc,m_hfontPlaying);
	pszName=GetDisplayFileName(pszFileName,szTemp);
	GetTextExtentPoint32(hdc,pszName,lstrlen(pszName),&sz);
	SelectFont(hdc,hfontOld);
	ReleaseDC(m_hwndList,hdc);
	return sz.cx+3+GetSystemMetrics(SM_CXSMICON);
}


/*
	c[o[쐬
*/
HWND Playlist::CreateToolbar()
{
	static TBBUTTON tbb[] = {
		{0,	CM_PLAYLIST_OPEN,	TBSTATE_ENABLED,
								BTNS_BUTTON | BTNS_DROPDOWN,	{0,0},	0,0},
		{1,	CM_PLAYLIST_SAVE,	0,					BTNS_BUTTON,
																{0,0},	0,0},
		{2,	CM_PLAYLIST_CLEAR,	0,					BTNS_BUTTON,
																{0,0},	0,0},
		{0,	0,					TBSTATE_ENABLED,	BTNS_SEP,
																{0,0},	0,0},
		{3,	CM_PLAYLIST_ADD,	TBSTATE_ENABLED,	BTNS_BUTTON,
																{0,0},	0,0},
		{4,	CM_PLAYLIST_DELETE,	0,					BTNS_BUTTON,
																{0,0},	0,0},
		{0,	0,					TBSTATE_ENABLED,	BTNS_SEP,
																{0,0},	0,0},
		{5,	CM_PLAYLIST_UP,		0,					BTNS_BUTTON,
																{0,0},	0,0},
		{6,	CM_PLAYLIST_DOWN,	0,					BTNS_BUTTON,
																{0,0},	0,0},
		{7,	CM_PLAYLIST_TOP,	0,					BTNS_BUTTON,
																{0,0},	0,0},
		{8,	CM_PLAYLIST_BOTTOM,	0,					BTNS_BUTTON,
																{0,0},	0,0},
	};
	HWND hwnd;
	int IconWidth,IconHeight;

	if (m_himlToolbar!=NULL) {
		ImageList_GetIconSize(m_himlToolbar,&IconWidth,&IconHeight);
	} else {
		IconWidth=16;
		IconHeight=16;
	}
	hwnd=CreateToolbarEx(m_hwnd,WS_CHILD | WS_VISIBLE |
							TBSTYLE_TOOLTIPS | TBSTYLE_FLAT | CCS_NODIVIDER,0,
					0,NULL,0,NULL,0,0,0,IconWidth,IconHeight,sizeof(TBBUTTON));
	if (hwnd==NULL)
		return NULL;
	SendMessage(hwnd,TB_SETEXTENDEDSTYLE,0,TBSTYLE_EX_DRAWDDARROWS);
	SendMessage(hwnd,TB_SETIMAGELIST,0,(LPARAM)
				(m_himlToolbar!=NULL?m_himlToolbar:
			ImageList_LoadImage(GetResourceInstance(),
				MAKEINTRESOURCE(IDB_PLAYLIST),16,1,RGB(255,0,255),IMAGE_BITMAP,
														LR_CREATEDIBSECTION)));
	if (m_himlToolbarHot!=NULL)
		SendMessage(hwnd,TB_SETHOTIMAGELIST,0,(LPARAM)m_himlToolbarHot);
	if (m_himlToolbarDisabled!=NULL)
		SendMessage(hwnd,TB_SETDISABLEDIMAGELIST,0,(LPARAM)m_himlToolbarDisabled);
	SendMessage(hwnd,TB_ADDBUTTONS,numberof(tbb),(LPARAM)tbb);
	return hwnd;
}


/*
	c[o[̃Xe[^Xݒ肷
*/
void Playlist::SetToolbarStat()
{
	int nCount,nSel;

	nCount=ListBox_GetCount(m_hwndList);
	if (nCount>0)
		nSel=ListBox_GetCurSel(m_hwndList);
	else
		nSel=-1;
	SendMessage(m_hwndToolbar,TB_ENABLEBUTTON,CM_PLAYLIST_SAVE,nCount>0);
	SendMessage(m_hwndToolbar,TB_ENABLEBUTTON,CM_PLAYLIST_UP,nSel>0);
	SendMessage(m_hwndToolbar,TB_ENABLEBUTTON,CM_PLAYLIST_DOWN,
													nSel>=0 && nSel<nCount-1);
	SendMessage(m_hwndToolbar,TB_ENABLEBUTTON,CM_PLAYLIST_TOP,nSel>0);
	SendMessage(m_hwndToolbar,TB_ENABLEBUTTON,CM_PLAYLIST_BOTTOM,
													nSel>=0 && nSel<nCount-1);
	SendMessage(m_hwndToolbar,TB_ENABLEBUTTON,CM_PLAYLIST_DELETE,nSel>=0);
	SendMessage(m_hwndToolbar,TB_ENABLEBUTTON,CM_PLAYLIST_CLEAR,nCount>0);
}


/*
	Xg쐬
*/
BOOL Playlist::CreateList()
{
	m_hwndList=CreateWindowEx(WS_EX_CLIENTEDGE,TEXT("LISTBOX"),NULL,
		WS_CHILD | WS_VISIBLE | WS_VSCROLL | WS_HSCROLL |
						LBS_NOTIFY | LBS_NOINTEGRALHEIGHT | LBS_OWNERDRAWFIXED,
			0,0,0,0,m_hwnd,(HMENU)IDC_PLAYLIST_LIST,GetWindowInstance(m_hwnd),
																		NULL);
	if (m_hwndList==NULL)
		return FALSE;
	m_pfnOldListProc=SubclassWindow(m_hwndList,ListWndProc);
	// ACe̍Zo
	HDC hdc;
	HFONT hfontOld;
	TEXTMETRIC tm;
	int nIconHeight;

	hdc=GetDC(m_hwndList);
	hfontOld=SelectFont(hdc,m_hfont);
	GetTextMetrics(hdc,&tm);
	SelectFont(hdc,hfontOld);
	ReleaseDC(m_hwndList,hdc);
	m_nTextHeight=tm.tmHeight;
	nIconHeight=GetSystemMetrics(SM_CYSMICON);
	m_nItemHeight=max(m_nTextHeight,nIconHeight)+2;
	ListBox_SetItemHeight(m_hwndList,0,m_nItemHeight);
	return TRUE;
}


static int ListBox_GetHitItem(HWND hwndList,int /*x*/,int y)
{
	int Index;

	Index=ListBox_GetTopIndex(hwndList)+y/ListBox_GetItemHeight(hwndList,0);
	if (Index<0 || Index>=ListBox_GetCount(hwndList))
		return -1;
	return Index;
}


static void ListBox_MoveItem(HWND hwndList,int From,int To)
{
	int Top=ListBox_GetTopIndex(hwndList);
	LPARAM lData;

	lData=ListBox_GetItemData(hwndList,From);
	ListBox_DeleteString(hwndList,From);
	ListBox_InsertItemData(hwndList,To,lData);
	ListBox_SetCurSel(hwndList,To);
	ListBox_SetTopIndex(hwndList,Top);
}


static void ListBox_EnsureVisible(HWND hwndList,int Index)
{
	int Top;

	Top=ListBox_GetTopIndex(hwndList);
	if (Index<Top) {
		ListBox_SetTopIndex(hwndList,Index);
	} else if (Index>Top) {
		RECT rc;
		int Rows;

		GetClientRect(hwndList,&rc);
		Rows=rc.bottom/ListBox_GetItemHeight(hwndList,0);
		if (Rows==0)
			Rows=1;
		if (Index>=Top+Rows)
			ListBox_SetTopIndex(hwndList,Index-Rows+1);
	}
}


void Playlist::DrawInsertMark(int Pos)
{
	HDC hdc;
	RECT rc;

	hdc=GetDC(m_hwndList);
	GetClientRect(m_hwndList,&rc);
	rc.top=(Pos-ListBox_GetTopIndex(m_hwndList))*m_nItemHeight-1;
	PatBlt(hdc,0,rc.top,rc.right-rc.left,2,DSTINVERT);
	ReleaseDC(m_hwndList,hdc);
}


LRESULT CALLBACK Playlist::ListWndProc(HWND hwnd,UINT uMsg,WPARAM wParam,
																LPARAM lParam)
{
	Playlist *pPlaylist=PlaylistPtr(GetParent(hwnd));

	switch (uMsg) {
	case WM_LBUTTONDOWN:
		{
			int Sel;

			SetFocus(hwnd);
			Sel=ListBox_GetHitItem(hwnd,LOWORD(lParam),HIWORD(lParam));
			if (Sel>=0) {
				if (pPlaylist->GetSel()!=Sel) {
					pPlaylist->SetSel(Sel);
					pPlaylist->SendNotify(NOTIFY_SELCHANGE);
				}
				SetCapture(hwnd);
			}
		}
		return 0;

	case WM_LBUTTONUP:
		if (GetCapture()==hwnd) {
			ReleaseCapture();
			if (pPlaylist->m_DragTimerID!=0) {
				KillTimer(hwnd,pPlaylist->m_DragTimerID);
				pPlaylist->m_DragTimerID=0;
			}
			if (pPlaylist->m_DropInsertPos>=0) {
				int From=ListBox_GetCurSel(hwnd),To;

				pPlaylist->DrawInsertMark(pPlaylist->m_DropInsertPos);
				To=pPlaylist->m_DropInsertPos;
				if (To>From)
					To--;
				SetWindowRedraw(hwnd,FALSE);
				ListBox_MoveItem(hwnd,From,To);
				pPlaylist->SetToolbarStat();
				if (pPlaylist->m_nPlayingFile==From)
					pPlaylist->m_nPlayingFile=To;
				else if (pPlaylist->m_nPlayingFile>From
											&& pPlaylist->m_nPlayingFile<=To)
						pPlaylist->m_nPlayingFile--;
				else if (pPlaylist->m_nPlayingFile>=To
											&& pPlaylist->m_nPlayingFile<From)
					pPlaylist->m_nPlayingFile++;
				SetWindowRedraw(hwnd,TRUE);
				pPlaylist->m_DropInsertPos=-1;
			}
		}
		return 0;

	case WM_MOUSEMOVE:
		if (GetCapture()==hwnd) {
			int y=(SHORT)HIWORD(lParam);
			RECT rc;

			GetClientRect(hwnd,&rc);
			if (y>=0 && y<rc.bottom) {
				int Insert,Count,Sel;

				if (pPlaylist->m_DragTimerID!=0) {
					KillTimer(hwnd,pPlaylist->m_DragTimerID);
					pPlaylist->m_DragTimerID=0;
				}
				Insert=ListBox_GetTopIndex(hwnd)+
					(y+pPlaylist->m_nItemHeight/2)/pPlaylist->m_nItemHeight;
				Count=ListBox_GetCount(hwnd);
				if (Insert>Count) {
					Insert=Count;
				} else {
					Sel=ListBox_GetCurSel(hwnd);
					if (Insert==Sel || Insert==Sel+1)
						Insert=-1;
				}
				if (pPlaylist->m_DropInsertPos>=0)
					pPlaylist->DrawInsertMark(pPlaylist->m_DropInsertPos);
				pPlaylist->m_DropInsertPos=Insert;
				if (pPlaylist->m_DropInsertPos>=0)
					pPlaylist->DrawInsertMark(pPlaylist->m_DropInsertPos);
				SetCursor(LoadCursor(NULL,IDC_ARROW));
			} else {
				UINT TimerID;

				if (pPlaylist->m_DropInsertPos>=0) {
					pPlaylist->DrawInsertMark(pPlaylist->m_DropInsertPos);
					pPlaylist->m_DropInsertPos=-1;
				}
				if (y<0)
					TimerID=TIMER_ID_UP;
				else
					TimerID=TIMER_ID_DOWN;
				if (TimerID!=pPlaylist->m_DragTimerID) {
					if (pPlaylist->m_DragTimerID!=0)
						KillTimer(hwnd,pPlaylist->m_DragTimerID);
					pPlaylist->m_DragTimerID=SetTimer(hwnd,TimerID,100,NULL);
				}
				SetCursor(LoadCursor(NULL,IDC_NO));
			}
		}
		return 0;

	case WM_RBUTTONDOWN:
		if (GetCapture()==hwnd) {
			ReleaseCapture();
			if (pPlaylist->m_DragTimerID!=0) {
				KillTimer(hwnd,pPlaylist->m_DragTimerID);
				pPlaylist->m_DragTimerID=0;
			}
			if (pPlaylist->m_DropInsertPos>=0) {
				pPlaylist->DrawInsertMark(pPlaylist->m_DropInsertPos);
				pPlaylist->m_DropInsertPos=-1;
			}
		} else {
			POINT pt;
			int Index;

			SetFocus(hwnd);
			pt.x=LOWORD(lParam);
			pt.y=HIWORD(lParam);
			Index=ListBox_GetHitItem(hwnd,pt.x,pt.y);
			if (Index>=0) {
				int NumItems=ListBox_GetCount(hwnd);
				HMENU hmenu;

				pPlaylist->SetSel(Index);
				hmenu=LoadMenu(GetResourceInstance(),
												MAKEINTRESOURCE(IDM_PLAYLIST));
				EnableMenuItem(hmenu,CM_PLAYLIST_UP,MF_BYCOMMAND |
											(Index>0?MFS_ENABLED:MFS_GRAYED));
				EnableMenuItem(hmenu,CM_PLAYLIST_DOWN,MF_BYCOMMAND |
									(Index+1<NumItems?MFS_ENABLED:MFS_GRAYED));
				EnableMenuItem(hmenu,CM_PLAYLIST_TOP,MF_BYCOMMAND |
											(Index>0?MFS_ENABLED:MFS_GRAYED));
				EnableMenuItem(hmenu,CM_PLAYLIST_BOTTOM,MF_BYCOMMAND |
									(Index+1<NumItems?MFS_ENABLED:MFS_GRAYED));
				ClientToScreen(hwnd,&pt);
				TrackPopupMenu(GetSubMenu(hmenu,0),TPM_RIGHTBUTTON,pt.x,pt.y,0,
														GetParent(hwnd),NULL);
				DestroyMenu(hmenu);
			}
		}
		return 0;

	case WM_TIMER:
		{
			int Pos;

			Pos=ListBox_GetTopIndex(hwnd);
			if (wParam==TIMER_ID_UP) {
				if (Pos>0)
					Pos--;
			} else
				Pos++;
			ListBox_SetTopIndex(hwnd,Pos);
		}
		return 0;

	case WM_KEYDOWN:
		if (wParam==VK_RETURN || wParam==VK_DELETE) {
			SendMessage(GetParent(hwnd),WM_KEYDOWN,wParam,lParam);
			return 0;
		}
		break;
	}
	return CallWindowProc(pPlaylist->m_pfnOldListProc,hwnd,uMsg,wParam,lParam);
}


BOOL Playlist::OnCreate(HWND hwnd,LPCREATESTRUCT pcs)
{
	m_hwnd=hwnd;
	m_hwndToolbar=CreateToolbar();
	CreateList();
	// EBhETCY𒲐
	RECT rc,rcWindow;
	SendMessage(m_hwndToolbar,TB_AUTOSIZE,0,0);
	SendMessage(m_hwndToolbar,TB_GETITEMRECT,
				SendMessage(m_hwndToolbar,TB_BUTTONCOUNT,0,0)-1,(LPARAM)&rc);
	rc.left=rc.top=0;
	rc.bottom=240;
	AdjustWindowRectEx(&rc,pcs->style,FALSE,pcs->dwExStyle);
	GetWindowRect(m_hwnd,&rcWindow);
	MoveWindow(m_hwnd,rcWindow.left,rcWindow.top,
									rc.right-rc.left,rc.bottom-rc.top,FALSE);
	return TRUE;
}


void Playlist::OnSize(int nWidth,int nHeight)
{
	int nClientTop;
	RECT rc;

	SendMessage(m_hwndToolbar,TB_AUTOSIZE,0,0);
	GetWindowRect(m_hwndToolbar,&rc);
	nClientTop=rc.bottom-rc.top;
	MoveWindow(m_hwndList,0,nClientTop,nWidth,nHeight-nClientTop,TRUE);
}


void Playlist::OnDrawItem(LPDRAWITEMSTRUCT pdis)
{
	switch (pdis->itemAction) {
	case ODA_DRAWENTIRE:
	case ODA_SELECT:
		{
			PlaylistItem *pItem;
			bool fPlaying;
			COLORREF crText;
			HBRUSH hbr;
			int nOldBkMode;
			COLORREF crOldText;
			HFONT hfontOld;
			int nIconWidth,nIconHeight;
			LPCTSTR pszFileName;
			TCHAR szTemp[MAX_PATH];

			pItem=(PlaylistItem*)pdis->itemData;
			if (pItem==NULL)
				break;
			fPlaying=(int)pdis->itemID==m_nPlayingFile;
			if ((pdis->itemState&ODS_SELECTED)==0) {
				crText=fPlaying?m_crPlayingTextColor:
						pItem->IsPlayed()?m_crPlayedTextColor:m_crTextColor;
				hbr=m_hbrBackground;
			} else {
				crText=fPlaying?m_crPlayingSelTextColor:m_crSelTextColor;
				hbr=CreateSolidBrush(m_crSelBgColor);
			}
			FillRect(pdis->hDC,&pdis->rcItem,hbr);
			if (hbr!=m_hbrBackground)
				DeleteObject(hbr);
			nOldBkMode=SetBkMode(pdis->hDC,TRANSPARENT);
			crOldText=SetTextColor(pdis->hDC,crText);
			hfontOld=SelectFont(pdis->hDC,fPlaying?m_hfontPlaying:m_hfont);
			nIconWidth=GetSystemMetrics(SM_CXSMICON);
			if (pItem->GetIcon()!=NULL) {
				nIconHeight=GetSystemMetrics(SM_CYSMICON);
				DrawIconEx(pdis->hDC,pdis->rcItem.left+1,
								pdis->rcItem.top+(m_nItemHeight-nIconHeight)/2,
					pItem->GetIcon(),nIconWidth,nIconHeight,0,NULL,DI_NORMAL);
			}
			pszFileName=GetDisplayFileName(pItem->GetFileName(),szTemp);
			TextOut(pdis->hDC,pdis->rcItem.left+nIconWidth+2,
							pdis->rcItem.top+(m_nItemHeight-m_nTextHeight)/2,
											pszFileName,lstrlen(pszFileName));
			SetBkMode(pdis->hDC,nOldBkMode);
			SetTextColor(pdis->hDC,crOldText);
			SelectFont(pdis->hDC,hfontOld);
			if ((int)pdis->itemID==m_DropInsertPos
									|| (int)pdis->itemID+1==m_DropInsertPos)
				PatBlt(pdis->hDC,pdis->rcItem.left,
							(int)pdis->itemID==m_DropInsertPos?
										pdis->rcItem.top:pdis->rcItem.bottom-1,
							pdis->rcItem.right-pdis->rcItem.left,1,DSTINVERT);
			if ((pdis->itemState&ODS_FOCUS)==0)
				break;
		}
		// Fall through
	case ODA_FOCUS:
		DrawFocusRect(pdis->hDC,&pdis->rcItem);
		break;
	}
}


LRESULT Playlist::OnNotify(LPNMHDR pnmhdr)
{
	switch (pnmhdr->code) {
	case TTN_NEEDTEXT:
		{
			LPTOOLTIPTEXT pttt=(LPTOOLTIPTEXT)pnmhdr;

			pttt->lpszText=(LPTSTR)(IDS_SHORTCMBASE+pttt->hdr.idFrom);
			pttt->hinst=GetResourceInstance();
		}
		break;
	case TBN_DROPDOWN:
		if (GetHistoryCount()>0) {
			LPNMTOOLBAR pnmtb=(LPNMTOOLBAR)pnmhdr;
			RECT rc;
			POINT pt;

			SendMessage(pnmtb->hdr.hwndFrom,TB_GETITEMRECT,
				SendMessage(pnmtb->hdr.hwndFrom,TB_COMMANDTOINDEX,pnmtb->iItem,
															0),(LPARAM)&rc);
			pt.x=rc.left;
			pt.y=rc.bottom;
			ClientToScreen(pnmtb->hdr.hwndFrom,&pt);
			TrackPopupMenu(m_hmenuRecentFiles,TPM_LEFTBUTTON,pt.x,pt.y,0,
																m_hwnd,NULL);
		}
		break;
	}
	return 0;
}


void Playlist::OnDropFiles(HDROP hdrop)
{
	SetForegroundWindow(m_hwnd);
	AddDropFiles(hdrop);
	DragFinish(hdrop);
}


/*
void Playlist::OnFontChange()
{
	HDC hdc;
	LOGFONT lf;
	HFONT hfont,hfontOld;
	TEXTMETRIC tm;
	int nIconHeight;

	GetObject(GetStockObject(DEFAULT_GUI_FONT),sizeof(LOGFONT),&lf);
	if (m_hfontPlaying!=NULL)
		DeleteFont(m_hfontPlaying);
	lf.lfWeight=FW_BOLD;
	m_hfontPlaying=CreateFontIndirect(&lf);
	hdc=GetDC(m_hwndList);
	hfontOld=SelectFont(hdc,m_hfontPlaying);
	GetTextMetrics(hdc,&tm);
	SelectFont(hdc,hfontOld);
	ReleaseDC(m_hwndList,hdc);
	m_nTextHeight=tm.tmHeight;
	nIconHeight=GetSystemMetrics(SM_CYSMICON);
	m_nItemHeight=max(m_nTextHeight,nIconHeight)+2;
	ListBox_SetItemHeight(m_hwndList,0,m_nItemHeight);
}
*/


void Playlist::OnCommand(int nID,HWND /*hwndCtrl*/,UINT uNotifyCode)
{
	switch (nID) {
	case CM_PLAYLIST_CLEAR:
		Clear();
		break;
	case CM_PLAYLIST_DELETE:
		{
			int nSel=GetSel();

			if (nSel>=0)
				DeleteFile(nSel);
		}
		break;
	case CM_PLAYLIST_UP:
		{
			int nSel=GetSel();

			if (nSel>0) {
				//LPARAM lData;

				SetWindowRedraw(m_hwndList,FALSE);
				/*
				lData=ListBox_GetItemData(m_hwndList,nSel);
				ListBox_DeleteString(m_hwndList,nSel);
				ListBox_InsertItemData(m_hwndList,nSel-1,lData);
				ListBox_SetCurSel(m_hwndList,nSel-1);
				*/
				ListBox_MoveItem(m_hwndList,nSel,nSel-1);
				SetToolbarStat();
				if (m_nPlayingFile==nSel)
					m_nPlayingFile--;
				else if (m_nPlayingFile==nSel-1)
					m_nPlayingFile++;
				ListBox_EnsureVisible(m_hwndList,nSel-1);
				SetWindowRedraw(m_hwndList,TRUE);
			}
		}
		break;
	case CM_PLAYLIST_DOWN:
		{
			int nSel=GetSel();

			if (nSel<GetCount()-1) {
				//LPARAM lData;

				SetWindowRedraw(m_hwndList,FALSE);
				/*
				lData=ListBox_GetItemData(m_hwndList,nSel);
				ListBox_DeleteString(m_hwndList,nSel);
				ListBox_InsertItemData(m_hwndList,nSel+1,lData);
				ListBox_SetCurSel(m_hwndList,nSel+1);
				*/
				ListBox_MoveItem(m_hwndList,nSel,nSel+1);
				SetToolbarStat();
				if (m_nPlayingFile==nSel)
					m_nPlayingFile++;
				else if (m_nPlayingFile==nSel+1)
					m_nPlayingFile--;
				ListBox_EnsureVisible(m_hwndList,nSel+1);
				SetWindowRedraw(m_hwndList,TRUE);
			}
		}
		break;
	case CM_PLAYLIST_TOP:
		{
			int nSel=GetSel();

			if (nSel>0) {
				//LPARAM lData;

				SetWindowRedraw(m_hwndList,FALSE);
				/*
				lData=ListBox_GetItemData(m_hwndList,nSel);
				ListBox_DeleteString(m_hwndList,nSel);
				ListBox_InsertItemData(m_hwndList,0,lData);
				ListBox_SetCurSel(m_hwndList,0);
				*/
				ListBox_MoveItem(m_hwndList,nSel,0);
				SetToolbarStat();
				if (m_nPlayingFile==nSel)
					m_nPlayingFile=0;
				else if (m_nPlayingFile>=0)
					m_nPlayingFile++;
				ListBox_EnsureVisible(m_hwndList,0);
				SetWindowRedraw(m_hwndList,TRUE);
			}
		}
		break;
	case CM_PLAYLIST_BOTTOM:
		{
			int nSel=GetSel();

			if (nSel<GetCount()-1) {
				//LPARAM lData;
				int nIndex;

				SetWindowRedraw(m_hwndList,FALSE);
				/*
				lData=ListBox_GetItemData(m_hwndList,nSel);
				ListBox_DeleteString(m_hwndList,nSel);
				nIndex=ListBox_AddItemData(m_hwndList,lData);
				ListBox_SetCurSel(m_hwndList,nIndex);
				*/
				nIndex=GetCount()-1;
				ListBox_MoveItem(m_hwndList,nSel,nIndex);
				SetToolbarStat();
				if (m_nPlayingFile==nSel)
					m_nPlayingFile=nIndex;
				else if (m_nPlayingFile>nSel)
					m_nPlayingFile--;
				ListBox_EnsureVisible(m_hwndList,nIndex);
				SetWindowRedraw(m_hwndList,TRUE);
			}
		}
		break;
	case CM_PLAYLIST_PLAY:
		SendNotify(NOTIFY_PLAY);
		break;
	case IDC_PLAYLIST_LIST:
		switch (uNotifyCode) {
		case LBN_SELCHANGE:
			SetToolbarStat();
			SendNotify(NOTIFY_SELCHANGE);
			break;
		case LBN_DBLCLK:
			SendNotify(NOTIFY_PLAY);
			break;
		}
		break;
	default:
		if (nID>=CM_PLAYLISTHISTORY_FIRST && nID<=CM_PLAYLISTHISTORY_LAST) {
			// t@CJ
			TCHAR szFilePath[MAX_PATH];

			m_History.Get(nID-CM_PLAYLISTHISTORY_FIRST,szFilePath);
			if (FileExists(szFilePath)) {
				Load(szFilePath);
			} else {
				TCHAR szText[MAX_MESSAGE_LENGTH];
				TCHAR szMessage[MAX_MESSAGE_LENGTH+MAX_PATH];

				if (!LoadString(GetResourceInstance(),IDS_FILENOTFOUND,szText,
															numberof(szText)))
					break;
				wsprintf(szMessage,szText,szFilePath);
				if (MessageBox(GetMainWindow(),szMessage,NULL,
						MB_YESNO | MB_ICONEXCLAMATION | MB_DEFBUTTON2)==IDYES)
					m_History.Remove(nID-CM_PLAYLISTHISTORY_FIRST);
			}
			break;
		}
		FORWARD_WM_COMMAND(GetMainWindow(),nID,m_hwnd,0,SendMessage);
	}
}


void Playlist::OnClose()
{
	Show(FALSE);
	SendNotify(NOTIFY_CLOSE);
}


void Playlist::OnDestroy()
{
	SubclassWindow(m_hwndList,m_pfnOldListProc);
	SetWindowLongPtr(m_hwnd,0,(LONG_PTR)(LPVOID)NULL);
	m_hwnd=NULL;
	m_hwndToolbar=NULL;
	m_hwndList=NULL;
}


LRESULT CALLBACK Playlist::PlaylistWndProc(HWND hwnd,UINT uMsg,
												WPARAM wParam,LPARAM lParam)
{
	switch (uMsg) {
	case WM_CREATE:
		{
			Playlist *pPlaylist=
						(Playlist*)((LPCREATESTRUCT)lParam)->lpCreateParams;

			SetWindowLongPtr(hwnd,0,(LONG_PTR)pPlaylist);
			return pPlaylist->OnCreate(hwnd,(LPCREATESTRUCT)lParam)?0:-1;
		}
	case WM_SIZE:
		PlaylistPtr(hwnd)->OnSize(LOWORD(lParam),HIWORD(lParam));
		return 0;
	case WM_DRAWITEM:
		PlaylistPtr(hwnd)->OnDrawItem((LPDRAWITEMSTRUCT)lParam);
		return 0;
	case WM_NOTIFY:
		return PlaylistPtr(hwnd)->OnNotify((LPNMHDR)lParam);
	case WM_DROPFILES:
		PlaylistPtr(hwnd)->OnDropFiles((HDROP)wParam);
		return 0;
/*
	case WM_FONTCHANGE:
		PlaylistPtr(hwnd)->OnFontChange();
		return 0;
*/
	case WM_SETFOCUS:
		SetFocus(PlaylistPtr(hwnd)->m_hwndList);
		return 0;
	case WM_CTLCOLORLISTBOX:
		return (LRESULT)PlaylistPtr(hwnd)->m_hbrBackground;
	case WM_KEYDOWN:
		switch (wParam) {
		case VK_RETURN:
			PlaylistPtr(hwnd)->OnCommand(CM_PLAYLIST_PLAY,NULL,0);
			break;
		case VK_DELETE:
			PlaylistPtr(hwnd)->OnCommand(CM_PLAYLIST_DELETE,NULL,0);
			break;
		}
		return 0;
	case WM_COMMAND:
		PlaylistPtr(hwnd)->OnCommand(GET_WM_COMMAND_ID(wParam,lParam),
											GET_WM_COMMAND_HWND(wParam,lParam),
											GET_WM_COMMAND_CMD(wParam,lParam));
		return 0;
	case WM_CLOSE:
		PlaylistPtr(hwnd)->OnClose();
		return 0;
	case WM_DESTROY:
		PlaylistPtr(hwnd)->OnDestroy();
		return 0;
	}
	return DefWindowProc(hwnd,uMsg,wParam,lParam);
}


void Playlist::SendNotify(int Type)
{
	FORWARD_WM_COMMAND(GetMainWindow(),0,m_hwnd,Type,SendMessage);
}


#if 0


static inline TCHAR GetAccelChar(int nIndex)
{
	return nIndex<10?nIndex+'0':nIndex-10+'A';
}


/*
	t@CɃt@Cǉ
*/
void Playlist::AddHistory(LPCTSTR pszFileName)
{
	int nMenuItems,i;
	TCHAR szText[MAX_PATH+4];

	if (m_nMaxFileHistory<=0)
		return;
	nMenuItems=GetMenuItemCount(m_hmenuRecentFiles);
	for (i=0;i<nMenuItems;) {
		GetMenuString(m_hmenuRecentFiles,i,szText,numberof(szText),
																MF_BYPOSITION);
		if (lstrcmpi(pszFileName,szText+4)==0) {
			if (i==0) {
				// ɍŏɂ
				return;
			} else {
				DeleteMenu(m_hmenuRecentFiles,i,MF_BYPOSITION);
				nMenuItems--;
				continue;
			}
		}
		szText[1]=GetAccelChar(i+1);
		ModifyMenu(m_hmenuRecentFiles,i,MF_BYPOSITION | MFT_STRING,
										CM_PLAYLISTHISTORY_FIRST+i+1,szText);
		i++;
	}
	if (i>=m_nMaxFileHistory)
		DeleteMenu(m_hmenuRecentFiles,m_nMaxFileHistory-1,MF_BYPOSITION);
	wsprintf(szText,TEXT("&0: %s"),pszFileName);
	InsertMenu(m_hmenuRecentFiles,0,MF_BYPOSITION | MFT_STRING,
											CM_PLAYLISTHISTORY_FIRST,szText);
}


/*
	t@Ct@C폜
*/
void Playlist::DeleteHistory(int nNo)
{
	int nItems,i;
	TCHAR szText[MAX_PATH+4];

	nItems=GetMenuItemCount(m_hmenuRecentFiles);
	if (nNo<0 || nNo>=nItems)
		return;
	DeleteMenu(m_hmenuRecentFiles,nNo,MF_BYPOSITION);
	nItems--;
	if (nItems>0) {
		for (i=nNo;i<nItems;i++) {
			GetMenuString(m_hmenuRecentFiles,i,szText,numberof(szText),
															MF_BYPOSITION);
			szText[1]=GetAccelChar(i);
			ModifyMenu(m_hmenuRecentFiles,i,MF_BYPOSITION | MFT_STRING,
											CM_PLAYLISTHISTORY_FIRST+i,szText);
		}
	}
}


#endif


/*
	t@CNA
*/
void Playlist::ClearHistory()
{
#if 0
	int nMenuItems,i;

	nMenuItems=GetMenuItemCount(m_hmenuRecentFiles);
	if (nMenuItems==0)
		return;
	for (i=0;i<nMenuItems;i++)
		DeleteMenu(m_hmenuRecentFiles,0,MF_BYPOSITION);
#endif
	m_History.Clear();
}


/*
	t@C̍ő吔ύX
*/
void Playlist::SetMaxHistory(int Max)
{
#if 0
	if (Max==0) {
		ClearHistory();
	} else if (Max<m_nMaxFileHistory) {
		int nItems,i;

		nItems=GetMenuItemCount(m_hmenuRecentFiles);
		if (nItems>Max) {
			for (i=Max;i<nItems;i++)
				DeleteMenu(m_hmenuRecentFiles,i,MF_BYPOSITION);
		}
	}
#endif
	m_History.SetMax(Max);
	m_nMaxFileHistory=Max;
}


/*
	t@C̃t@C擾
*/
int Playlist::GetHistoryCount()
{
	return m_History.GetCount();
}


/*
	t@Cǂݍ
*/
bool Playlist::LoadHistory(LPTSTR pszFileName)
{
	return m_History.Load(pszFileName,TEXT("RecentPlaylist"),TEXT("File"));
}


/*
	t@Cۑ
*/
bool Playlist::SaveHistory(LPTSTR pszFileName)
{
	return m_History.Save(pszFileName,TEXT("RecentPlaylist"),TEXT("File"));
}


bool Playlist::Init(HINSTANCE hinst)
{
	WNDCLASS wc;

	wc.style=0;
	wc.lpfnWndProc=PlaylistWndProc;
	wc.cbClsExtra=0;
	wc.cbWndExtra=sizeof(Playlist*);
	wc.hInstance=hinst;
	wc.hIcon=NULL;
	wc.hCursor=LoadCursor(NULL,IDC_ARROW);
	wc.hbrBackground=(HBRUSH)(COLOR_3DFACE+1);
	wc.lpszMenuName=NULL;
	wc.lpszClassName=PLAYLIST_WINDOW_CLASS;
	return RegisterClass(&wc)!=0;
}


Playlist::Playlist()
{
	// ϐ̏
	m_hwnd=NULL;
	m_hwndToolbar=NULL;
	m_himlToolbar=NULL;
	m_himlToolbarHot=NULL;
	m_himlToolbarDisabled=NULL;
	m_hwndList=NULL;
	LOGFONT lf;
	GetObject(GetStockObject(DEFAULT_GUI_FONT),sizeof(LOGFONT),&lf);
	m_hfont=CreateFontIndirect(&lf);
	lf.lfWeight=FW_BOLD;
	m_hfontPlaying=CreateFontIndirect(&lf);
	m_fFindSubDirectory=FALSE;
	lstrcpy(m_szFileMasks,TEXT("*.*"));
	m_FileNameType=FILENAME_FILENAME;
	m_hmenuRecentFiles=CreatePopupMenu();
	m_nMaxFileHistory=10;
	m_History.SetMax(m_nMaxFileHistory);
	m_History.SetMenu(m_hmenuRecentFiles,CM_PLAYLISTHISTORY_FIRST);
	m_crTextColor=RGB(0,0,0);
	m_crBgColor=RGB(255,255,255);
	m_crSelTextColor=RGB(255,255,255);
	m_crSelBgColor=RGB(0,128,0);
	m_crPlayingTextColor=RGB(0,0,0);
	m_crPlayingSelTextColor=RGB(255,255,255);
	m_crPlayedTextColor=RGB(0,0,0);
	m_hbrBackground=CreateSolidBrush(m_crBgColor);
}


Playlist::~Playlist()
{
	if (m_hwnd!=NULL)
		DestroyWindow(m_hwnd);
	DeleteFont(m_hfont);
	DeleteFont(m_hfontPlaying);
	DestroyMenu(m_hmenuRecentFiles);
	DeleteObject(m_hbrBackground);
}


bool Playlist::Create(HWND hwndOwner,HINSTANCE hinst)
{
	m_nHorizontalExtent=0;
	m_nPlayingFile=-1;
	m_DropInsertPos=-1;
	m_DragTimerID=0;
	// EBhE̍쐬
	return CreateWindowEx(WS_EX_ACCEPTFILES | WS_EX_TOOLWINDOW,
		PLAYLIST_WINDOW_CLASS,TITLE_TEXT,WS_POPUP | WS_CAPTION |
								WS_SYSMENU | WS_THICKFRAME | WS_CLIPCHILDREN,
					CW_USEDEFAULT,CW_USEDEFAULT,CW_USEDEFAULT,CW_USEDEFAULT,
											hwndOwner,NULL,hinst,this)!=NULL;
}


/*
	\Ԃݒ肷
*/
void Playlist::Show(BOOL fShow)
{
	if (m_hwnd!=NULL)
		ShowWindow(m_hwnd,fShow?SW_SHOW:SW_HIDE);
}


/*
	ACe擾
*/
int Playlist::GetCount()
{
	return ListBox_GetCount(m_hwndList);
}


/*
	IĂACe擾
*/
int Playlist::GetSel()
{
	return ListBox_GetCurSel(m_hwndList);
}


/*
	ACeI
*/
BOOL Playlist::SetSel(int nIndex)
{
	if (ListBox_SetCurSel(m_hwndList,nIndex)==LB_ERR)
		return FALSE;
	SetToolbarStat();
	return TRUE;
}


/*
	Đ̃ACeݒ肷
*/
BOOL Playlist::SetPlaying(int nIndex)
{
	if (nIndex==m_nPlayingFile)
		return TRUE;
	if (nIndex<0) {
		m_nPlayingFile=-1;
	} else if (nIndex<ListBox_GetCount(m_hwndList)) {
		m_nPlayingFile=nIndex;
		((PlaylistItem*)ListBox_GetItemData(m_hwndList,nIndex))->SetPlayed();
	} else
		return FALSE;
	InvalidateRect(m_hwndList,NULL,TRUE);
	return TRUE;
}


/*
	t@C擾
*/
BOOL Playlist::GetFile(int nIndex,LPTSTR pszFileName)
{
	PlaylistItem *pItem=(PlaylistItem*)ListBox_GetItemData(m_hwndList,nIndex);

	if (pItem==NULL)
		return FALSE;
	pItem->GetFileName(pszFileName);
	return TRUE;
}


/*
	t@Cǉ
*/
BOOL Playlist::AddFile(LPCTSTR pszFileName)
{
	BOOL fResult;

	if (lstrcmpi(GetFileExtensionPtr(pszFileName),TEXT(".m3u"))==0)
		fResult=LoadM3UFile(pszFileName);
	else
		fResult=InsertFile(ListBox_GetCount(m_hwndList),pszFileName);
	return fResult;
}


/*
	̃t@Cǉ
*/
BOOL Playlist::AddFiles(const FileList *pFileList)
{
	int nNumFiles,i;
	TCHAR szFileName[MAX_PATH];

	nNumFiles=pFileList->GetCount();
	for (i=0;i<nNumFiles;i++) {
		pFileList->GetFile(i,szFileName);
		AddFile(szFileName);
	}
	return TRUE;
}


/*
	tH_t@Cǉ
*/
BOOL Playlist::AddDirectoryFiles(LPCTSTR pszDirectory)
{
	FileList Files;

	if (!Files.AddDirectoryFiles(pszDirectory,m_szFileMasks,
														m_fFindSubDirectory))
		return FALSE;
	return AddFiles(&Files);
}


/*
	t@CtH_̃t@Cǉ
*/
BOOL Playlist::AddFileOrDirectoryFiles(LPCTSTR pszPath)
{
	DWORD dwAttributes;
	BOOL fResult;

	dwAttributes=GetFileAttributes(pszPath);
	if (dwAttributes==0xFFFFFFFF)
		return FALSE;
	if ((dwAttributes&FILE_ATTRIBUTE_DIRECTORY)==0)
		fResult=AddFile(pszPath);
	else
		fResult=AddDirectoryFiles(pszPath);
	return fResult;
}


/*
	hbvꂽt@Cǉ
*/
BOOL Playlist::AddDropFiles(HDROP hdrop)
{
	int nNumFiles,i;
	TCHAR szFilePath[MAX_PATH];

	nNumFiles=DragQueryFile(hdrop,0xFFFFFFFF,NULL,0);
	for (i=0;i<nNumFiles;i++) {
		DragQueryFile(hdrop,i,szFilePath,MAX_PATH);
		if (!AddFileOrDirectoryFiles(szFilePath))
			return FALSE;
	}
	return TRUE;
}


/*
	t@C}
*/
BOOL Playlist::InsertFile(int nIndex,LPCTSTR pszFileName)
{
	PlaylistItem *pItem;
	int nExtent;

	if (nIndex<0)
		return FALSE;
	pItem=new PlaylistItem(pszFileName);
	if (ListBox_InsertItemData(m_hwndList,nIndex,pItem)==LB_ERR) {
		delete pItem;
		return FALSE;
	}
	SetToolbarStat();
	nExtent=CalcFileNameHExtent(pszFileName);
	if (nExtent>m_nHorizontalExtent) {
		m_nHorizontalExtent=nExtent;
		ListBox_SetHorizontalExtent(m_hwndList,nExtent);
	}
	if (m_nPlayingFile>=nIndex) {
		m_nPlayingFile++;
		InvalidateRect(m_hwndList,NULL,TRUE);
	}
	return TRUE;
}


/*
	t@C폜
*/
BOOL Playlist::DeleteFile(int nIndex)
{
	PlaylistItem *pItem;

	pItem=(PlaylistItem*)ListBox_GetItemData(m_hwndList,nIndex);
	if (ListBox_DeleteString(m_hwndList,nIndex)==LB_ERR)
		return FALSE;
	delete pItem;
	if (m_nPlayingFile==nIndex) {
		m_nPlayingFile=-1;
		InvalidateRect(m_hwndList,NULL,TRUE);
		SendNotify(NOTIFY_FILEDELETE);
	} else if (m_nPlayingFile>nIndex)
		m_nPlayingFile--;
	SetToolbarStat();
	m_nHorizontalExtent=CalcListHExtent();
	ListBox_SetHorizontalExtent(m_hwndList,m_nHorizontalExtent);
	return TRUE;
}


/*
	XgNA
*/
void Playlist::Clear()
{
	int Count,i;
	PlaylistItem *pItem;

	Count=GetCount();
	for (i=0;i<Count;i++) {
		pItem=(PlaylistItem*)ListBox_GetItemData(m_hwndList,i);
		delete pItem;
	}
	ListBox_ResetContent(m_hwndList);
	ListBox_SetHorizontalExtent(m_hwndList,0);
	m_nHorizontalExtent=0;
	if (m_nPlayingFile>=0) {
		m_nPlayingFile=-1;
		SendNotify(NOTIFY_FILEDELETE);
	}
	SetToolbarStat();
}


/*
	Xgt@C
*/
int Playlist::FindFile(LPCTSTR pszFileName)
{
	PlaylistItem *pItem;
	int Count,i;

	Count=GetCount();
	for (i=0;i<Count;i++) {
		pItem=(PlaylistItem*)ListBox_GetItemData(m_hwndList,i);
		if (lstrcmpi(pItem->GetFileName(),pszFileName)==0)
			return i;
	}
	return -1;
}


void Playlist::VisibleFile(int Index)
{
	ListBox_EnsureVisible(m_hwndList,Index);
}


HICON Playlist::GetFileIcon(int Index)
{
	return ((PlaylistItem*)ListBox_GetItemData(m_hwndList,Index))->GetIcon();
}


bool Playlist::GetDisplayFileName(int Index,LPTSTR pszFileName)
{
	TCHAR szFileName[MAX_PATH];

	if (Index<0 || Index>=GetCount())
		return false;
	lstrcpy(pszFileName,GetDisplayFileName(
		((PlaylistItem*)ListBox_GetItemData(m_hwndList,Index))->GetFileName(),
																szFileName));
	return true;
}


bool Playlist::IsFilePlayed(int Index)
{
	return ((PlaylistItem*)ListBox_GetItemData(m_hwndList,Index))->IsPlayed();
}


/*
	`bvwv̕\Ԃݒ肷
*/
void Playlist::ShowTipHelp(BOOL fShow)
{
	ShowToolbarTipHelp(m_hwndToolbar,fShow);
}


/*
	M3U`̃Xgǂݍ
*/
BOOL Playlist::LoadM3UFile(LPCTSTR pszFileName)
{
	HANDLE hFile;
	DWORD dwFileSize,dwRead;
	LPTSTR pszList,p;
	TCHAR szDirectory[MAX_PATH];

	hFile=CreateFile(pszFileName,GENERIC_READ,FILE_SHARE_READ,NULL,
									OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
	if (hFile==INVALID_HANDLE_VALUE)
		return FALSE;
	dwFileSize=GetFileSize(hFile,NULL);
	if (dwFileSize==0 || dwFileSize==0xFFFFFFFF) {
		CloseHandle(hFile);
		return FALSE;
	}
#ifndef UNICODE
	// ANSI
	pszList=new char[dwFileSize+1];
	if (!ReadFile(hFile,pszList,dwFileSize,&dwRead,NULL)
													|| dwRead!=dwFileSize) {
		delete pszList;
		CloseHandle(hFile);
		return FALSE;
	}
	pszList[dwFileSize]='\0';
#else	// ndef UNICODE
	// UNICODE֕ϊ
	LPSTR pszAnsiList;
	int nLength;

	pszAnsiList=new char[dwFileSize];
	if (!ReadFile(hFile,pszAnsiList,dwFileSize,&dwRead,NULL)
													|| dwRead!=dwFileSize) {
		delete pszAnsiList;
		CloseHandle(hFile);
		return FALSE;
	}
	nLength=MultiByteToWideChar(CP_ACP,0,pszAnsiList,dwFileSize,NULL,0);
	pszList=new WCHAR[nLength+1];
	MultiByteToWideChar(CP_ACP,0,pszAnsiList,dwFileSize,pszList,nLength+1);
	pszList[nLength]='\0';
	delete [] pszAnsiList;
#endif	// UNICODE
	CloseHandle(hFile);
	GetFileDirectory(szDirectory,pszFileName);
	p=pszList;
	while (*p!='\0') {
		if (*p=='\r' || *p=='\n') {
			p++;
		} else {
			int i;
			TCHAR szFileName[MAX_PATH];

			for (i=0;i<MAX_PATH && *p!='\0' && *p!='\r' && *p!='\n';i++)
				szFileName[i]=*p++;
			if (i<MAX_PATH) {
				szFileName[i]='\0';
				if ((IsCharAlpha(szFileName[0]) && szFileName[1]==':')
						|| (szFileName[0]=='\\' && szFileName[1]=='\\')) {
					AddFile(szFileName);
				} else {
					// ΃pX
					if (lstrlen(szDirectory)+1+i<MAX_PATH) {
						TCHAR szFilePath[MAX_PATH];

						GetAbsolutePath(szFilePath,szDirectory,szFileName);
						AddFile(szFilePath);
					}
				}
			} else {
				while (*p!='\0' && *p!='\r' && *p!='\n')
					p++;
			}
		}
	}
	delete [] pszList;
	m_History.Add(pszFileName);
	return TRUE;
}


/*
	t@C烊Xgǂݍ
*/
BOOL Playlist::Load(LPCTSTR pszFileName)
{
	Clear();
	return LoadM3UFile(pszFileName);
}


/*
	t@CփXgۑ
*/
BOOL Playlist::Save(LPCTSTR pszFileName)
{
	HANDLE hFile;
	int nCount,i;
	PlaylistItem *pItem;
	char szBuff[MAX_PATH+2];
	int nLength;
	DWORD dwWrite;

	hFile=CreateFile(pszFileName,GENERIC_WRITE,FILE_SHARE_READ,NULL,
									CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
	if (hFile==INVALID_HANDLE_VALUE)
		return FALSE;
	nCount=ListBox_GetCount(m_hwndList);
	for (i=0;i<nCount;i++) {
		pItem=(PlaylistItem*)ListBox_GetItemData(m_hwndList,i);
#ifndef UNICODE
		pItem->GetFileName(szBuff);
#else
		WCHAR szWideText[MAX_PATH];
		pItem->GetFileName(szWideText);
		WideCharToMultiByte(CP_ACP,0,szWideText,-1,szBuff,sizeof(szBuff),
																	NULL,NULL);
#endif
		lstrcatA(szBuff,"\r\n");
		nLength=lstrlenA(szBuff);
		if (!WriteFile(hFile,szBuff,nLength,&dwWrite,NULL)
												|| dwWrite!=(DWORD)nLength) {
			CloseHandle(hFile);
			return FALSE;
		}
	}
	return CloseHandle(hFile);
}


/*
	Ĩt@CXgǂݍ
*/
BOOL Playlist::LoadList()
{
	Settings *pSettings;
	int nFiles,i;
	TCHAR szName[8],szFileName[MAX_PATH];

	if (m_nMaxFileHistory==0)
		return TRUE;
	pSettings=OpenProgramSettings(TEXT("PlaylistFiles"),Settings::OPEN_READ);
	if (pSettings==NULL)
		return FALSE;
	if (!pSettings->ReadInt(TEXT("FileCount"),&nFiles) || nFiles<=0) {
		delete pSettings;
		return FALSE;
	}
	for (i=0;i<nFiles;i++) {
		wsprintf(szName,TEXT("File%d"),i);
		if (!pSettings->ReadString(szName,szFileName,numberof(szFileName))
														|| szFileName[0]=='\0')
			break;
		AddFile(szFileName);
	}
	delete pSettings;
	return TRUE;
}


/*
	t@CXgۑ
*/
BOOL Playlist::SaveList()
{
	Settings *pSettings;
	int nFiles,i;
	TCHAR szFileName[MAX_PATH],szName[8];

	pSettings=OpenProgramSettings(TEXT("PlaylistFiles"),Settings::OPEN_WRITE);
	if (pSettings==NULL)
		return FALSE;
	pSettings->Clear();
	nFiles=ListBox_GetCount(m_hwndList);
	pSettings->WriteInt(TEXT("FileCount"),nFiles);
	for (i=0;i<nFiles;i++) {
		PlaylistItem *pItem=(PlaylistItem*)ListBox_GetItemData(m_hwndList,i);

		pItem->GetFileName(szFileName);
		wsprintf(szName,TEXT("File%d"),i);
		if (!pSettings->WriteString(szName,szFileName))
			break;
	}
	delete pSettings;
	return TRUE;
}


BOOL Playlist::ClearSaveList()
{
	Settings *pSettings;

	pSettings=OpenProgramSettings(TEXT("PlaylistFiles"),Settings::OPEN_WRITE);
	if (pSettings==NULL)
		return FALSE;
	pSettings->Clear();
	pSettings->WriteInt(TEXT("FileCount"),0);
	delete pSettings;
	return TRUE;
}


/*
	TufBNg邩Ȃݒ肷
*/
void Playlist::FindSubDirectory(BOOL fFind)
{
	m_fFindSubDirectory=fFind;
}


/*
	t@C}XNݒ肷
*/
void Playlist::SetFileMasks(LPCTSTR pszMasks)
{
	lstrcpy(m_szFileMasks,pszMasks);
}


/*
	t@C̕\`ݒ肷
*/
void Playlist::SetFileNameType(FileNameType Type)
{
	if (Type!=m_FileNameType) {
		m_FileNameType=Type;
		if (ListBox_GetCount(m_hwndList)>0) {
			m_nHorizontalExtent=CalcListHExtent();
			ListBox_SetHorizontalExtent(m_hwndList,m_nHorizontalExtent);
			InvalidateRect(m_hwndList,NULL,TRUE);
		}
	}
}


void Playlist::SetColors(COLORREF crTextColor,COLORREF crBgColor,
					COLORREF crSelTextColor,COLORREF crSelBgColor,
					COLORREF crPlayingTextColor,COLORREF crPlayingSelTextColor,
					COLORREF crPlayedTextColor)
{
	m_crTextColor=crTextColor;
	m_crBgColor=crBgColor;
	m_crSelTextColor=crSelTextColor;
	m_crSelBgColor=crSelBgColor;
	m_crPlayingTextColor=crPlayingTextColor;
	m_crPlayingSelTextColor=crPlayingSelTextColor;
	m_crPlayedTextColor=crPlayedTextColor;
	DeleteObject(m_hbrBackground);
	m_hbrBackground=CreateSolidBrush(crBgColor);
	InvalidateRect(m_hwndList,NULL,TRUE);
}


void Playlist::SetTipColor(COLORREF crText,COLORREF crBack)
{
	HWND hwndToolTips;

	hwndToolTips=(HWND)SendMessage(m_hwndToolbar,TB_GETTOOLTIPS,0,0);
	SendMessage(hwndToolTips,TTM_SETTIPTEXTCOLOR,crText,0);
	SendMessage(hwndToolTips,TTM_SETTIPBKCOLOR,crBack,0);
}


bool Playlist::SetToolbarImage(HIMAGELIST himl,HIMAGELIST himlHot,
													HIMAGELIST himlDisabled)
{
	m_himlToolbar=himl;
	m_himlToolbarHot=himlHot;
	m_himlToolbarDisabled=himlDisabled;
	if (m_hwndToolbar!=NULL) {
		HWND hwnd;
		RECT rc;

		hwnd=CreateToolbar();
		if (hwnd==NULL)
			return false;
		DestroyWindow(m_hwndToolbar);
		m_hwndToolbar=hwnd;
		GetClientRect(m_hwnd,&rc);
		OnSize(rc.right,rc.bottom);
		SetToolbarStat();
	}
	return true;
}
