/******************************************************************************
*                                                                             *
*    StatusBar.cpp                          Copyright(c) 2007-2009 itow,y.    *
*                                                                             *
******************************************************************************/

/*
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <windows.h>
#include <windowsx.h>
#include <tchar.h>
#include "LafPlayer.h"
#include "StatusBar.h"


#define LP_STATUSBAR_CLASS TEXT("LafPlayer StatusBar")

#define MARGIN 2




LRESULT CALLBACK LPStatusBar::StatusBarWndProc(HWND hwnd,UINT uMsg,
												WPARAM wParam,LPARAM lParam)
{
	switch (uMsg) {
	case WM_CREATE:
		{
			LPStatusBar *pStatusBar=
						(LPStatusBar*)((LPCREATESTRUCT)lParam)->lpCreateParams;
			LOGFONT lf;

			GetObject(GetStockObject(DEFAULT_GUI_FONT),sizeof(LOGFONT),&lf);
			pStatusBar->m_hfont=CreateFontIndirect(&lf);
			pStatusBar->m_ClientHeight=abs(lf.lfHeight)+MARGIN*2;
			pStatusBar->m_Height=pStatusBar->m_ClientHeight+
												GetSystemMetrics(SM_CXEDGE)*2;
			SetWindowLongPtr(hwnd,0,(LONG_PTR)pStatusBar);
		}
		return 0;

	case WM_SIZE:
		{
			LPStatusBar *pStatusBar=GetStatusBar(hwnd);
			int Width=LOWORD(lParam),Size;
			int i;

			Size=0;
			for (i=1;i<NUM_PARTS;i++)
				Size+=pStatusBar->m_Parts[i].Size;
			pStatusBar->m_Parts[0].Size=
							max(Width-Size,pStatusBar->m_MinFirstPartWidth);
			for (i=1;i<NUM_PARTS;i++)
				pStatusBar->m_Parts[i].Offset=pStatusBar->m_Parts[i-1].Offset+
												pStatusBar->m_Parts[i-1].Size;
		}
		return 0;

	case WM_PAINT:
		{
			LPStatusBar *pStatusBar=GetStatusBar(hwnd);
			PAINTSTRUCT ps;
			HFONT hfontOld;
			COLORREF crOldText;
			int OldBkMode;
			int i;

			BeginPaint(hwnd,&ps);
			FillRect(ps.hdc,&ps.rcPaint,pStatusBar->m_hbrBackground);
			hfontOld=(HFONT)SelectObject(ps.hdc,pStatusBar->m_hfont);
			crOldText=::SetTextColor(ps.hdc,pStatusBar->m_crTextColor);
			OldBkMode=SetBkMode(ps.hdc,TRANSPARENT);
			for (i=0;i<NUM_PARTS;i++) {
				if (ps.rcPaint.right>pStatusBar->m_Parts[i].Offset
						&& ps.rcPaint.left<pStatusBar->m_Parts[i].Offset+
												pStatusBar->m_Parts[i].Size) {
					/*
					TextOut(ps.hdc,pStatusBar->m_Parts[i].Offset,MARGIN,
										pStatusBar->m_Parts[i].Text,
										lstrlen(pStatusBar->m_Parts[i].Text));
					*/
					RECT rc;

					rc.left=pStatusBar->m_Parts[i].Offset+MARGIN;
					rc.top=0;
					rc.right=rc.left+pStatusBar->m_Parts[i].Size-MARGIN;
					rc.bottom=pStatusBar->m_ClientHeight;
					DrawText(ps.hdc,pStatusBar->m_Parts[i].Text,-1,&rc,
										DT_LEFT | DT_SINGLELINE | DT_VCENTER);
				}
			}
			SelectObject(ps.hdc,hfontOld);
			::SetTextColor(ps.hdc,crOldText);
			SetBkMode(ps.hdc,OldBkMode);
			EndPaint(hwnd,&ps);
		}
		return 0;

	case WM_RBUTTONDOWN:
		FORWARD_WM_COMMAND(GetParent(hwnd),GetWindowID(hwnd),hwnd,
											NOTIFY_RBUTTONDOWN,SendMessage);
		return 0;

	case WM_DESTROY:
		{
			LPStatusBar *pStatusBar=GetStatusBar(hwnd);

			DeleteObject(pStatusBar->m_hfont);
			pStatusBar->m_hwnd=NULL;
			SetWindowLongPtr(hwnd,0,(LONG_PTR)(LPVOID)NULL);
		}
		return 0;
	}
	return DefWindowProc(hwnd,uMsg,wParam,lParam);
}


LPStatusBar *LPStatusBar::GetStatusBar(HWND hwnd)
{
	return (LPStatusBar*)GetWindowLongPtr(hwnd,0);
}


BOOL LPStatusBar::Init()
{
	WNDCLASS wc;

	wc.style=CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc=StatusBarWndProc;
	wc.cbClsExtra=0;
	wc.cbWndExtra=sizeof(LPStatusBar*);
	wc.hInstance=GetInstance();
	wc.hIcon=NULL;
	wc.hCursor=LoadCursor(NULL,IDC_ARROW);
	wc.hbrBackground=NULL;
	wc.lpszMenuName=NULL;
	wc.lpszClassName=LP_STATUSBAR_CLASS;
	return RegisterClass(&wc)!=0;
}


LPStatusBar::LPStatusBar(HWND hwndParent,int ID)
{
	int i;

	m_hbrBackground=CreateSolidBrush(RGB(0,0,0));
	m_crTextColor=RGB(224,224,224);
	m_Parts[0].Offset=0;
	for (i=0;i<NUM_PARTS;i++) {
		m_Parts[i].Size=0;
		m_Parts[i].Text[0]='\0';
	}
	m_MinFirstPartWidth=0;
	m_hwnd=CreateWindowEx(WS_EX_CLIENTEDGE,LP_STATUSBAR_CLASS,TEXT(""),
					WS_CHILD,0,0,0,0,hwndParent,(HMENU)ID,GetInstance(),this);
}


LPStatusBar::~LPStatusBar()
{
	if (m_hwnd!=NULL)
		DestroyWindow(m_hwnd);
	DeleteObject(m_hbrBackground);
}


void LPStatusBar::Show(BOOL fShow)
{
	ShowWindow(m_hwnd,fShow?SW_SHOW:SW_HIDE);
}


void LPStatusBar::SetText(int Part,LPCTSTR pText)
{
	RECT rc;

	lstrcpy(m_Parts[Part].Text,pText);
	rc.left=m_Parts[Part].Offset;
	rc.right=rc.left+m_Parts[Part].Size;
	rc.top=MARGIN;
	rc.bottom=m_ClientHeight-MARGIN;
	InvalidateRect(m_hwnd,&rc,TRUE);
	UpdateWindow(m_hwnd);
}


void LPStatusBar::SetSize(int Part,LPCTSTR pText)
{
	HDC hdc;
	HFONT hfontOld;
	SIZE sz;
	RECT rc;

	hdc=GetDC(m_hwnd);
	hfontOld=(HFONT)SelectObject(hdc,m_hfont);
	GetTextExtentPoint32(hdc,pText,lstrlen(pText),&sz);
	sz.cx+=MARGIN*2;
	if (Part>0)
		m_Parts[Part].Size=sz.cx;
	else
		m_MinFirstPartWidth=sz.cx;
	SelectObject(hdc,hfontOld);
	ReleaseDC(m_hwnd,hdc);
	GetClientRect(m_hwnd,&rc);
	SendMessage(m_hwnd,WM_SIZE,0,MAKELPARAM(rc.right,rc.bottom));
}


void LPStatusBar::SetBackgroundColor(COLORREF crColor)
{
	if (m_hbrBackground!=NULL)
		DeleteObject(m_hbrBackground);
	m_hbrBackground=CreateSolidBrush(crColor);
	if (m_hwnd!=NULL)
		InvalidateRect(m_hwnd,NULL,TRUE);
}


void LPStatusBar::SetTextColor(COLORREF crColor)
{
	m_crTextColor=crColor;
	if (m_hwnd!=NULL)
		InvalidateRect(m_hwnd,NULL,TRUE);
}
