/******************************************************************************
*                                                                             *
*    Volume.cpp                             Copyright(c) 2007-2009 itow,y.    *
*                                                                             *
******************************************************************************/

/*
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <windows.h>
#include "Volume.h"


#define VOLUME_SLIDER_CLASS TEXT("LafPlayer Volume Slider")

#define KNOB_WIDTH 3




LRESULT CALLBACK VolumeSlider::WndProc(HWND hwnd,UINT uMsg,
												WPARAM wParam,LPARAM lParam)
{
	switch (uMsg) {
	case WM_CREATE:
		{
			VolumeSlider *pSlider=
					(VolumeSlider*)((LPCREATESTRUCT)lParam)->lpCreateParams;

			SetWindowLongPtr(hwnd,0,(LONG_PTR)pSlider);
		}
		return 0;

	case WM_PAINT:
		{
			VolumeSlider *pSlider=GetPointer(hwnd);
			PAINTSTRUCT ps;
			RECT rc,rcKnob;

			BeginPaint(hwnd,&ps);
			GetClientRect(hwnd,&rc);
			rc.left+=3;
			rc.right-=3;
			DrawEdge(ps.hdc,&rc,BDR_SUNKENOUTER,BF_RECT | BF_ADJUST);
			rc.top=rc.bottom-1-KNOB_WIDTH/2-
					((rc.bottom-rc.top-KNOB_WIDTH)*pSlider->m_Volume+
													MAX_VOLUME/2)/MAX_VOLUME;
			rcKnob.top=rc.top-KNOB_WIDTH/2;
			rcKnob.left=rc.left-3;
			rcKnob.right=rc.right+3;
			rcKnob.bottom=rc.top+KNOB_WIDTH/2+1;
			DrawEdge(ps.hdc,&rcKnob,BDR_RAISEDINNER,BF_RECT | BF_MIDDLE);
			rc.top+=KNOB_WIDTH/2+1;
			if (rc.top<rc.bottom) {
				HBRUSH hbr;

				hbr=CreateSolidBrush(pSlider->m_VolumeColor);
				FillRect(ps.hdc,&rc,hbr);
				DeleteObject(hbr);
			}
			EndPaint(hwnd,&ps);
		}
		return 0;

	case WM_LBUTTONDOWN:
		{
			VolumeSlider *pSlider=GetPointer(hwnd);
			int Pos=HIWORD(lParam)-(1+KNOB_WIDTH/2),Max;
			RECT rc;

			GetClientRect(hwnd,&rc);
			Max=rc.bottom-(KNOB_WIDTH+2);
			if (Pos<0)
				Pos=0;
			else if (Pos>Max)
				Pos=Max;
			pSlider->m_Volume=MAX_VOLUME-(Pos*MAX_VOLUME+Max/2)/Max;
			InvalidateRect(hwnd,NULL,TRUE);
			pSlider->SendNotifyMessage(NOTIFY_VOLUMECHANGE);
			SetCapture(hwnd);
		}
		return 0;

	case WM_LBUTTONUP:
		if (GetCapture()==hwnd)
			ReleaseCapture();
		return 0;

	case WM_MOUSEMOVE:
		if (GetCapture()==hwnd) {
			VolumeSlider *pSlider=GetPointer(hwnd);
			RECT rc;
			int Volume,Max;

			GetClientRect(hwnd,&rc);
			Max=rc.bottom-(KNOB_WIDTH+2);
			Volume=MAX_VOLUME-(((SHORT)HIWORD(lParam)-(1+KNOB_WIDTH/2))*
														MAX_VOLUME+Max/2)/Max;
			if (Volume<0)
				Volume=0;
			else if (Volume>MAX_VOLUME)
				Volume=MAX_VOLUME;
			if (Volume!=pSlider->m_Volume) {
				pSlider->m_Volume=Volume;
				pSlider->SendNotifyMessage(NOTIFY_VOLUMECHANGE);
				InvalidateRect(hwnd,NULL,TRUE);
			}
		}
		return 0;

	case WM_DESTROY:
		{
			VolumeSlider *pSlider=GetPointer(hwnd);

			pSlider->m_hwnd=NULL;
		}
		return 0;
	}
	return DefWindowProc(hwnd,uMsg,wParam,lParam);
}


VolumeSlider* VolumeSlider::GetPointer(HWND hwnd)
{
	return (VolumeSlider*)GetWindowLongPtr(hwnd,0);
}


void VolumeSlider::SendNotifyMessage(int Type)
{
	SendMessage(GetParent(m_hwnd),WM_COMMAND,
				MAKEWPARAM(GetWindowLong(m_hwnd,GWL_ID),Type),(LPARAM)m_hwnd);
}


bool VolumeSlider::Init(HINSTANCE hinst)
{
	WNDCLASS wc;

	wc.style=CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc=WndProc;
	wc.cbClsExtra=0;
	wc.cbWndExtra=sizeof(VolumeSlider*);
	wc.hInstance=hinst;
	wc.hIcon=NULL;
	wc.hCursor=LoadCursor(NULL,IDC_ARROW);
	wc.hbrBackground=(HBRUSH)(COLOR_3DFACE+1);
	wc.lpszMenuName=NULL;
	wc.lpszClassName=VOLUME_SLIDER_CLASS;
	return RegisterClass(&wc)!=0;
}


VolumeSlider::VolumeSlider(HWND hwndParent,int ID,int Left,int Top,
										int Width,int Height,HINSTANCE hinst)
{
	m_hwnd=CreateWindowEx(0,VOLUME_SLIDER_CLASS,NULL,WS_CHILD | WS_VISIBLE,
						Left,Top,Width,Height,hwndParent,(HMENU)ID,hinst,this);
	m_Volume=MAX_VOLUME;
	m_VolumeColor=RGB(128,192,255);
}


VolumeSlider::~VolumeSlider()
{
	if (m_hwnd!=NULL)
		DestroyWindow(m_hwnd);
}


void VolumeSlider::SetVolume(int Volume)
{
	if (Volume!=m_Volume) {
		m_Volume=Volume;
		InvalidateRect(m_hwnd,NULL,TRUE);
	}
}


void VolumeSlider::SetColor(COLORREF Color)
{
	m_VolumeColor=Color;
	InvalidateRect(m_hwnd,NULL,TRUE);
}
